/**
 * Copyright (c) 2013 - 2015 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements autocomplete widget against form input element
 * @author Joe McClintock
 */

/**
* The luxsearch widget
* @module luxsearch
* @jqbase
* @requires jquery
* @requires underscore
* @requires jqueryui
*/
define(['ojs/ojcore', 'knockout', 'jquery', 'underscore', 'jqueryui-amd/widgets/autocomplete'],
    function(oj, ko, $, _, autocomplete) {
        'use strict';
        (function() {

            $.widget('GBU.luxautocomplete', autocomplete, {

                options: {
                    includeCategory: true
                },

                _create: function() {
                    this._super();
                    this.widget().menu('option', 'items', '> :not(.ui-autocomplete-category)');
                },

                _renderMenu: function(ul, items) {
                    var that = this,
                        currentCategory = '';

                    _.each(items, function(item) {
                        if (that.options.includeCategory && item.category !== currentCategory) {
                            ul.append($('<li/>', {'class': 'ui-autocomplete-category', text: item.category}));
                            currentCategory = item.category;
                        }
                        that._renderItemData(ul, item);
                    });
                },

                _renderItemData: function(ul, item) {
                    return this._renderItem(ul, item).data('ui-autocomplete-item', item);
                },

                _renderItem: function(ul, item) {
                    var image = '',
                        container,
                        link;

                    if (item.icon && item.icon !== '') {
                        image = $('<img/>', {src: item.icon});
                    }

                    // Note that it's important to use the text property etc rather than purely string concatenation since
                    // we should not trust that the content coming back from the server is sanitized/encoded properly against XSS
                    container = $('<li>');
                    link = $('<a/>', {title: item.desc}).append(image).appendTo(container);
                    $('<div/>', {'class': 'item-text', text: item.label}).appendTo(link);

                    return container.appendTo(ul);

                }
            });
        }());
    });