/**
 * @module luxDashboard
 * @jqbase
 * @requires jquery
 * @requires jqueryui
 */

/**
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * luxDashboard is a jQuery widget.
 *
 * To create the luxDashboard pattern, call luxDashboard({}) on any div element
 *
 */

var obj = /** @lends module:luxDashboard */{
/**
         * Options
         * <dl>

<dt>metaStore</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: The meta store defines the meta information about the tile, such as     <ul>
                                                                                                                        <li><b>tileId</b>: unique identifier for a tile</li>
                                                                                                                        <li><b>bluePrintId</b>: the bluePrint used for the stamping the tile</li>
                                                                                                                        <li><b>collapsedSizeClass</b>: css class that represents the tile content in collapsed state</li>
                                                                                                                        <li><b>expandedSizeClass</b>: css class that represents the tile content in expanded state</li>
                                                                                                                        <li><b>order</b>: positive natural number (1-based index) </li>
                                                                                                                        <li><b>inserted</b>: boolean value representing whether a tile is currently inserted in the Dashboard </li>
                                                                                                                        <li><b>flipped</b>: boolean value representing whether a tile is currently flipped in the Dashboard</li>
                                                                                                                        <li><b>expanded</b>: boolean value representing whether a tile is currently expanded in the Dashboard</li>
                                                                                                                </ul>
                    metaStore option allows you to choose between the default store (predefined RESTful store based on ojCollection) or use your custom store.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>custom store object or 'useDefault' string</b>:  If you pass in an object, it would be used as a custom store. If you pass in 'useDefault' string, the predefined
                            default store will be used instead.
                            <ul>
                                <li><i>default value</i> : 'useDefault'
<pre class="prettyprint" >
define([
    'ojs/ojcore',
    'knockout',
    'jquery',
    'luxhierarchy',
    './codeMirrorMixin',
    './customStore/customMetaStore',
    './customStore/metaStoreAdapterOverrides',
    'ojs/ojknockout',
    'ojs/ojbutton',
    'luxBindingBridge',
    'luxDashboard'
], function(oj, ko, $, LuxHierarchy, codeMirrorMixin, customMetaStore, metaStoreAdapterOverrides) {

    var VM_App_mainContent = function() {
        var self = this;

        self.metaStore = customMetaStore;
        self.metaStoreAdapterOverrides = metaStoreAdapterOverrides;

        self.ctrlStore = 'useDefault';
        self.defaultCtrlStoreURL = ko.observable('js/persistenceLayer/i18n/ctrlURL.js');

    };
    return VM_App_mainContent;
});
}</pre>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>ctrlStore</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: The ctrl store defines the control information about the tile, such as     <ul>
                                                                                                                        <li><b>tileId</b>: unique identifier for a tile</li>
                                                                                                                        <li><b>allow_insertRemove</b>: Defines whether a tile can be removed from the dashboard</li>
                                                                                                                        <li><b>allow_flip</b>:Defines whether a tile can be flipped</li>
                                                                                                                        <li><b>allow_reorder</b>:Defines whether a tile can be reordered</li>
                                                                                                                        <li><b>allow_resize</b>:Defines whether a tile can be expanded/collapsed</li>
                                                                                                                        <li><b>allow_flipExpand</b>:Defines whether a tile can be flipped and expanded at the same time</li>
                                                                                                                </ul>
                    ctrlStore is the client side persistence layer that defines the admin control exerted on the tile. This defines which actions can be performed on the tile by the user.
                    ctrlStore option allows you to choose between the default store (predefined RESTful store based on ojCollection) or use your custom store.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>custom store object or 'useDefault' string</b>:  If you pass in an object, it would be used as a custom store. If you pass in 'useDefault' string, the predefined
                            default store will be used instead.
                            <ul>
                                <li><i>default value</i> : 'useDefault'
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>defaultMetaStoreURL(optional)</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: When using default store for the meta information (i.e. when you define metaStore option as 'useDefault'), you need to define the RESTful server URL for
                    interacting with the meta store. The default store uses ojCollection/ojModel API.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>String representing the URL</b>: Provide a string representing the URL of the RESTful resource.
                            <ul>
                                <li><i>default value</i> : ''
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>defaultCtrlStoreURL(optional)</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: When using default store for the control information (i.e. when you define ctrlStore option as 'useDefault'), you need to define the RESTful server URL for
                    interacting with the ctrl store. The default store uses ojCollection/ojModel API.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>String representing the URL</b>: Provide a string representing the URL of the RESTful resource.
                            <ul>
                                <li><i>default value</i> : ''
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>metaWriteBack(optional)</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option decides whether changes made in UI (such as flipping a tile or removing a tile) updates the store or not. This is typically important during
                logging in and logging out process. The application typically would not want to persist changes made by a user who has not yet been logged-in, however as soon as the user logs in
                it is important to save the changes made by the user.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>Boolean value</b>
                            <ul>
                                <li><i>default value</i> : false (boolean)
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>ctrlWriteBack(optional)</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: Typically developers don't need to use this option since a separate 'luxDashboardAdmin' pattern is provided for this use case. This option should
                typically be only used for the administrator. After setting this option to true, hovering over the upper left corner of the tile
                shows a gear icon which opens a admin control dialog upon clicking.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>Boolean value</b>
                            <ul>
                                <li><i>default value</i> : false (boolean)
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>orderObserver(optional)</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for turning off persistence of reordering information. This option is only relevant when 'metaWriteBack' option is set to 'true', ignored otherwise.
                Use-case: Sometime developers might choose to turn-off persistence of ordering information for performance reasons.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>Boolean value</b>
                            <ul>
                                <li><i>default value</i> : true (boolean)
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>eventsChannel(optional)</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for performing custom actions based on custom events inside tiles. This options leverages the 'js-signals' library included in OJET distribution.</li>
                <li>Please find luxDashboard, 'Events Channel' demo for more details. </li>
                <li>The API can be leveraged with <a href="https://millermedeiros.github.io/js-signals/">js-signals</a> library that is included in <a href="http://jet.us.oracle.com/oj-components/technologyStack.html">OJET Technology Stack</a></li>
            </ul>
        </div>
    </div>
</dd>

<dt>metaStoreAdapterOverrides</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: metaStoreAdapterOverrides allows developers to override default adapter functionality or insert new functionality in the adapter. Developers would also need
                 to override adapter in case of implementing custom stores. Please refer to 'Overriding Adapter' demo and 'Custom store' demo for relevant use-cases.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>object with adapter methods</b>:
                            <ul>
                                <li><i>default value</i> : null
                                </li>
                                <li><i>Example</i> : This example shows how adapter overriding can be used for custom cleanup when a certain tile is removed from the dashboard.
<pre class="prettyprint" >
var metaStoreAdapterOverrides = {
    'update': function(tileId, key, newVal) {
        var self = this;
        var model = self.metaStore.get(tileId);
        if (key === 'inserted' && newVal === false) {
            console.log(tileId + ' removed custom cleanup after tile is removed');
        }
        model.set(key, newVal);
        model.save();
    }
};
</pre>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>ctrlStoreAdapterOverrides</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: ctrlStoreAdapterOverrides allows developers to override default adapter functionality or insert new functionality in the adapter. Developers would
                also need to override adapter in case of implementing custom stores. Please refer to 'Overriding Adapter' demo and 'Custom store' demo for relevant use-cases.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>object with adapter methods</b>:
                            <ul>
                                <li><i>default value</i> : null
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>defaultMetaStoreExtender</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for extending the default meta store.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>object with adapter methods</b>:
                            <ul>
                                <li><i>default value</i> : null
                                </li>
                                <li><i>Example</i> : This example demonstrates how the default store can be extended to parse incoming data.
                                                    In this example the default model (model used in the default store) is extended with parse method.
                                                    The data coming from server does not have collapsedSizeClass and expandedSizeClass properties on it.
                                                    The example demonstrates how this properties can be added into the store during parse of the incoming data from the server.
<pre class="prettyprint" >
define(['ojs/ojcore', 'knockout', 'jquery', 'ojs/ojmodel'], function(oj, ko, $) {

    var parseData = function(data) {
        return {
            "tileId": data.tileId,
            "bluePrintId": data.bluePrintId,
            "collapsedSizeClass": "oj-masonrylayout-tile-1x1",
            "expandedSizeClass": "oj-masonrylayout-tile-3x1",
            "order": data.order,
            "inserted": data.inserted,
            "flipped": data.flipped,
            "expanded": data.expanded
        }
    };

    var MetaModel = oj.Model.extend({
        idAttribute: 'tileId',
        parse: parseData
    });

    return MetaModel;
});
</pre>
Finally in the ViewModel the property is passed as Dashboard option.
<pre class="prettyprint" >
self.defaultMetaStoreExtender = {model: extendingDefaultStore_metaModel};
</pre>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>defaultCtrlStoreExtender</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for extending the default control store.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>object with adapter methods</b>:
                            <ul>
                                <li><i>default value</i> : null
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>bluePrintPathPrefix</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This defines the path of blue-prints used for the luxDashboard tiles. The path is relative to path defined for ojModules in your application.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>String representing the directory location prfix</b>: Provide a string representing the directory location.
                            <ul>
                                <li><i>default value</i> : 'luxDashboardTileBluePrints/'
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>translations</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for both ‘init’ time as well as run time ‘setter’ (overridden) or ‘getter’ of the i18n
                strings on the component (both batch or granular level, meaning set all strings at once or one at a time).
                LUX Dashboard uses internationalized strings with following priority:
                    <ol>
                        <li><b>translations option</b></li>
                        <li><b>i18n resource bundle</b></li>
                        <li><b>widget defaults</b></li>
                    </ol>
                </li>
                <li><i>Arguments</i>:
                    The translations option accepts object with following keys
                    <ul>
                        <li><b>RemoveTile</b></li>
                        <li><b>ConfigureTile</b></li>
                        <li><b>FlipBack</b></li>
                        <li><b>FlipForward</b></li>
                        <li><b>ReorderTitle</b></li>
                        <li><b>ReorderLabel</b></li>
                        <li><b>ResizeCollapse</b></li>
                        <li><b>ResizeExpand</b></li>
                        <li><b>tileDialogTitle</b></li>
                        <li><b>addDashboardTilesButtonLabel</b></li>
                        <li><b>dashboardTileLabel</b></li>
                    </ul>
                </li>
                <li><i>Example</i>:

                    <ul>
                        <li><b>Overriding strings during initializing the widget</b>:
                             Initialize the component, overriding some translated resources. One or more strings can be overridden. If you override one string then, the other translations remain intact.
<pre class="prettyprint" >
    $('#divId_searchCol').luxDashboard({
        metaStore: 'useDefault',
        defaultMetaStoreURL: 'js/persistenceLayer/i18n/admin-metaURL.json',
        metaWriteBack: false,
        orderObserver: true,

        ctrlStore: 'useDefault',
        defaultCtrlStoreURL: 'js/persistenceLayer/i18n/admin-ctrlURL.json',
        ctrlWriteBack: false,
        translations: {"RemoveTile": "RemoveTileAppDuringInit"}  // set one during init.
    });
</pre>
                        </li>

                        <li><b>Set the translations option, after initialization</b>:
                             Use standard jQueryUI widget factory 'option' method as follows:
<pre class="prettyprint" >
    $('#divId_searchCol').luxDashboard("option", "translations.RemoveTile", "RemoveTileAppUsingMethod");
</pre>
                        </li>

                        <li><b>Set the translations option, after initialization</b>:
                             Use standard jQueryUI widget factory 'option' method as follows:
<pre class="prettyprint" >
    var str = $('#divId_searchCol').luxDashboard("option", "translations.RemoveTile");
</pre>
                        </li>
                    </ul>

                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>titleDescriptionInMetaStore</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for alternate way of implementing internationalization in the luxDashboard. When using this option the 'title' and 'description'
                        strings must be defined in the metaStore. When locale is changed, the store must also be changed to reflect the translated versions
                        of the 'title' and 'description' strings for the new language. To update all the strings (generic and tile specific) call the
                        'refreshLocaleAndMetaStore' method on the luxDashboard by passing the new 'metaURL'. ('metaURL' is only required when using
                        'defaultMetaStoreURL' option. Not required for custom stores.)
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>Boolean value</b>
                            <ul>
                                <li><i>default value</i> : false (boolean)
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>dashboardOverrides</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: dashboardOverrides allows developers to alter default behavior of the dashboard by overriding default functionality in viewmodel.js
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>object with methods/properties to override</b>:
                            <ul>
                                <li><i>default value</i> : null
                                </li>
                                <li><i>Example</i> : This example shows overriding 'selectionListViewOptionChange' method and adding a new property into the viewmodel.js called 'newProperty'.
<pre class="prettyprint" >
var dashboardOverrides = {
    newProperty: 'newProp',
    'selectionListViewOptionChange': function(event, data) {
        //overriding default implementation of 'selectionListViewOptionChange' in the viewmodel.js
    }
};
</pre>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

* </dl>
*/
    options: {

    }
};

/**
 * The method can be used to update values of strings representing text in multiple languages from the nls resource bundle when locale is changed.
 *
 * @method module:luxDashboard#refreshLocale
 * @public
 */

/**
 * luxDashboard allows for swiftly replacing the data stores used. When using defaultMetaStore simply updating the value of
 * 'defaultMetaStoreURL' observable option is sufficient. However when using custom stores or overriding stores/adapters developers would need to call
 * this method to refresh the contents of the Dashboard to reflect the changed store. When updating both (meta and control) stores simultaneously,
 * developers instead should use 'reloadBothStores' option to avoid two refreshes on the screen.
 *
 * @method module:luxDashboard#reloadMetaStore
 * @param {string} defaultMetaStoreURL - (required only in case of using defaultMetaStore)
 * @public
 */

/**
 * When using 'titleDescriptionInMetaStore' option on the luxDashboard; 'title' and 'description' properties must be defined inside the metaStore.
 * However when the locale of the application is changed, the developers need to both refresh the generalized translated strings and tile specific
 * translated strings (title and description). This can be achieved together using 'refreshLocaleAndMetaStore' method
 *
 * @method module:luxDashboard#refreshLocaleAndMetaStore
 * @param {string} defaultMetaStoreURL - (required only in case of using defaultMetaStore)
 * @public
 */

/**
 * luxDashboard allows for swiftly replacing the data stores used. When using defaultCtrlStore simply updating the value of
 * 'defaultCtrlStoreURL' observable option is sufficient. However when using custom stores or overriding stores/adapters developers would need to call
 * this method to refresh the contents of the Dashboard to reflect the changed store. When updating both (meta and control) stores simultaneously,
 * developers instead should use 'reloadBothStores' option to avoid two refreshes on the screen.
 *
 * @method module:luxDashboard#reloadCtrlStore
 * @param {string} defaultCtrlStoreURL - (required only in case of using defaultCtrlStore)
 * @public
 */

/**
 * When updating both (meta and contorl) stores simultaneously,
 * developers should use 'reloadBothStores' option to avoid two refreshes on the screen.
 *
 * @method module:luxDashboard#reloadBothStores
 * @param {object} object
 * @param {array} object.metaStore - (required only in case of using custom stores)
 * @param {array} object.ctrlStore - (required only in case of using custom stores)
 * @param {array} object.defaultMetaStoreURL - (required only in case of using defaultMetaStore)
 * @param {array} object.metaStoreAdapterOverrides - (required only in case of overriding metaStoreAdapter)
 * @param {array} object.ctrlStoreAdapterOverrides - (required only in case of overriding ctrlStoreAdapter)
 * @public
 */

/**
 * insertTile method allows developers to programmatically insert a tile into the dashboard.
 * Note that a tile to be inserted must already be configured (have valid metaStore and ctrlStore information associated with it.)
 * and metaStore's inserted property must be false ("inserted": false). If the tile is already inserted, no action is taken.
 * Fires 'tileInserted' upon successful insertion of a tile.
 *
 * @method module:luxDashboard#insertTile
 * @param {object} object
 * @param {array} object.tileId - tileId of the tile to be inserted as defined in the metaStore and ctrlStore
 * @param {array} object.index - zero based index for the position where the tile would be inserted
 * @public
 */

/**
 * Returns an array of all the tiles currently present in the Dashboard (including removed tiles but not the once not loaded in the store)
 *
 * @method module:luxDashboard#getAllTileIds
 * @returns {array} array of tileId
 * @public
 */

/**
 * Returns default meta store (oj.Collection) constructor for use with 'defaultMetaStoreExtender'. This is useful if you are overriding a method in 'defaultMetaStoreExtender'
 * and wants to refer to the 'super' of the same method.
 *
 * @method module:luxDashboard#getDefaultMetaStore
 * @returns {object} parent store
 * @public
 */

/**
 * Returns default ctrl store (oj.Collection) constructor for use with 'defaultCtrlStoreExtender'. This is useful if you are overriding a method in 'defaultCtrlStoreExtender'
 * and wants to refer to the 'super' of the same method.
 *
 * @method module:luxDashboard#getDefaultCtrlStore
 * @returns {object} parent store
 * @public
 */

/**
 * Returns an array of all the inserted tiles.
 *
 * @method module:luxDashboard#getInsertedTileIds
 * @returns {array} array of tileId
 * @public
 */

/**
 * This event is triggered when promises are resolved asynchronously for both stores.
 *
 * @event module:luxDashboard#dataReady
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 */

/**
 * This event is triggered when a tile is removed form the luxDashboard.
 *
 * @event module:luxDashboard#tileRemoved
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 * @param {object} data.tileId tileId of the tile that is removed.
 */

/**
 * This event is triggered when a tile is tileInserted into the luxDashboard.
 *
 * @event module:luxDashboard#tileInserted
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 * @param {object} data.tileId tileId of the tile that is inserted.
 */

/**
 * This event is triggered when a tile is resized in luxDashboard
 *
 * @event module:luxDashboard#tileResized
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 * @param {object} data.tileId tileId of the tile that is resized.
 */

/**
 * This event is triggered when a tile is flipped in luxDashboard.
 *
 * @event module:luxDashboard#tileFlipped
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 * @param {object} data.tileId tileId of the tile that is flipped.
 * @param {object} data.flipDirection direction in which tile is flipped.
 */
