/*global define */
/*
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements a Dashboard pattern that can be used to organize and display contents in tiles.
 * @author Shailesh Zirpe
 */

define(['require',
    'ojs/ojcore',
    'jquery',
    'underscore',
    'knockout',
    './utils/preventChildrenBinding',
    'text!./view.tmpl.html',
    './viewmodel',
    'text!./adminCtrl.tmpl.html',
    'text!./contextMenu.tmpl.html',
    './defaultStore/metaCollection',
    './defaultStore/ctrlCollection',
    'jqueryui-amd/widget'
],

function(req, oj, $, _, ko, utils, template, VM, adminCtrl, contextMenu, MetaCollection, CtrlCollection) {

    'use strict';

    $.widget('GBU.luxDashboard',
        /** @lends module:luxbreadcrumbs */
        {
            version: '5.1.0',

            _create: function() {
                var widgetObj = this;

                var $tem = $(template);
                var preventChildrenBinding = $tem.get(0);
                var childOfPreventChildrenBinding = $tem.find('.childOfPreventChildrenBinding').get(0);

                // i18n related options // LUX-573 translations option
                var i18nStringsDefaults = {
                    'RemoveTile':                   'Remove',
                    'ConfigureTile':                'Configure Tile',
                    'FlipBack':                     'Flip Back',
                    'FlipForward':                  'Flip Forward',
                    'ReorderTitle':                 'Drag to reorder',
                    'ReorderLabel':                 'Drag to reorder, context menu available',
                    'ResizeCollapse':               'Collapse tile',
                    'ResizeExpand':                 'Expand tile',
                    'tileDialogTitle':              'Click on tiles to add them',
                    'addDashboardTilesButtonLabel': 'Add tiles to dashboard',
                    'dashboardTileLabel':           'tile'
                };
                var i18nStringsNLSBundleDefined = {};
                _.each(i18nStringsDefaults, function(value, key, list) {
                    var keyStr = 'luxDashboard' + '.' + key;
                    var valueFromNLSBundle = oj.Translations.getTranslatedString(keyStr);
                    if (valueFromNLSBundle !== keyStr) { // defined in NLS Bundle
                        i18nStringsNLSBundleDefined[key] = valueFromNLSBundle;
                    }
                });
                widgetObj.options.translations = $.extend({}, i18nStringsDefaults, i18nStringsNLSBundleDefined, widgetObj.options.translations); // object on the right has highest priority
                // end of i18n related options

                widgetObj.options.vm = new VM(widgetObj);
                $.extend(widgetObj.options.vm, widgetObj.options.dashboardOverrides); // a new option for extending/overriding viewmodel.js methods.

                //append the admin control dialog KO tempalte, the idea is to use same markup for mutliple dashboards adminCtrl
                if (!$(document.body).find('#adminCtrlLuxDashboard').length){ // add only one instance
                    $('body').append($(adminCtrl).get(0));
                }
                if (!$(document.body).find('#luxDashboardContextMenuTemplate').length){ // add only one instance
                    $('body').append($(contextMenu).get(0));
                }

                widgetObj.element.append(preventChildrenBinding);
                ko.applyBindings(widgetObj.options.vm, childOfPreventChildrenBinding);
            },

            options: {
                metaWriteBack:               false,
                orderObserver:               true,
                ctrlWriteBack:               false,
                metaStore:                   'useDefault',
                ctrlStore:                   'useDefault',
                defaultMetaStoreURL:         '',
                defaultCtrlStoreURL:         '',
                bluePrintPathPrefix:         'luxDashboardTileBluePrints/',
                eventsChannel:               'notDefined',
                titleDescriptionInMetaStore: false, //LUX-597
                translations:                {} // LUX-573
            },

            refreshLocale: function() {
                var widgetObj = this;
                widgetObj.options.vm.refreshLocale();
            },

            refreshLocaleAndMetaStore: function(metaURL) {
                var widgetObj = this;
                widgetObj.options.vm.refreshLocaleAndMetaStore(metaURL);
            },

            reloadMetaStore: function(metaURL) {
                var widgetObj = this;
                widgetObj.options.vm.reloadMetaStore(metaURL);
            },

            reloadCtrlStore: function(ctrlURL) {
                var widgetObj = this;
                widgetObj.options.vm.reloadCtrlStore(ctrlURL);
            },

            reloadBothStores: function(options) {
                var widgetObj = this;
                widgetObj.options.vm.reloadBothStores(options);
            },

            getAllTileIds: function() {
                var widgetObj = this;
                return widgetObj.options.vm.getAllTileIds();
            },

            getInsertedTileIds: function() {
                var widgetObj = this;
                return widgetObj.options.vm.getInsertedTileIds();
            },

            insertTile: function(options) {
                var widgetObj = this;
                widgetObj.options.vm.insertTile(options);
            },

            getDefaultMetaStore: function(options) {
                return MetaCollection;
            },

            getDefaultCtrlStore: function(options) {
                return CtrlCollection;
            },

            _setOption: function(key, value) {
                var widgetObj = this;
                var vm = widgetObj.options.vm;
                switch (key) {
                case 'metaWriteBack':
                    if (ko.isObservable(vm.metaWriteBack)) {
                        vm.metaWriteBack(value); //ko observable setter
                    }
                    break;
                case 'defaultMetaStoreURL':
                    if (ko.isObservable(vm.defaultMetaStoreURL)) {
                        vm.defaultMetaStoreURL(value); //ko observable setter
                    }
                    break;
                case 'orderObserver':
                    if (ko.isObservable(vm.orderObserver)) {
                        vm.orderObserver(value); //ko observable setter
                    }
                    break;
                case 'ctrlWriteBack':
                    if (ko.isObservable(vm.ctrlWriteBack)) {
                        vm.ctrlWriteBack(value); //ko observable setter
                    }
                    break;
                case 'defaultCtrlStoreURL':
                    if (ko.isObservable(vm.defaultCtrlStoreURL)) {
                        vm.defaultCtrlStoreURL(value); //ko observable setter
                    }
                    break;
                case 'translations':
                    widgetObj.options.translations = vm.setI18nStringsKOMapping(value);
                    break;
                default:
                    break;
                }
            }

        });

});
