/*global define */
/*
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements a Dashboard Admin widget that can be used in conjunction with luxDashboard pattern to enforce admin control on the Dashboard.
 * @author Shailesh Zirpe
 */
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'underscore',
    'luxhierarchy',
    'ojs/ojknockout',
    'ojs/ojknockout',
    'luxDashboard',
    'luxbreadcrumbs'
], function(oj, ko, $, _, LuxHierarchy) {

    var VM_dashboardAdmin = function(widgetObj) {
        var self = this;
        var myRootNode, dashNodes;

        var refreshDashboardOptions = function() {
            self.metaStore = 'useDefault';
            self.defaultMetaStoreURL = '';
            self.metaWriteBack = true;
            self.orderObserver = true;

            self.ctrlStore = 'useDefault';
            self.defaultCtrlStoreURL = '';
            self.ctrlWriteBack = true;

            self.metaStoreAdapterOverrides = {};
            self.ctrlStoreAdapterOverrides = {};

            self.defaultMetaStoreExtender = {};
            self.defaultCtrlStoreExtender = {};
        };

        self.widgetObj = widgetObj;
        self.dataReady = ko.observable(false);

        self.dashboards = widgetObj.options.dashboards; // doesn't need to be an observable!
        self.dashboardId = '';

        myRootNode = new LuxHierarchy().parseJSON('{"action": "Dashboards", "actionLabel": "Dashboards", "childrenLabel": "List Of dashboards"}');
        dashNodes = {};
        _.each(self.dashboards, function(db, index, list) {

            dashNodes[db.dashboardId] = myRootNode.addChild();
            dashNodes[db.dashboardId].action = db.dashboardId;
            dashNodes[db.dashboardId].actionLabel = db.label;
            dashNodes[db.dashboardId].childrenLabel = 'Roles';

            _.each(db.roles, function(role, index, list) {
                dashNodes[db.dashboardId][role.roleId] = dashNodes[db.dashboardId].addChild();
                dashNodes[db.dashboardId][role.roleId].action = role.roleId;
                dashNodes[db.dashboardId][role.roleId].actionLabel = role.roleLabel;
                dashNodes[db.dashboardId][role.roleId].roleSelected = 'roleSelected';
            });
        });

        self.luxBreadcrumbsOnSelect = function(event, data) {
            var dashboard, role;

            refreshDashboardOptions(); // do this every time

            // selection is dashboardId
            if (data.childrenLabel === 'Roles') {
                self.dataReady(false);
                self.dashboardId = data.action;
            }
            // selection is role
            if (data.roleSelected === 'roleSelected') {

                /* read the dashbaord admin confiuration for a role and set for the dashbaord options,
                 * then enable bindings using dataReady(true)
                 */
                dashboard = _.find(self.dashboards, function(dashboard) {
                    return dashboard.dashboardId === data.parent.action;
                });
                role = _.find(dashboard.roles, function(role) {
                    return role.roleId === data.action;
                });
                _.each(role, function(value, key, list) {
                    if (key != ('roleId' || 'roleLabel')) {
                        self[key] = role[key];
                    }
                });

                if (self.dataReady()) { // changing roles on preselected dashboard

                    $('#' + data.parent.action).luxDashboard('reloadBothStores', {
                        metaStore:                 self.metaStore,
                        ctrlStore:                 self.ctrlStore,
                        defaultMetaStoreURL:       self.defaultMetaStoreURL,
                        defaultCtrlStoreURL:       self.defaultCtrlStoreURL,
                        metaStoreAdapterOverrides: self.metaStoreAdapterOverrides,
                        ctrlStoreAdapterOverrides: self.ctrlStoreAdapterOverrides
                    }); // URL can be undefined in case of custome stores

                }

                self.dataReady(true);
                self.widgetObj.element.trigger('dataReady', {uuid: self.widgetObj.uuid});

            }
        };

        self.makeSelection = function(options){
            self.luxBreadcrumbsData(myRootNode.findNodeByAction(options));
            self.luxBreadcrumbsOnSelect('e', myRootNode.findNodeByAction(options));
        };

        self.luxBreadcrumbsData = ko.observable(myRootNode.findNodeByAction('Dashboards'));
        self.luxBreadcrumbsfixToTop = ko.observable(true);
        self.luxBreadcrumbsNavigateToCurrentNode = true;

    };

    return VM_dashboardAdmin;
});
