define([
    'knockout'
], function(ko) {
    'use strict';
    // Defines knockout form field observables in a javascript object.
    // The name of each observable is mapped to the name of the "observable"
    // attribute in the form definition JSON input for each field / widget.
    function formFields() {
        var self = {};

        function setViewModel(viewModel) {
            self.viewModel = viewModel;
        }

        function getViewModel() {
            return self.viewModel;
        }

        function getFields() {
            return self.fields;
        }

        function _createObservableArray(name, value) {
            var observableObj = {'name': name, 'value': ''},
                observableMap = value.map(function(item) {
                    return ko.observable(item);
                });

            observableObj.value = ko.observableArray(observableMap);

            self.fields.push(observableObj);
        }

        function _createObservable(name, value) {
            var observableObj = {'name': name, 'value': ko.observable()};

            if (value) {
                observableObj.value(value);
            }

            self.fields.push(observableObj);
        }

        function _write(name) {
            var observableObj = {'name': name, 'value': ko.observable()};

            self.fields.push(observableObj);
        }

        function _getObservable(name) {
            var observableOjb = ko.utils.arrayFirst(self.fields(), function(observable) {
                return observable.name === name;
            });

            if (observableOjb && observableOjb.value) {
                return observableOjb.value;
            }

            return null;
        }

        function _getFormFieldObject(name) {
            var observableOjb = ko.utils.arrayFirst(self.fields(), function(observable) {
                return observable.name === name;
            });

            if (observableOjb) {
                return observableOjb;
            }

            return null;
        }

        function save(name, value) {
            var observableOjb = ko.utils.arrayFirst(self.fields(), function(observable) {
                return observable.name === name;
            });

            if (observableOjb && observableOjb.value) {
                if (Array.isArray(observableOjb.value)) {
                    value.forEach(function(item, idx) {
                        observableOjb.value[idx](item);
                    });
                    return;
                }
                if (ko.isObservable(value)) {
                    observableOjb = value;
                    return;
                }

                observableOjb.value(value);
                return;
            }

            if (Array.isArray(value)) {
                _createObservableArray(name, value);
                return;
            }

            if (value) {
                _createObservable(name, value);
                return;
            }

            _write(name);
        }

        function getFieldObject(name) {
            if (name) {
                return _getFormFieldObject(name);
            }

            return null;
        }

        function read(name) {
            if (name) {
                return _getObservable(name);
            }

            return null;
        }

        self.id = Date.now();
        self.fields = ko.observableArray();

        self.save = save;
        self.read = read;
        self.getFields = getFields;
        self.getFieldObject = getFieldObject;

        self.viewModel = null;
        self.setViewModel = setViewModel;
        self.getViewModel = getViewModel;

        self.observablesCallbacks = {

            updateObservable: function updateFormField(event, data) {
                if (data.option === 'value') {
                    this(data.value);
                }
                event.preventDefault();
            },

            callBackClosure: function ObservabeToFieldBinding(observable) {
                // bind callback form field observable
                return self.observablesCallbacks.updateObservable.bind(self.read(observable));
            }
        };

        self.onChangeCallback = function onChangeCB(vm) {
            return self.observablesCallbacks.callBackClosure.bind(vm);
        };

        return self;
    }

    return formFields;
});