define([
    'gridapi.jquery',
    'gridapi.lodash'
], function ($, _) {
    'use strict';

    return {
        onBarEnter : function () {
            var that = this;
            return function (event) {
                var $bar = $(this);
                that.timeouts.barEnter = setTimeout(function () {
                    if (!that.$gantt.hasClass('dragging-viewport')) {
                        that.showBarTooltip({
                            event : event,
                            $bar : $bar,
                            bar : $bar.attr('data-bar'),
                            recordId : that.getElementDataId($bar)
                        });
                    }
                }, that.options.tooltips.delay);
            };
        },

        onBarExit : function () {
            var that = this;
            return function (event) {
                clearTimeout(that.timeouts.barEnter);
                that.timeouts.barExit = setTimeout(function () {
                    that.hideTooltip();
                }, 50);
            };
        },

        onBarTooltipEnter : function () {
            var that = this;
            return function (event) {
                clearTimeout(that.timeouts.barExit);
            };
        },

        onRowClick : function () {
            var that = this;
            return function (event) {
                if (!$(this).is('.bar-selected') && !that.$gantt.is('.dragging-viewport')) {
                    var recordId = that.getElementDataId($(this));
                    that.widget.grid.selectCell(recordId);
                    that.scrollFocusIntoView({ recordId : recordId, force : false });
                }
            };
        },

        onRowDoubleClick : function () {
            var that = this;
            return function (event) {
                if (!that.$gantt.is('.dragging-viewport')) {
                    var recordId = that.getElementDataId($(this));
                    that.scrollFocusIntoView({ recordId : recordId, force : true });
                }
            };
        },

        onSummaryBarDoubleClick : function () {
            var that = this;
            return function (event) {
                var recordId = that.getElementDataId($(this).parent());
                that.widget.grid.view.toggleChildren({
                    recordId : recordId,
                    rowsetId : that.ganttRowsetId
                });
                event.stopPropagation();
                event.preventDefault();
            };
        },

        showBarTooltip : function (args) {
            var that = this;
            this.hideTooltip();
            var barRenderer = that.barRenderers[args.bar];
            var tooltipData = that.widget.getBarTooltipData({ recordId : args.recordId, barConfig : barRenderer.config });

            this.$tooltip = $(barRenderer.makeTooltip({ tooltipData : tooltipData, recordId : args.recordId }));

            $('body').append(this.$tooltip);
            var top;
            var width = this.$tooltip.outerWidth();
            var offset = args.$bar.offset();
            var left = args.event.clientX - width / 2;
            if (left + width > $(window).width()) {
                left = $(window).width() - width - 10;
            }

            if (this.options.tooltips.constrain &&
                    (args.$bar.offset().top + args.$bar.height() + this.$tooltip.height() >
                    this.$chartContainer.height() + this.$chartContainer.offset().top)) {
                this.$tooltip.addClass('tooltip-above');
                top = offset.top - this.$tooltip.outerHeight() - 6;
            } else {
                this.$tooltip.addClass('tooltip-below');
                top = offset.top + args.$bar.height() + 6;
            }

            _.delay(function () {
                that.$tooltip.css('left', left);
                that.$tooltip.css('top', top);
                that.$tooltip.addClass('show-tooltip');
            }, 20);
        }

    };
});
