define([
    'gridapi.jquery',
    'gridapi.lodash',
    'ganttapi.gantt/locale',
    'gridapi.logger',
    'gridapi.jqueryui',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-layout',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-label-options',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-bar-options',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-theme-selector-value',
    'ganttapi.gantt/config/barConfigView',
    'ganttapi.gantt/enums/labelPosition',
    'ganttapi.gantt/enums/barColor',
    'ganttapi.gantt/enums/barProgressStyle',
    'ganttapi.gantt/enums/barThemes'
], function ($, _, locale, logger, jqui, layoutMarkup, labelOptionsMarkup, barOptionsMarkup, themeValue, BarConfigView, LABEL_POSITION, BAR_COLORS, BAR_PROGRESS_STYLE, BAR_THEMES) {
    'use strict';

    $.widget('pgbu.barConfigurator', {
        // Defaults
        options : {
            locale : locale.barConfigurator,
            config : {
                chart : {
                    showExpandedSummaryBar : true
                },
                layout : {}
            },
            dataFns : {
                getLabelFields : null,
                getStartFields : null,
                getFinishFields : null,
                getPercentFields : null,
                getBarTypes : null
            },
            defaultLabel : null
        },

        _create : function () {
            var that = this;

            this.widgetEventPrefix = 'gantt-config-';

            this.timeouts = {};
            if (this.options.config == null || this.options.config.bars == null) {
                this.options.config = _.cloneDeep(this.options.defaultConfig);
            }
            this.setupConfig();
            this.applyConfig(this.options.config);

            this.labelFields = this.options.dataFns.getLabelFields();
            this.flatLabelFields = _.flatten(_.pluck(this.labelFields, 'children'));
            this.barTypes = this.options.dataFns.getBarTypes();

            this.selection = {};
            this.view = new BarConfigView(this);
            this.view.init();
            this.selectBar(that.config.bars.order[0]);

            this.appliedTheme = BAR_THEMES.classic;
        },

        setupConfig : function () {
            this.config = {
                summaryBar : _.clone(this.options.config.summaryBar, true),
                barOrder : [],
                labels : {},
                layout : this.options.config.layout,
                bars : {
                    order : [],
                    definitions : {}
                },
                chart : this.options.config.chart
            };
        },

        applyDefaultConfig : function () {
            this.appliedTheme = BAR_THEMES.classic;
            this.config = {
                summaryBar : _.clone(this.options.defaultConfig.summaryBar, true),
                barOrder : [],
                labels : {},
                layout : this.options.config.layout,
                bars : {
                    order : [],
                    definitions : {}
                },
                chart : this.options.config.chart
            };
            this.applyConfig(_.clone(this.options.defaultConfig, true));
            this.view.resetConfig();
            this.selectBar(this.config.bars.order[0]);
            this._trigger('config-updated');
        },

        applyConfig : function (config) {
            var that = this;
            var labelIndex = 1;
            _.forEach(config.bars, function (barConfig, index) {
                var barId = 'bar' + index;
                that.config.bars.order.push(barId);
                that.config.bars.definitions[barId] = barConfig;
                if (barConfig.labels != null && barConfig.labels.length) {
                    _.forEach(barConfig.labels, function (labelConfig) {
                        var labelId = 'label' + labelIndex;
                        ++labelIndex;
                        that.config.labels[labelId] = labelConfig;
                        labelConfig.barId = barId;
                        labelConfig.id = labelId;
                    });
                }
            });

            this.config.bars.definitions.summary = config.summaryBar;
            if (config.summaryBar.labels != null && config.summaryBar.labels.length) {
                _.forEach(config.summaryBar.labels, function (labelConfig) {
                    var labelId = 'label' + labelIndex;
                    ++labelIndex;
                    that.config.labels[labelId] = labelConfig;
                    labelConfig.barId = 'summary';
                    labelConfig.id = labelId;
                });
            }
        },

        selectBar : function (barId) {
            this.selection = {
                id : barId,
                type : 'bar',
                config : this.config.bars.definitions[barId]
            };
            this.view.selectBar(barId);
        },

        selectLabel : function (labelId) {
            this.selection = {
                id : labelId,
                type : 'label',
                config : this.config.labels[labelId]
            };
            this.view.selectLabel(labelId);
        },

        applyTheme : function (theme) {
            this.appliedTheme = theme;
            this.appliedTheme.usedColors = [];
            for (var i = 0; i < this.config.bars.order.length; i++) {
                var barId = this.config.bars.order[i];
                var barConfig = this.config.bars.definitions[barId];
                if (barConfig.render.showProgress) {
                    barConfig.render.progressStyle = theme.defaultProgressStyle;
                    this.view.updateBarProgressStyle(barId);
                }
                this.resetBarColors({ barConfig : barConfig, barId : barId, skipResetUsedColors : true });
            }
            if (this.selection.type === 'bar') {
                this.selectBar(this.selection.id);
            }
            this.view.applyBarSettings(this.selection.id);
        },

        resetAppliedThemeColors : function () {
            this.appliedTheme.usedColors = [];
            for (var i = 0; i < this.config.bars.order.length; i++) {
                var barId = this.config.bars.order[i];
                if (barId !== 'summary') {
                    this.appliedTheme.usedColors.push(this.config.bars.definitions[barId].render.color);
                }
            }
        },

        getBarColors : function (args) {
            var colors = {
                bar : null,
                inProgress : null
            };
            var barColor = args.color || this.appliedTheme.typeColors[args.barConfig.type];
            if (!args.skipResetUsedColors) {
                this.resetAppliedThemeColors();
            }
            if (barColor == null) {
                var availableColors = _.difference(this.appliedTheme.alternativeColors, this.appliedTheme.usedColors);
                if (availableColors.length) {
                    barColor = availableColors[0];
                } else {
                    barColor = this.appliedTheme.alternativeColors[0];
                }
                this.appliedTheme.usedColors.push(barColor);
            }
            switch (args.barConfig.render.progressStyle) {
            case BAR_PROGRESS_STYLE.CLASSICWEB.key:
            case BAR_PROGRESS_STYLE.CLASSIC.key :
                colors.bar = this.appliedTheme.typeColors.progress;
                colors.inProgress = barColor;
                break;
            case BAR_PROGRESS_STYLE.MODERN.key :
                colors.bar = barColor;
                colors.inProgress = BAR_COLORS[barColor].defaultProgress;
                break;
            default :
                colors.bar = barColor;
                break;
            }

            return colors;
        },

        resetBarColors : function (args) {
            var colors = this.getBarColors(args);
            this.updateBarColor(
                {
                    color : colors.bar,
                    barId : args.barId,
                    ignoreProgressColor : true
                });
            this.updateBarProgressColor(
                {
                    color : colors.inProgress,
                    barId : args.barId
                });
        },


        addBar : function (args) {
            var barId = 'bar' + _.uniqueId(this.config.bars.order.length + 1);
            var usedBarTypes = _.map(_.omit(this.config.bars.definitions, 'summary'), function (value) {
                return value.barId !== 'summary' ? value.type : null;
            });
            var barTypes = _.map(this.barTypes, function (barType) {
                return barType.key;
            });
            var availableBarTypes = _.difference(barTypes, usedBarTypes);
            var type = barTypes[0];
            if (availableBarTypes.length) {
                type = availableBarTypes[0];
            }

            var barType = _.findWhere(this.barTypes, { key : type });
            var config = {
                type : type,
                render : {
                    size : 'M'
                }
            };
            if (barType.configRender.showProgressCheckbox) {
                if (barType.configRender.defaultShowProgress) {
                    config.render.progressStyle = this.appliedTheme.defaultProgressStyle;
                    config.render.showProgress = true;
                }
            }
            if (barType.configRender.defaultShowCritical && barType.configRender.showCriticalCheckbox) {
                config.render.showCritical = true;
            }

            var colors = this.getBarColors({ barConfig : config });
            config.render.color = colors.bar;
            config.render.progressColor = colors.inProgress;

            this.config.bars.definitions[barId] = config;
            this.config.bars.order.push(barId);
            this.view.addBar(barId);
            this.selectBar(barId);
            this._trigger('config-updated');
        },

        // Todo: if label selected add next to it
        addLabel : function (args) {
            var labelId = 'label' + _.uniqueId(this.config.bars.order.length + 1);

            var labelOptions = {
                id : labelId,
                key : this.options.defaultLabel,
                render : {
                    position : 'L',
                    size : 'M',
                    color : '000000'
                },
                barId : this.selection.type === 'bar' ? this.selection.id : this.selection.config.barId
            };
            /* jshint ignore:start */
            while (_.find(_.values(this.config.labels), function (labelConfig) {
                return labelConfig.barId === labelOptions.barId && labelConfig.render.position === labelOptions.render.position;
            }) != null) {
                var positionIndex = LABEL_POSITION.order.indexOf(labelOptions.render.position);
                if (positionIndex < LABEL_POSITION.order.length - 1) {
                    labelOptions.render.position = LABEL_POSITION.order[positionIndex + 1];
                } else {
                    return;
                }
            }
            /* jshint ignore:end */
            this.config.labels[labelId] = labelOptions;
            this.view.addLabel(labelId);
            this.selectLabel(labelId);
            this._trigger('config-updated');
        },

        removeBar : function (args) {
            if (args.barId == null) {
                args.barId = this.selection.id;
            }
            delete this.config.bars.definitions[args.barId];
            this.config.bars.order = _.without(this.config.bars.order, args.barId);
            this.view.removeBar(args.barId);
            this._trigger('config-updated');
        },

        removeLabel : function (args) {
            if (args.labelId == null) {
                args.labelId = this.selection.id;
            }
            delete this.config.labels[args.labelId];
            this.view.removeLabel(args.labelId);
            this._trigger('config-updated');
        },

        updateBarColor : function (args) {
            if (args.barId == null) {
                args.barId = this.selection.id;
            }
            var config = this.config.bars.definitions[args.barId];
            config.render.color = args.color;
            this.view.updateBarColor(args.barId);
            if (!args.ignoreProgressColor && config.render.showProgress && config.render.progressStyle === BAR_PROGRESS_STYLE.MODERN.key) {
                config.render.progressColor = BAR_COLORS[config.render.color].defaultProgress;
                this.view.updateBarProgressColor(args.barId);
                this.view.applyBarSettings(args.barId);
            }
            this._trigger('config-updated');
        },

        updateBarProgressColor : function (args) {
            if (args.barId == null) {
                args.barId = this.selection.id;
            }
            this.config.bars.definitions[args.barId].render.progressColor = args.color;
            this.view.updateBarProgressColor(args.barId);
            this._trigger('config-updated');
        },

        swapProgressColor : function (args) {
            if (args.barId == null) {
                args.barId = this.selection.id;
            }
            var config = this.config.bars.definitions[args.barId];
            var progressColor = config.render.progressColor;
            this.updateBarProgressColor({ color : config.render.color });
            this.updateBarColor({ color : progressColor });
            this.view.applyBarSettings(args.barId);
            this._trigger('config-updated');
        },

        updateBarShowProgress : function (showProgress) {
            var config = this.config.bars.definitions[this.selection.id];
            config.render.showProgress = showProgress;
            if (showProgress) {
                var style = BAR_PROGRESS_STYLE[this.appliedTheme.defaultProgressStyle];
                if (this.selection.id === 'summary') {
                    style = BAR_PROGRESS_STYLE.MODERN;
                }
                this.updateBarProgressStyle({ barId : this.selection.id, style : style });
            } else {
                var currentColor = config.render.color;
                if (config.render.progressStyle === BAR_PROGRESS_STYLE.CLASSIC.key || config.render.progressStyle === BAR_PROGRESS_STYLE.CLASSICWEB.key) {
                    currentColor = config.render.progressColor;
                }
                delete config.render.progressColor;
                delete config.render.progressStyle;
                this.resetBarColors({ barConfig : config, color : currentColor, barId : this.selection.id });
            }
            this.view.updateBarShowProgress(this.selection.id);
            this.view.applyBarSettings(this.selection.id);
            this._trigger('config-updated');
        },


        updateBarHeight : function (newHeight) {
            this.selection.config.render.size = newHeight.key;
            this.view.positioner.updateBarHeight(this.selection.id);
            this._trigger('config-updated');
        },

        updateBarType : function (args) {
            var newType = args.type;
            if (args.barId == null) {
                args.barId = this.selection.id;
            }
            var config = this.config.bars.definitions[args.barId];
            config.type = newType.key;
            var barType = _.findWhere(this.barTypes, { key : config.type });
            this.sanitizeBarConfig({ config : config, barType : barType });
            if (barType.configRender.showProgressCheckbox) {
                if (barType.configRender.defaultShowProgress) {
                    config.render.showProgress = true;
                } else {
                    config.render.showProgress = false;
                }
                this.updateBarShowProgress(config.render.showProgress);
            }
            if (barType.configRender.showCriticalCheckbox) {
                config.render.showCritical = barType.configRender.defaultShowCritical;
            }
            var color = null;
            if (args.barId === 'summary') {
                color = this.appliedTheme.summaryColor;
            }
            this.resetBarColors({ barConfig : config, barId : args.barId, color : color });

            this.view.applyBarSettings(this.selection.id);

            this._trigger('config-updated');
        },

        sanitizeBarConfig : function (args) {
            var config = args.config;
            var barType = args.barType;

            if (!barType.configRender.showNameField) {
                delete config.name;
            }
            if (!barType.configRender.showProgressField && config.keys) {
                delete config.keys.percent;
            }
            if (!barType.configRender.showEndField && config.keys) {
                delete config.keys.end;
            }
            if (!barType.configRender.showStartField && config.keys) {
                delete config.keys.start;
            }
            if (!barType.configRender.showProgressCheckbox) {
                delete config.render.showProgress;
                delete config.render.progressColor;
                delete config.render.progressStyle;
            }
            if (!barType.configRender.showCriticalCheckbox) {
                delete config.render.showCritical;
            }
            delete config.render.showRelationships;
        },

        updateBarName : function (name) {
            this.config.bars.definitions[this.selection.id].name = name;
            this._trigger('config-updated');
        },

        updateBarStart : function (start) {
            var config = this.config.bars.definitions[this.selection.id];
            if (config.keys == null) {
                config.keys = {};
            }
            config.keys.start = start.dataIndex;
            this._trigger('config-updated');
        },

        updateBarEnd : function (end) {
            var config = this.config.bars.definitions[this.selection.id];
            if (config.keys == null) {
                config.keys = {};
            }
            config.keys.end = end.dataIndex;
            this._trigger('config-updated');
        },

        updateBarProgress : function (progress) {
            var config = this.config.bars.definitions[this.selection.id];
            if (config.keys == null) {
                config.keys = {};
            }
            if (progress != null && config.showProgress !== true) {
                config.render.showProgress = true;
                config.keys.percent = progress.dataIndex;
                config.render.alwaysShowProgress = true;
                config.render.progressColor = BAR_COLORS[config.render.color].defaultProgress;
            } else {
                config.render.showProgress = false;
                delete config.keys.percent;
            }
            this.view.updateBarShowProgress(this.selection.id);
            this._trigger('config-updated');
        },

        updateLabelHeight : function (newHeight) {
            this.selection.config.render.size = newHeight.key;
            this.view.positioner.updateLabelHeight(this.selection.id);
            this._trigger('config-updated');
        },

        updateLabelColor : function (args) {
            if (args.labelId == null) {
                args.labelId = this.selection.id;
            }

            this.config.labels[args.labelId].render.color = args.color;
            this.view.updateLabelColor(args.labelId);
            this._trigger('config-updated');
        },

        updateLabelField : function (args) {
            if (args.labelId == null) {
                args.labelId = this.selection.id;
            }
            this.config.labels[args.labelId].key = args.labelField.dataIndex;
            this.view.updateLabelField(args);
            this._trigger('config-updated');
        },

        updateShowExpandedSummaryBar : function (show) {
            this.config.chart.showExpandedSummaryBars = show;
            this._trigger('config-updated');
        },

        updateBarProgressStyle : function (args) {
            if (args.barId == null) {
                args.barId = this.selection.id;
            }

            var config = this.config.bars.definitions[args.barId];
            var currentStyle = config.render.progressStyle;
            config.render.progressStyle = args.style.key;
            this.view.updateBarProgressStyle(args.barId);
            var currentColor = config.render.color;
            if (currentStyle === BAR_PROGRESS_STYLE.CLASSIC.key || currentStyle === BAR_PROGRESS_STYLE.CLASSICWEB.key) {
                currentColor = config.render.progressColor;
            }
            this.resetBarColors({ barConfig : config, color : currentColor, barId : args.barId });

            this.view.applyBarSettings(this.selection.id);

            this._trigger('config-updated');
        },

        updateBarShowCritical : function (showCritical) {
            this.config.bars.definitions[this.selection.id].render.showCritical = showCritical;

            this._trigger('config-updated');
        },

        getUpdatedConfig : function () {
            var that = this;
            this.view.positioner.updateConfig();
            var summaryBar = that.config.bars.definitions.summary;
            var labels = [];
            _.forEach(_.values(that.config.labels), function (labelConfig) {
                if (labelConfig.barId === 'summary') {
                    labels.push(_.omit(labelConfig, 'barId'));
                }
            });
            summaryBar.labels = labels;
            var relationshipBarSet = false;
            var bars = _.map(this.config.bars.order, function (barId) {
                var barConfig = that.config.bars.definitions[barId];
                var barType = _.findWhere(that.barTypes, { key : barConfig.type });
                that.sanitizeBarConfig({ config : barConfig, barType : barType });
                if (barType.configRender.showRelationships && !relationshipBarSet) {
                    barConfig.render.showRelationships = true;
                    relationshipBarSet = true;
                }
                var labels = [];
                _.forEach(_.values(that.config.labels), function (labelConfig) {
                    if (labelConfig.barId === barId) {
                        labels.push(_.omit(labelConfig, 'barId'));
                    }
                });
                barConfig.labels = labels;
                return barConfig;
            });

            if (!relationshipBarSet) {
                bars[0].render.showRelationships = true;
            }

            return {
                bars : bars,
                layout : {
                    rowHeight : this.config.layout.rowHeight
                },
                summaryBar : summaryBar,
                chart : this.config.chart

            };
        },


        destroy : function () {
            // remove any editors and menus
            _.forEach(this.timeouts, function (timeout) {
                clearTimeout(timeout);
            });
            this.element.children().each(function () {
                $(this).remove();
            });

            this.element.empty();
            this._tearDownTriggers();
            // Does nice things like unbind all namespaced events on the original element
            $.Widget.prototype.destroy.call(this);
        },

        _tearDownTriggers : function () {
            this.element.off(this.widgetEventPrefix);
        }

    });
});
