define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.columns/core/AdvancedColumnFilterEditor',
    'gridapi.grid/constants',
    'markup!gridapi.grid/advancedFilters/advancedFilterHeaderTemplate.htm'
], function ($, _, AdvancedColumnFilterEditor, constants, headerTemplate) {
    'use strict';

    var AdvancedFilters = function (args) {
        this.widget = args.widget;
        this.view = args.view;
        this.options = this.widget.options;
        this.resetFilters();
        this.orderedFilters = [];
    };

    AdvancedFilters.prototype = {
        constructor : AdvancedFilters,
        resetFilters : function () {
            this.filters = {};
        },
        advancedFilterUpdated : function (filterEditor, filterValue) {
            var dataIndex = filterEditor.column.dataIndex;
            var advancedFilter = _.get(this.filters, dataIndex);
            if (advancedFilter) {
                advancedFilter.filterData = filterValue;
            }
            this.widget.applyAdvancedFilters(this.getFilterViewData());
        },
        applyFilterViewData : function (filterViewData) {
            var that = this;
            _.forEach(filterViewData, function (filterValue, dataIndex) {
                var advancedFilter = _.get(that.filters, dataIndex);
                if (advancedFilter) {
                    advancedFilter.editor.setFilterData(filterValue);
                    advancedFilter.filterData = advancedFilter.editor.getFilterData();
                }
            });
        },
        getFilterViewData : function () {
            return _.reduce(this.filters, function (filterViewData, filter) {
                var data = filter.editor.getFilterData();
                if (data) {
                    filterViewData[filter.column.dataIndex] = data;
                }
                return filterViewData;
            }, {});
        },
        focusFilter : function (dataIndex) {
            var filter = _.get(this.filters, dataIndex);
            if (filter && filter.editor) {
                filter.editor.focusEditor();
                this.view.selectColumnFilterCell(dataIndex);
            }
        },
        focusNextFilter : function (dataIndex, moveRight) {
            var direction = moveRight ? 1 : -1;
            var currentIndex = _.indexOf(this.orderedFilters, dataIndex);
            var nextIndex = currentIndex + direction;
            var nextFilter = this.filters[this.orderedFilters[nextIndex]];
            while (nextFilter && !_.get(nextFilter, 'column.advancedFilters.enabled')) {
                nextIndex += direction;
                nextFilter = this.filters[this.orderedFilters[nextIndex]];
            }
            if (nextFilter && nextFilter.editor != null) {
                this.view.selectColumnFilterCell(nextFilter.column.dataIndex);
                setTimeout(function () {
                    nextFilter.editor.focusEditor();
                }, 50);
            }
        },
        make : function (column) {
            var dataIndex = column.dataIndex;
            var initialData = _.get(this.widget.config.advancedFilters, dataIndex, null);
            var editor = new AdvancedColumnFilterEditor(column, {
                advancedFilters : this,
                advancedFilterFns : _.get(column, 'advancedFilters.advancedFilterEditorFns', {}),
                initialData : initialData
            });
            var $header = $(headerTemplate({
                id : _.uniqueId('grid-'), // todo not like this
                dataIndex : dataIndex,
                width : column.width
            })).append(editor.getElement());
            this.filters[dataIndex] = {
                column : column,
                editor : editor,
                $header : $header,
                filterData : initialData
            };
            this.orderedFilters.push(dataIndex);
            return $header;
        },
        destroy : function () {
            _.forEach(this.filters, function (filter) {
                filter.editor.destroy();
            });
        }
    };
    return AdvancedFilters;
});
