define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.columnMenu/ColumnMenuView',
    'gridapi.grid/view/menuUtils',
    'gridapi.grid/utils'
], function ($, _, ColumnMenuView, menuUtils, utils) {
    'use strict';

    function ColumnMenu(gridView) {
        this.grid = gridView.widget;
        this.destroy = this.destroy.bind(this);
        this.timeouts = utils.timeoutFactory();
        menuUtils.allowRefocusOn(this);
    }

    ColumnMenu.prototype = {
        render : function ($th, event, keyboard) {
            // clear any previous instances
            this.destroy({ fromCreate : true });

            var column = this.grid.columnDefinitions[$th.attr('data-index')];
            if (!this.grid.options.columnMenu.enabled || !column.menu || column.dataIndex === 'empty-column') {
                return;
            }

            var group = $th.closest('div.viewport').data('group');
            var position = $th.offset();
            this.view = new ColumnMenuView(this, column, group);
            this.view.render({
                left : position.left + 'px',
                top : (position.top + $th.outerHeight()) + 'px',
                fromKeyboard : keyboard
            });
            return this.view.$menu;
        },

        isOpen : function () {
            return this.view != null;
        },

        destroy : function (args) {
            if (this.view != null) {
                this.view.destroy();
                this.view = null;
                if (args == null) {
                    this.grid.requestBrowserFocus();
                }
                return true;
            }
            this.timeouts.clear();
            return false;
        },

        _getColumnSortOrder : function (column) {
            var grid = this.grid;
            for (var index = 0; index < grid.config.sortBy.length; index++) {
                var sort = grid.config.sortBy[index];
                if (column.dataIndex === sort.dataIndex) {
                    return sort.order;
                }
            }
            return '';
        }
    };

    return ColumnMenu;
});
