define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/constants'
], function ($, _, constants) {
    'use strict';


    return function (grid) {
        var $columnMoveShade = grid.$grid.find('.col-move-shade'),
            $columnMoveBar = grid.$grid.find('.col-move-bar'),
            $colGroupDropShade = grid.$grid.find('.col-group-drop-shade'),
            appendBeforeIndex, currentIndex, columns = [], $th = $(), viewport, offsets = {}, widths = {},
            positions = {
                shade : {}
            }, direction = -1, scrollTimer, currentDropGroupViewport;

        var mousemoveFn = function () {
            return function (event) {
                // left click button
                if (event.which === 1) {
                    onMoveFn(event.clientX + $(window).scrollLeft());
                }
            };
        };
        var onMoveFn = function (mouseX) {
            direction = mouseX - offsets.grid - positions.pointer;
            positions.pointer = mouseX - offsets.grid;
            positionShadeFn();
            if (currentDropGroupViewport == null) {
                calcColumnMoveFn();
            }
        };
        var mouseupFn = function () {
            return function (event) {
                var eventData;
                if (event.which === 1) {
                    // switch of mouse events
                    grid.$grid.off('mousemove.col-move');
                    $(document.body).off('mouseup.col-move mouseleave.col-move');
                    if (scrollTimer != null) {
                        clearTimeout(scrollTimer);
                        scrollTimer = null;
                    }

                    if (currentDropGroupViewport == null) {
                        eventData = moveColumnFn();
                        grid.widget._trigger('view-updated', null, { action : 'column-move', column : $th.attr('data-index'), userAction : true, 'columnGroupUpdate': eventData });
                    } else {
                        eventData = dropIntoGroupFn();
                        grid.widget._trigger('view-updated', null, { action : 'column-drop', column : $th.attr('data-index'), viewport : currentDropGroupViewport.group, origin : viewport.group, userAction : true, 'columnGroupUpdate': eventData });
                    }


                    // hide the column move shade and bar
                    $colGroupDropShade.hide();
                    $columnMoveBar.hide();
                    $columnMoveShade.hide();
                    $th.removeClass('col-moving');
                    grid.resize();
                    grid.reselectRecords();
                    // This is very important. Dont ever comment it out. Commenting it out is bad for you.
                    columns = [];
                }
            };
        };

        var mouseLeaveFn = function () {
            return function (event) {
                grid.$grid.off('mousemove.col-move');
                $(document.body).off('mouseup.col-move mouseleave.col-move');
                if (scrollTimer != null) {
                    clearTimeout(scrollTimer);
                    scrollTimer = null;
                }

                // hide the column move shade and bar
                $colGroupDropShade.hide();
                $columnMoveBar.hide();
                $columnMoveShade.hide();
                $th.removeClass('col-moving');

                // This is very important. Dont ever comment it out. Commenting it out is bad for you.
                columns = [];
            };
        };

        var createEventData = function (source, dest) {
            var getGroup = function (group) {
                var order = [];
                if (!_.isUndefined(group)) {
                    group.columns.forEach(function (val) {
                        if (val.dataIndex !== constants.EMPTY_COLUMN) {
                            order.push(val.dataIndex);
                        }
                    });
                    return {
                        'id': group.id,
                        'order': order
                    };
                }
            };

            return _.assign({}, {
                'oldGroup': getGroup(source)
            }, {
                'newGroup': getGroup(dest)
            });
        };


        var moveColumnFn = function () {
            if (currentIndex + 1 !== appendBeforeIndex && currentIndex !== appendBeforeIndex) {
                _.forEach(['thead', 'tbody', 'tfoot'], function (tag) {
                    _.forEach(grid.viewports['$group-' + viewport.group].find(tag).children(), function (tr) {
                        var $ths = $(tr).children(),
                            $thDrag = $ths.eq(currentIndex),
                            $thDrop = $ths.eq(appendBeforeIndex < columns.length ? appendBeforeIndex : appendBeforeIndex - 1);
                        if (appendBeforeIndex < columns.length) {
                            $thDrop.before($thDrag);
                        } else {
                            $thDrop.after($thDrag);
                        }
                    });
                });

                var columnDataIndex = columns[currentIndex].dataIndex;
                var columnToMove = _.find(viewport.groupConfig.columns, function (column) {
                    return column.dataIndex === columnDataIndex;
                });
                var appendBefore = true;
                var referenceColumn = columns[appendBeforeIndex];
                if (referenceColumn == null) {
                    referenceColumn = _.last(columns);
                    appendBefore = false;
                }
                var referenceIndex = referenceColumn.dataIndex;
                viewport.groupConfig.columns = _.without(viewport.groupConfig.columns, columnToMove);
                var columnToInsertAfter = _.find(viewport.groupConfig.columns, function (column) {
                    return column.dataIndex === referenceIndex;
                });
                var insertIndex = viewport.groupConfig.columns.indexOf(columnToInsertAfter);
                if (!appendBefore) {
                    insertIndex++;
                }
                viewport.groupConfig.columns.splice(insertIndex, 0, columnToMove);
                return createEventData(viewport.groupConfig);
            }
        };

        var dropIntoGroupFn = function () {
            var dropGroup = currentDropGroupViewport.group;
            var headersToMove = grid.viewports['$group-' + viewport.group].find('th:eq(' + currentIndex + ')');
            // Fix the header cells
            _.forEach(grid.viewports['$group-' + dropGroup].find('thead').children(':not([data-id="header"])'), function (tr, index) {
                var $lastColumn = $(tr).find('[data-index="empty-column"]');
                if ($lastColumn.length) {
                    $lastColumn.before(headersToMove.eq(index));
                } else {
                    $(tr).append(headersToMove.eq(index));
                }
            });
            // Move the column config into the proper group
            var currentGroupConfig = viewport.groupConfig;
            var newGroupConfig = grid.viewports[_.first(grid.config.rowsets).id][dropGroup].groupConfig;
            var configToMove = currentGroupConfig.columns.splice(currentIndex, 1)[0];

            if (newGroupConfig.columns[newGroupConfig.columns.length - 1].dataIndex === 'empty-column') {
                newGroupConfig.columns.splice(newGroupConfig.columns.length - 1, 0, configToMove);
            } else {
                newGroupConfig.columns.push(configToMove);
            }

            // If it is moving to be the last column or will make a new last column need to rerender all the rows.
            if (grid.lastVisibleColumn === columns[currentIndex].dataIndex) {
                grid.lastVisibleColumn = columns[currentIndex - 1].dataIndex;
                grid.widget._forceViewRender();
            } else if (_.last(grid.config.groups) === newGroupConfig) {
                grid.lastVisibleColumn = columns[currentIndex].dataIndex;
                grid.widget._forceViewRender();
            } else { // Else just move the cells around.
                var cellsToMove = grid.viewports['$group-' + viewport.group].find('tbody').children().find('td:eq(' + currentIndex + ')');
                var footerCellsToMove = grid.viewports['$group-' + viewport.group].find('tfoot').children().find('td:eq(' + currentIndex + ')');

                _.forEach(grid.viewports['$group-' + dropGroup].find('tbody').children(), function (tr, index) {
                    $(tr).append(cellsToMove.eq(index));
                });
                _.forEach(grid.viewports['$group-' + dropGroup].find('tfoot').children(), function (tr, index) {
                    $(tr).append(footerCellsToMove.eq(index));
                });
            }

            grid.resize();
            return createEventData(viewport.groupConfig, newGroupConfig);
        };

        /*
         * This function needs to be FAST as it occurs in mousemove
         */
        var calcColumnMoveFn = function () {
            positions.table = viewport.$table.position().left;
            if (direction > 0 && appendBeforeIndex < columns.length) {
                if (positions.shade.right > columns[appendBeforeIndex].middle + positions.table) {
                    appendBeforeIndex++;
                    positionMoveBarFn();
                }
            } else if (direction < 0 && appendBeforeIndex > 0) {
                if (positions.shade.left < columns[appendBeforeIndex - 1].middle + positions.table) {
                    appendBeforeIndex--;
                    positionMoveBarFn();
                }
            }
        };

        var positionShadeFn = function () {
            if (scrollTimer != null) {
                clearTimeout(scrollTimer);
                scrollTimer = null;
            }

            var leftPosition = positions.pointer - widths.thMiddle,
                scrollAmount = 0;
            // Check if its the moving within the same viewport
            if (leftPosition > offsets.viewport && (leftPosition + widths.th) < offsets.viewportRight) {
                $columnMoveShade.css('left', leftPosition);
                positions.shade.left = leftPosition - offsets.viewport;
                positions.shade.right = positions.shade.left + widths.th;
                $columnMoveShade.show();
                currentDropGroupViewport = null;
                return;
            }
            // If outside of viewport check others
            if (positions.pointer < offsets.viewport || positions.pointer > offsets.viewportRight) {
                positionViewportDrop();
                return;
            } else {
                $colGroupDropShade.hide();
                currentDropGroupViewport = null;
            }

            // Check boundaries of the current viewport to see if it needs to scroll
            if (leftPosition < offsets.viewport) {
                leftPosition = offsets.viewport;
                scrollAmount = 15;
            } else if ((leftPosition + widths.th) > offsets.viewportRight) {
                leftPosition = offsets.viewport + widths.viewport - widths.th;
                scrollAmount = -15;
            }
            // Duplication for speed
            positions.shade.left = leftPosition - offsets.viewport;
            positions.shade.right = positions.shade.left + widths.th;
            $columnMoveShade.css('left', leftPosition);
            $columnMoveShade.show();
            // scroll if on edge
            if (viewport.groupConfig.type === 'flex' && scrollAmount !== 0) {
                scrollTimer = setInterval(function () {
                    scrollFn(scrollAmount);
                }, 150);
            }
        };

        var scrollFn = function (amount) {
            var $scroller = grid.pseudoScrollers.horizontal[viewport.group].$scroller;
            $scroller.scrollLeft($scroller.scrollLeft() - amount);
            direction = -amount;
            calcColumnMoveFn();
        };
        var withinViewportFn = function (viewToCheck) {
            var values = {};
            values.left = viewToCheck.offset().left - offsets.grid;
            values.width = viewToCheck.width();
            values.right = values.left + values.width;
            values.fits = positions.pointer < values.right && positions.pointer > values.left;
            values.group = viewToCheck.data('group');
            values.$viewport = viewToCheck;
            return values;
        };

        var positionViewportDrop = function () {
            var groupToCheck, dropViewport;
            // Check if its the current drop viewport still
            if (currentDropGroupViewport != null && (positions.pointer < currentDropGroupViewport.right && positions.pointer > currentDropGroupViewport.left)) {
                return;
            }
            // If not check the viewports to the left and right of the current based on mouse position.
            if (positions.pointer < offsets.viewport) {
                groupToCheck = viewport.groupConfig.prev;  // Check left
                while (groupToCheck != null) {
                    dropViewport = withinViewportFn(grid.viewports[grid.firstRowset.id][groupToCheck.id].$viewport);
                    if (dropViewport.fits) {
                        break;
                    } else {
                        dropViewport = null;
                        groupToCheck = groupToCheck.prev;
                    }
                }
            } else {
                groupToCheck = viewport.groupConfig.next; // Check right
                while (groupToCheck != null) {
                    dropViewport = withinViewportFn(grid.viewports[grid.firstRowset.id][groupToCheck.id].$viewport);
                    if (dropViewport.fits) {
                        break;
                    } else {
                        dropViewport = null;
                        groupToCheck = groupToCheck.next;
                    }
                }
            }
            // If none hide it
            if (dropViewport == null) {
                $colGroupDropShade.hide();
                $colGroupDropShade.css('width', 0);
                $colGroupDropShade.css('left', 0);
                currentDropGroupViewport = null;
            } else {
                if (currentDropGroupViewport !== dropViewport) {
                    // Check that you can drop into the viewport
                    if (dropViewport.group === 'row-selector-group' || dropViewport.group === 'menu-button-group' ||
                        grid.widget._trigger('before-change-column-group', null,
                            {
                                currentGroup : viewport.group,
                                dropGroup : dropViewport.group,
                                column : $th.attr('data-index')
                            }) === false) {
                        return;
                    }
                    $colGroupDropShade.css('width', dropViewport.width);
                    $colGroupDropShade.css('left', dropViewport.left);
                    currentDropGroupViewport = dropViewport;
                }
                $colGroupDropShade.show();
                $columnMoveBar.hide();
                $columnMoveShade.hide();
            }
        };


        var positionMoveBarFn = function () {
            var activeColumn = columns[appendBeforeIndex];
            if (appendBeforeIndex - 1 === currentIndex) {
                activeColumn = columns[currentIndex];
            } else if (appendBeforeIndex === columns.length) {
                activeColumn = columns[appendBeforeIndex - 1];
            }

            positions.line = activeColumn.$th.offset().left - offsets.grid;
            // Adjust for last column
            if (appendBeforeIndex === columns.length) {
                positions.line += activeColumn.width;
            }
            // Adjust line for scrolling to keep it in viewport
            if (positions.line > offsets.viewportRight) {
                positions.line = offsets.viewportRight;
            } else if (positions.line < offsets.viewport) {
                positions.line = offsets.viewport;
            }
            $columnMoveBar.css('left', positions.line);
            $columnMoveBar.show();
        };

        var updateColumnsFn = function () {
            _.forEach(viewport.$thead.children().children(), function (th) {
                var $header = $(th);
                var dataIndex = $header.attr('data-index');
                if (dataIndex !== 'row-selector' && dataIndex !== 'empty-column') {
                    var colLeft = $header.position().left;
                    var width = parseInt($header.attr('width'), 10);
                    columns.push({
                        dataIndex : dataIndex,
                        $th : $header,
                        left : colLeft,
                        right : colLeft + width,
                        width : width,
                        middle : colLeft + width / 2
                    });
                    if ($th.data('index') === dataIndex) {
                        currentIndex = columns.length - 1;
                        appendBeforeIndex = currentIndex + 1;
                    }
                }
            });
        };

        return {

            initColumnDrag : function ($column, activeViewport, mouseX) {
                // Do not let them drag just 1 column around, as it would empty out a group.
                var visibleColumns = _.filter(activeViewport.groupConfig.columns, function (column) {
                    return column.visible && column.dataIndex !== 'empty-column';
                });

                if (visibleColumns.length > 1) {
                    $th = $column;
                    viewport = activeViewport;


                    $th.addClass('col-moving');
                    offsets.grid = grid.$grid.offset().left;
                    offsets.viewport = viewport.$viewport.offset().left - offsets.grid;

                    widths.th = parseInt($th.attr('width'), 10);
                    widths.thMiddle = widths.th / 2;
                    widths.viewport = viewport.$viewport.width();
                    offsets.viewportRight = offsets.viewport + widths.viewport;

                    positions.pointer = mouseX - offsets.grid;
                    $columnMoveShade.css('width', widths.th);

                    updateColumnsFn();
                    currentDropGroupViewport = null;
                    // switch mouse move and down event to on
                    grid.$grid.on('mousemove.col-move', _.throttle(mousemoveFn(), 20));
                    $(document.body).on('mouseup.col-move', mouseupFn());
                    $(document.body).on('mouseleave.col-move', mouseLeaveFn());
                    onMoveFn(mouseX + $(window).scrollLeft());
                }
            }
        };
    };
})
;
