define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/gridMixins'
], function ($, _, mixins) {
    'use strict';

    var renderer = function () {
    };

    renderer.extend = mixins.extendFn;

    // alignment of text
    var columnAlignment = {
        left : '', // 'cell-align-left', no need to do this, save some css speed
        right : 'cell-align-right',
        center : 'cell-align-center'
    };

    var renderPopover = function (header, tooltip, content) {
        return '<div class="orcl-popover" tabindex="0" aria-role="button">' +
                    '<div class="orcl-popover-outer">' +
                        '<i class="pgbu-icon pgbu-icon-popover-help" title="' + tooltip + '"><span class="spoken-text">' + tooltip + '</span></i>' +
                    '</div>' +
                    '<div class="orcl-popover-inner">' +
                        '<div class="orcl-popover-content" aria-hidden="true">' +
                            (header != null ? '<div class="header">' + header + '</div>' : '') +
                            '<div>' +
                                content +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>';
    };

    _.assign(renderer.prototype, {
        /**
         * Override to build cell html
         * Must be in string format for performance - NO JQUERY!!
         *
         * @method create
         * @private
         */
        create : function (column, options) {
            // Add cell, classes and renderer
            var html = '<th tabindex="-1" role="' + this.getAriaRole(column) + '" unselectable="on" data-index="' + column.dataIndex + '" id="' + options.id + '" ';
            html += this.getClassMarkup(column);
            if (column.abbr != null) {
                html += ' abbr="' + column.abbr + '"';
            }
            html += ' width="' + column.width + '"';
            html += this.getAriaMarkup(column, options);
            html += ' >';
            html += '<div class="cell-positioner"> ';
            html += '<div unselectable="on" class="cell-renderer ' + this.getAlignment(column) + '">';
            html += this.getHeaderMarkup(column);
            if (column.required && !column.hideAsterisk) {
                html += '<span  aria-hidden="true" role="presentation" class="column-required-marker">*</span>';
            }
            if (column.popoverContent) {
                html += renderPopover(column.popoverHeader, options.popoverTooltip, column.popoverContent);
            }
            html += '</div>';
            if (column.filterable) {
                html += '<div class="column-filtered-icon"><i class="pgbu-icon pgbu-icon-small pgbu-icon-filter"></i></div>';
            }
            if (column.sortable) {
                html += '<div class="column-sort-icon"><i class="pgbu-icon pgbu-icon-small"></i></div>';
            }
            if (column.resizable) {
                html += '<div class="col-resize-anchor"></div>';
            }
            html += '</div></th>';

            return html;
        },

        attachListeners : function () {

        },

        getAriaRole : function (/* column */) {
            return 'columnheader';
        },

        getHeaderMarkup : function (column) {
            return column.title;
        },

        getAriaMarkup : function (column, options) {
            var descriptionIds = this.getDescribedBy(column, options);
            var tooltipText = column.tooltip ? column.tooltip : column.title;
            var html = '';
            if (tooltipText != null && tooltipText.length) {
                html += ' title="' + tooltipText + '"';
                html += ' aria-label="' + tooltipText + '"';
                html += ' aria-labelledby="' + options.id + '"';
            }
            if (descriptionIds.length) {
                html += ' aria-describedby="' + descriptionIds.join(' ') + '" ';
            }
            return html;
        },

        getDescribedBy : function (column, options) {
            var describedBy = [];
            if (column.sortable) {
                describedBy.push(options.descriptionIds.cellState.columnSort);
            }
            return describedBy;
        },

        getClassMarkup : function (column) {
            var classes = ' class="pgbu-cell ';
            classes += this.getClasses(column);
            classes += column.columnClass + ' ';
            classes += '" ';
            return classes;
        },
        getClasses : function () {
            return '';
        },

        getAlignment : function (column) {
            return ' ' + columnAlignment[column.headerAlign] || columnAlignment.headerAlign;
        }
    });

    return renderer;
});
