define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/columns/core/editor',
    'gridapi.grid/gridMixins',
    'gridapi.jqueryui-scroll-parent'
], function ($, _, Editor, mixins) {
    'use strict';

    function LuxEditor() {
        Editor.apply(this, arguments);
    }

    LuxEditor.extend = mixins.extendFn;

    LuxEditor.prototype = _.assign(Object.create(Editor.prototype), {
        closeEditorOnMouseDown : function (event) {
            return !$(event.target).parents('.cell-editor, .oj-datepicker-content').length;
        },

        setAriaMarkup : function ($editorInput, options) {
            var that = this;
            $editorInput.attr('aria-labelledby', this.ariaLabels);
            if (options.column.describedBy != null && options.column.describedBy.length) {
                var ids = _.map(options.column.describedBy, function (item) {
                    return that.grid.descriptionIds.columnEditors[item] || '';
                });
                // Issue LUX-249
                if (options.column.spinner) {
                    ids.push(that.grid.descriptionIds.columnEditors.num_spin);
                }
                var describedBy = $editorInput.attr('aria-describedby');  // LUX-495
                if (describedBy != null && (options.column.displayType != 'date')) {  // LUX-495 part2
                    describedBy += ' ' + ids.join(' ');
                } else {
                    describedBy = ids.join(' ');
                }
                $editorInput.attr('aria-describedby', describedBy);
            }
        },

        setPosition : function ($editor) {
            var editorOffset = $editor.offsetParent().offset();  // it would be 0, 0 if non-static parent is not found
            var cellOffset = this.$cell.offset();

            $editor.css('top', cellOffset.top - editorOffset.top);
            $editor.css('left', cellOffset.left - editorOffset.left);
        },

        initialize : function (options) {
            var that = this;

            that.init(options);

            var $editor = that.make();
            $editor.attr('id', this.getEditorId());

            if (options.column.required || options.column.autoNumbered) {
                var message = (options.column.autoNumbered && (options.recordAttributes.newRow || options.fromQuickAddRow)) ? this.grid.options.locale.autoNumbered : this.grid.options.locale.required;
                $editor.find('input').attr('placeholder', '[' + message + ']');
            }

            if (!$editor.is('.cell-editor') && !$editor.is('.modal')) {
                throw new Error('Editor.create() - No "cell-editor" class.');
            }

            // If the grid is in the modal add the in-modal class that should boost the zIndex of the editor
            if (options.grid.options.inModal) {
                $editor.addClass('in-modal');
            }

            // append to document as editor is absolute in its positioning
            that.grid.$grid.parent().append($editor);

            this.setPosition($editor);
            this.setSize($editor);

            var $editorInput = this.getEditorInput();
            this.setAriaMarkup($editor, options);

            if (this.column.maxLength != null && this.column.maxLength > 0) {
                $editorInput.attr('maxlength', this.column.maxLength);
            }
            this.setDisabled($editorInput);
            that.initListeners();

            var $body = $(document.body);
            $body.on('mousedown' + that.grid.widget.eventNamespace + '-editor', function (event) {
                if (that.closeEditorOnMouseDown(event)) {
                    $body.off('mousedown' + that.grid.widget.eventNamespace + '-editor');
                    $editorInput.trigger('blur');
                    that.commit({});
                }
            });


            return $editor;
        }
    });

    return LuxEditor;
});
