define(
    ['gridapi.jquery',
        'gridapi.lodash',
        'gridapi.columns/core/editor',
        'gridapi.columns/core/renderer',
        'gridapi.columns/core/headerRenderer',
        'ojs/ojcore',
        'knockout',
        'ojs/ojdatetimepicker'
    ],
    function ($, _, editor, renderer, headerRenderer, oj) {
        'use strict';

        return {
            defaultOptions : {
                width : '100px',
                columnClass : 'date-column',
                dateType : 'date',
                disableColumnFilter: false
            },
            /*
            * default formatter for this column, user can override this while creating the grid widget in widget options.
            */
            formatter: oj.Validation.converterFactory(oj.ConverterFactory.CONVERTER_TYPE_DATETIME)
                .createConverter({
                    formatType: 'date',
                    dateFormat: 'long'
                }),
            headerRenderer: headerRenderer.extend({

            }),

            renderer: renderer.extend({
                format: function (renderOptions) {
                    return renderOptions.column.formatter.format(renderOptions.data);
                }
            }),
            editor: editor.extend({

                make: function () {
                    var that = this;
                    var $editor = $('<div class="cell-editor cell-editor-date cell-editor-current"></div>');
                    that.$editor = $editor;
                    this.isValid = true;
                    this.dateField = $('<input type = "text" />');
                    $editor.append(this.dateField);
                    this.dateField.ojInputDate({
                        converter: that.column.formatter,
                        validators: that.column.validators, // note: validatorS is not any specific perperty handled by columnDefFunctions.js. validator (singuler) is.
                        displayOptions: { 'messages': 'notewindow' }
                    });

                    /*
                     * Set the initial value only if it is valid, else set by render method.
                     * Note: value must be ISOString, but when you want to display invalid data this option can never be used.
                     * Way around: When the option is not set, the element's value property is used as its initial value if it exists.
                     */
                    if (!this.invalid) {
                        var val = this.record[this.column.dataIndex];
                        this.dateField.ojInputDate('option', 'value', val);
                    }

                    // edit and click-outside
                    $(document.body).on('mousedown' + that.grid.widget.eventNamespace + '-editor', function (event) {
                        var $target = $(event.target);
                        if (!$target.parents('.cell-editor, .oj-datepicker-content').length) {
                            that.dateField.off('ojoptionchange');
                        }
                    });

                    this.dateField.on('ojoptionchange', function (e, data) {
                        if (!that.getCommitStarted() && data.option !== 'rawValue') {
                            if (!(e.originalEvent && e.originalEvent.type === 'blur')) {
                                // if e.originalEvent is undefined (for enter or click events)
                                // if e.originalEvent is undefined and not blur (could be valid or invalid option change)
                                that.commit();
                            } // else:  do nothing 'blur' event is handled seperately  (escape key)
                        }
                    });

                    return $editor;
                },

                show: function (startString) {
                    this.render(startString);
                    var $editor = this.getEditor();
                    $editor.show();
                    this.drawFocusToInput();
                },

                render: function (/* startString */) {
                    if (!this.invalid) {
                        this.ignoreSelectText = true;
                    } else {
                        this.getEditorInput().val(this.data);
                        // this.dateField.off('ojoptionchange');
                    }
                },

                onKeyDown : function (event) {
                    var that = this;
                    //var grid = this.grid;
                    switch (event.keyCode) {
                    case $.ui.keyCode.TAB :
                        that.onTab(event);
                        break;
                    case $.ui.keyCode.ESCAPE :
                        this.dateField.off('ojoptionchange');
                        event.stopPropagation();
                        event.preventDefault();
                        break;
                    }
                },

                getNewData: function () {
                    var valid = this.validateUserInput();
                    var date_ISO_String = '';
                    if (valid) {
                        date_ISO_String = this.dateField.ojInputDate('option', 'value');
                    } else {
                        date_ISO_String = this.getEditorInput().val();
                    }
                    // console.log(' date_ISO_String: ' + date_ISO_String + ' valid: '+valid);
                    return {
                        value: date_ISO_String,
                        valid: valid
                    };
                },

                validateUserInput: function (/* data */) {
                    return this.dateField.ojInputDate('validate');
                },

                destroy: function () {
                    /*
                     * Issue LUX-233 (COLUMN GROUPS:  MULTI CELL EDIT OF DATE COLUMN BEHAVES INCORRECTLY)
                     * When 'editOnClick: true', newly created editor is destoryed because of setTimeout below.
                     * Resolution: identify current editor and don't destory it.
                     * end of lifecycle
                     */
                    $('.cell-editor').removeClass('cell-editor-current');
                    setTimeout(function () {
                        var $endOfLifecycleEditor = $('.cell-editor').not('.cell-editor-current');
                        $endOfLifecycleEditor.remove();
                    }, 10);
                },

                drawFocusToInput : function (select) {
                    var $input;
                    var value;
                    var inputElement;

                    var $editorInput = this.getEditorInput();
                    // draw input cursor into editor
                    if ($editorInput.length) {
                        $input = $editorInput;
                    } else {
                        $input = this.getEditor();
                    }

                    var touchInteractions = this.grid.touchInteractions;

                    if (!(touchInteractions && touchInteractions.options.enabled && touchInteractions._isTouch)) {   // not touch
                        $input.focus();
                    }
                    value = $input.val();
                    select = !this.ignoreSelectText && (select || this.detectSelectText());
                    inputElement = $input.get(0);
                    if (_.isFunction(inputElement.setSelectionRange)) {
                        inputElement.setSelectionRange(select ? 0 : value.length, value.length);
                    }
                }

            })
        };
    });
