/* global define */
define([
    'gridapi.jquery',
    'gridapi.underscore',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/headerRenderer',
    'gridapi.jqueryui'
], function ($, _, editor, renderer, headerRenderer) {
    'use strict';

    /**
     * Picker column assumes that data is held in an array.  Objects in the array at the minimal should have id, parentId and name
     * [{
     *     id : 'fooId',
     *     parentId : 'fooParentId',
     *     name : 'fooName'
     * }]
     */

    var pickerDialogTemplate = _.template('<div id="dialog-modal" title="<%=title%>">' +
        '<input id="picker-filter-input" type="text" />' +
        '<select id="picker-select-list" multiple="multiple" size="10">' +
        '<% for (var i = 0; i < dataOptions.length; i++) { %>' +
        '<%  var dataOption = dataOptions[i]; %>' +
        '<option value="<%=dataOption.id%>"><%=dataOption.name%></option>' +
        '<% } %>' +
        '</select>' +
        '<br>' +
        '<button id="picker-modal-accept" type="button" class="btn btn-primary ui-button ui-widget ui-state-default ui-corner-all ui-button-text-only" role="button" aria-disabled="false"><span class="ui-button-text">OK</span></button>');

    return {
        defaultOptions : {
            listDelim : ','
        },
        formatter : {
            format : function (datas, column) {
                var values = '';
                if (!_.isArray(datas)) {
                    datas = [datas];
                }
                _.forEach(datas, function (data, index) {
                    if (data != null) {
                        values += data.name;

                        if (index < datas.length - 1) {
                            // if not last item then add a delimiter
                            values += column.listDelim;
                        }
                    }
                });
                return values;
            },

            validate : function (/* datas */) {
                return true;
            }
        },

        headerRenderer : headerRenderer.extend({

        }),

        renderer : renderer.extend({
            format : function (renderOptions, data /*, record */) {
                var column = renderOptions.column;
                return column.formatter.format(data, column);
            }
        }),

        editor : editor.extend({

            init : function (options) {
                // have to clone as data will be an array for multi-select
                this.newData = _.clone(options.data);
                this.constructor.__super__.init(options);
            },

            make : function () {
                var that = this;
                var $editor = $('<div class="cell-editor cell-editor-picker" id="pgbu-grid-cell-editor"><input type="text" /><button class="picker-btn btn btn-primary" title="' + that.column.picker.locale.title + '">+</button></div>');
                var $editButton = $editor.find('button');
                $editButton.button();
                if (that.$modalDialog) {
                    that.$modalDialog.empty();
                }
                that.createPickerModal(that.column.picker);
                $editButton.click(function () {
                    that.$modalDialog.dialog('open');
                });
                return $editor;
            },

            createPickerModal: function (pickerOptions) {
                var that = this;
                var data = that.column.picker.tree.mediatorFns.load();

                // TODO: Separate the contents out into its own UI component of filterable option selection
                that.$modalDialog = $(pickerDialogTemplate({
                    title: pickerOptions.locale.title,
                    dataOptions: data
                }));
                that.$modalDialog.dialog({
                    autoOpen: false,
                    dialogClass: 'modal-task',
                    modal: true
                });

                // Handle updating the options based on filter input
                var $filterInput = that.$modalDialog.find('#picker-filter-input');
                var $filterSelect = that.$modalDialog.find('#picker-select-list');
                $filterInput.bind('change keyup', function () {
                    var regex = new RegExp($.trim($(this).val()), 'gi');
                    $filterSelect.find('option').each(function () {
                        var $this = $(this);
                        if ($this.text().match(regex) !== null) {
                            $this.filter('span > option').unwrap();
                        } else {
                            $this.filter(':not(span > option)').wrap('<span>').parent().hide();
                        }
                    });
                });

                // Handle accept button click event
                that.$acceptButton = that.$modalDialog.find('#picker-modal-accept');
                that.$acceptButton.click(function () {
                    var selectedOptions = that.$modalDialog.find('#picker-select-list').val();

                    // Our values
                    that.newData = [];
                    if (_.isArray(selectedOptions)) {
                        for (var i = 0; i < selectedOptions.length; i++) {
                            var dataObject = _.find(data, function (testData) {
                                return (testData.id === selectedOptions[i]);
                            });
                            if (dataObject) {
                                that.newData.push(dataObject);
                            }
                        }
                    }
                    that.getEditorInput().val(that.column.formatter.format(that.newData, that.column));
                    that.commit();
                    that.$modalDialog.dialog('close');
                });
            },

            render : function (keyCode, shiftKey) {
                var that = this;
                var data = _.isArray(this.data) ? this.data : [this.data];
                that.$modalDialog.find('#picker-select-list').val(_.pluck(data, 'id'));

                if (keyCode == null) {
                    this.getEditorInput().val(this.column.formatter.format(this.newData, this.column));
                } else {
                    this.constructor.__super__.render(keyCode, shiftKey);
                }
            },

            getNewData : function () {
                return { value: this.newData, valid: true };
            }
        })
    };
});
