define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/editor',
    'gridapi.oui'
], function ($, _, keyCode, renderer, editor, oui) {
    'use strict';

    return {
        defaultOptions : {
            width : '75px',
            columnClass : 'color-column',
            expandable : false,
            colorPicker : {
                fluid : true
            }
        },
        renderer : renderer.extend({
            getValueMarkup : function (renderOptions) {
                var data = renderOptions.record[renderOptions.column.dataIndex];
                var titleAttr = '';
                if (data) {
                    if (data.split('#').length === 1) {
                        data = '#' + data;
                    }

                    var colorName = oui.lang().color[data.substring(1).toUpperCase()];
                    if (colorName) {
                        titleAttr = ' title="' + colorName + '"';
                    }
                }

                    // !important to override bootstrap's '@media print' rules that make all background-colors transparent
                return '<i class="thumbnail color-picker"' + titleAttr + ' style="background-color:' + data + ' !important"></i>';
            },
            getExcelCell : function (renderOptions) {
                var value = this.getExcelCellData(renderOptions);
                return {
                    color: value,
                    applyFill: true
                };
            }
        }),
        editor : editor.extend({
            init : function (/* options */) {
                editor.prototype.init.apply(this, arguments);
                this.ignoreSelectText = true;
            },
            make : function () {
                var that = this;
                var toRet = $('<div id="pgbu-grid-cell-editor" class="cell-editor cell-editor-color"></div>');
                that.colorPicker = new oui.ColorPicker(
                        _.assign({}, this.column.colorPicker,
                            {
                                fluid : true,
                                appendTo : toRet
                            }), this.data);
                this.fromKeyboard = false;

                toRet.append(that.colorPicker.element());

                that.colorPicker.on('change', function () {
                    if (!that.fromKeyboard) {
                        that.commit({});
                    }
                });

                return toRet;
            },

            render : function () {
                this.colorPicker.init();
                this.colorPicker.show();
            },

            onKeyDown : function (event) {
                var that = this;
                switch (event.keyCode) {
                case keyCode.TAB :
                    that.onTab(event);
                    break;
                case keyCode.ENTER:
                            // Happens before change event and ends up committing same value
                    this.colorPicker.element().trigger('change');
                        /* falls through */
                case keyCode.ESCAPE:
                    editor.prototype.onKeyDown.apply(this, arguments);
                    break;
                }
            },

            getEditorInput : function () {
                return this.colorPicker.element();
            },

            getNewData : function () {
                return {
                    value : this.colorPicker.color(),
                    valid : true
                };
            }
        })
    };
}
);
