define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/constants',
    'gridapi.grid/utils/keyCode',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/headerRenderer'
], function ($, _, constants, keyCode, editor, renderer, headerRenderer) {
    'use strict';

    return {
        defaultOptions : {
            width : '75px',
            columnClass : 'number-column',
            allowAllKeys : false
        },
        headerRenderer : headerRenderer.extend({}),

        renderer : renderer.extend({
            format : function (renderOptions) {
                return renderOptions.column.formatter.format(renderOptions.data);
            },
            getExcelCell : function (renderOptions) {
                var value = this.getExcelCellData(renderOptions);
                var cell = {};

                if (!value) {
                    return cell;
                } else if (renderOptions.column.displayType === constants.COLUMN_DISPLAY_TYPE.COST || /[^0-9]/.test(value)) {
                    cell.inline = true;
                    cell.value = value;
                } else if (renderOptions.column.displayType === constants.COLUMN_DISPLAY_TYPE.PERCENT || /%/.test(value)) {
                    cell.value = parseFloat(value.replace(/(,|%)/g, '')) / 100;
                    cell.percent = true;
                    cell.applyNumberFormat = true;
                } else {
                    cell.value = value;
                }

                return cell;
            }
        }),

        editor : editor.extend({

            formatValueForInput : function () {
                if (!this.invalid) {
                    return this.column.formatter.format(this.data, { truncate : false });
                } else {
                    return this.data;
                }
            },


            getNewData : function () {
                var value = this.getEditorInput().val();
                var valid = this.validateUserInput(value);
                if (valid) {
                    value = this.column.formatter.unformat(value);
                }

                return {
                    value : value,
                    valid : valid
                };
            },

            onKeyDown : function (event) {
                        // PRIMCR-2408 - this basically indicates a global accesskey, so don't bother trying to process/preventDefault
                if (event.shiftKey && event.altKey) {
                    return;
                }
                if (this.column.allowAllKeys || this.checkKeyCode(event)) {
                    editor.prototype.onKeyDown.call(this, event); // Default behavior for all allowed keys
                } else {
                    event.preventDefault();
                }
            },

            checkKeyCode : function (event) {
                return (event.which >= keyCode.NUMROW_0 && event.which <= keyCode.NUMROW_9) || // 0-9
                            (event.which >= keyCode.NUMPAD_0 && event.which <= keyCode.NUMPAD_9) || // numpad 0-9
                            (event.which === keyCode.ENTER) ||
                            (event.which === keyCode.TAB) ||
                            (event.which === keyCode.ESCAPE) ||
                            (event.which === keyCode.COMMA) ||
                            (event.which === keyCode.LEFT) ||
                            (event.which === keyCode.RIGHT) ||
                            (event.which === keyCode.UP) ||
                            (event.which === keyCode.DOWN) ||
                            (event.which === keyCode.END) ||
                            (event.which === keyCode.HOME) ||
                            (event.which === keyCode.BACKSPACE) ||
                            (event.which === keyCode.DELETE) ||
                            (event.which === keyCode.NUMPAD_DECIMAL) ||
                            (event.which === keyCode.NUMPAD_SUBTRACT) ||
                            (event.which === keyCode.NUMPAD_ADD) ||
                            (event.which === keyCode.DASH) || // - _ negative
                            (event.which === keyCode.FF_DASH) || // - _ negative
                            (event.which === keyCode.EQUALS) || // = + positive
                            (event.which === keyCode.PERIOD) || // period
                            (event.which === keyCode.LETTER_A && event.ctrlKey) || // Allow for control a
                            (event.which === keyCode.LETTER_C && event.ctrlKey) || // Allow for control c
                            (event.which === keyCode.LETTER_X && event.ctrlKey) || // Allow for control c
                            (event.which === keyCode.LETTER_V && event.ctrlKey);// Allow for control v
            }

        }
            )
    };
});

