define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.moment',
    'gridapi.grid/utils/keyCode',
    'gridapi.grid/columns/core/editor',
    'gridapi.grid/columns/core/renderer',
    'gridapi.grid/columns/core/headerRenderer',
    'gridapi.oui'
], function ($, _, moment, keyCode, editor, renderer, headerRenderer, oui) {
    'use strict';

    var EXCEL_NON_NUMERIC_REGEX = /[^0-9]/;

    return {
        defaultOptions : {
            width : '100px',
            columnClass : 'spinner-column',
            spinnerMin : null,
            spinnerMax : null,
            spinnerStep : 1,
            spinnerType : 'number',
            allowAllKeys : false
        },
        renderer : renderer.extend({
            format : function (renderOptions) {
                return renderOptions.column.formatter.format(renderOptions.data);
            },
            getExcelCell : function (renderOptions) {
                var value = this.format(renderOptions);

                return {
                    value: value,
                    inline: EXCEL_NON_NUMERIC_REGEX.test(value)
                };
            }
        }),

        headerRenderer : headerRenderer.extend({

        }),

        editor : editor.extend({

            make : function () {
                var clonedFormatter = this.column.formatter.withOptions();
                clonedFormatter.format = _.wrap(clonedFormatter.format, function (fn, value) {
                    return fn(value, { truncate: false });
                });
                this.numField = new oui.NumberField({
                    type : this.column.spinnerType,
                    spinner : {
                        controls : true,
                        step : this.column.spinnerStep
                    },
                    min : this.column.minValue,
                    max : this.column.maxValue,
                    fluid : true,
                    formatter : clonedFormatter
                });

                var $el = this.numField.getElement();
                $el.wrap('<div class="cell-editor cell-editor-spinner"></div>');

                return $el.parent();
            },

            getNewData : function () {
                // Need to manually trigger change for the oui widgets to detect data being different
                var $editor = this.getEditorInput();
                $editor.trigger('change');
                var data = $editor.val();
                var valid = this.validateUserInput(data);
                if (valid) {
                    data = this.numField.data();
                }
                return {
                    value : data,
                    valid : valid
                };
            },

            render : function (startString) {
                var that = this;
                // initialize spinner jQuery widget
                this.numField.init();

                // set the value
                this.numField.value(this.data);
                if (startString != null) {
                    // set typed character into input
                    this.getEditorInput().val(startString);
                    this.ignoreSelectText = true;

                    // Trigger change so the number field knows this is the current value
                    this.getEditorInput().trigger('change');
                }
                // stop double click on the spinner buttons
                that.getEditor().find('.btn').on('dblclick', function (event) {
                    event.stopPropagation();
                });
            },

            drawFocusToInput : function (select, startString) {
                var $input = this.getEditorInput();
                var value;
                editor.prototype.drawFocusToInput.apply(this, arguments);
                if (startString != null) {
                    $input.val(startString);
                }
                value = $input.val();
                select = !this.ignoreSelectText && (select || this.detectSelectText());
                $input.get(0).setSelectionRange(select ? 0 : value.length, value.length);
            },

            onKeyDown : function (event) {
                // PRIMCR-2408 - this basically indicates a global accesskey, so don't bother trying to process/preventDefault
                if (event.shiftKey && event.altKey) {
                    return;
                }
                if (this.column.allowAllKeys || this.checkKeyCode(event)) {
                    editor.prototype.onKeyDown.call(this, event); // Default behavior for all allowed keys
                } else {
                    event.preventDefault();
                }
            },

            checkKeyCode : function (event) {
                if (this.column.spinnerType === 'number') {
                    return (event.keyCode >= keyCode.NUMROW_0 && event.keyCode <= keyCode.NUMROW_9) || // 0-9
                        (event.keyCode >= keyCode.NUMPAD_0 && event.keyCode <= keyCode.NUMPAD_9) || // numpad 0-9
                        (event.keyCode === keyCode.ENTER) ||
                        (event.keyCode === keyCode.TAB) ||
                        (event.keyCode === keyCode.ESCAPE) ||
                        (event.keyCode === keyCode.COMMA) ||
                        (event.keyCode === keyCode.LEFT) ||
                        (event.keyCode === keyCode.RIGHT) ||
                        (event.keyCode === keyCode.UP) ||
                        (event.keyCode === keyCode.DOWN) ||
                        (event.keyCode === keyCode.END) ||
                        (event.keyCode === keyCode.HOME) ||
                        (event.keyCode === keyCode.BACKSPACE) ||
                        (event.keyCode === keyCode.DELETE) ||
                        (event.keyCode === keyCode.NUMPAD_DECIMAL) ||
                        (event.keyCode === keyCode.NUMPAD_SUBTRACT) ||
                        (event.keyCode === keyCode.NUMPAD_ADD) ||
                        (event.keyCode === keyCode.DASH) || // - _ negative
                        (event.keyCode === keyCode.EQUALS) || // = + positive
                        (event.keyCode === keyCode.PERIOD) || // period
                        (event.keyCode === keyCode.LETTER_A && event.ctrlKey) || // Allow for control a
                        (event.keyCode === keyCode.LETTER_C && event.ctrlKey) || // Allow for control c
                        (event.keyCode === keyCode.LETTER_X && event.ctrlKey) || // Allow for control c
                        (event.keyCode === keyCode.LETTER_V && event.ctrlKey);// Allow for control v
                } else {
                    return true;
                }
            }
        })
    };
});
