define([
    'gridapi.lodash',
    'gridapi.logger',
    'gridapi.grid/utils/searchTermRegExpBuilder'
], function (_, logger, searchBuilder) {
    'use strict';

    var makeTester = function makeTester(column, value) {
        if (column.dataType === 'string' && (column.displayType === 'text' || column.displayType === 'textarea')) {
            return searchBuilder(value);
        }
        return null;
    };
    var rejectFiltersOfId = function rejectFiltersOfId(columnId) {
        this._filters = _.reject(this._filters, {
            column : {
                dataIndex : columnId
            }
        });
    };

    function ColumnMatcher(widget) {
        this.widget = widget;
        this._filters = [];
    }

    ColumnMatcher.prototype = {
        removeFilter : function (columnId) {
            var len = this._filters.length;
            rejectFiltersOfId.call(this, columnId);
            //>>excludeStart("validation", pragmas.production)
            logger.assert(len === this._filters.length + 1, 'filter wasnt removed - bad columnId/dataIndex given?');
            //>>excludeEnd("validation")
            this.widget.view.setColumnsFiltered();
        },
        setFilter : function (column, value) {
            rejectFiltersOfId.call(this, column.dataIndex);

            var tester = makeTester.call(this, column, value);
            //>>excludeStart("validation", pragmas.production)
            logger.assert(tester && tester.test, 'A valid tester fn does not exist for column ' + column.dataIndex);
            logger.assert(column && column.filterable, 'Do not attempt to filter columns that are not filterable');
            //>>excludeEnd("validation")
            this._filters.push({
                column : column,
                value : value,
                tester : tester
            });
            this.widget.view.setColumnsFiltered();
        },
        forEach : function (cb) {
            _.forEach(_.pluck(this._filters, 'column'), cb);
        },
        hasFilters : function () {
            return this._filters.length;
        },
        clear : function () {
            this._filters = [];
            this.widget.view.setColumnsFiltered();
        },
        test : function (record) {
            if (!this.hasFilters()) {
                return true;
            }
            var filters = this._filters;
            var store = this.widget.store;
            for (var i = 0, len = filters.length; i < len; ++i) {
                var filter = filters[i];
                var dataIndex = filter.column.dataIndex;
                var recordValue = store.getValueFromRecord(record, dataIndex);
                if (!filter.tester.test(recordValue)) {
                    return false;
                }
            }
            return true;
        }
    };

    return ColumnMatcher;
});
