define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.moment',
    'gridapi.logger',
    'gridapi.grid/constants'
], function ($, _, moment, logger, constants) {
    'use strict';

    function defaultCache() {
        return {
            attributes : null,
            siblingId : null,
            rollupValues : {},
            stackedRollupValues : {},
            timephasedRollupValues : {},
            childIds : [],
            parentId : null,
            childIdsByType : null,
            matchesSearch : false,
            node : null,
            recordType : null,
            rolledUp : false,
            placement : null
        };
    }
    function getRecordCache(recordId) {
        var cache = this.recordCache[recordId];
        if (cache == null) {
            cache = defaultCache();
            cache.recordId = recordId;
            this.recordCache[recordId] = cache;
        }
        return cache;
    }

    var DataCache = function (args) {
        this.widget = args.widget;
        this.resetDataCache();
    };

    DataCache.prototype = {
        constructor : DataCache,

        resetDataCache : function () {
            this.recordCache = {};
            this.recordCache[constants.DUMMY_RECORD_ID] = defaultCache();
        },

        resetRecordCache : function (recordId, args) {
            if (recordId != null && (args == null || args.clear == null || !args.clear.length)) {
                delete this.recordCache[recordId];
            } else {
                var recordCache = this.getRecordCache(recordId, args);
                var defaults = defaultCache();
                for (var i = 0, len = args.clear.length; i < len; i++) {
                    var cacheAttribute = args.clear[i];
                    recordCache[cacheAttribute] = defaults[cacheAttribute];
                }
            }
        },

        setChildren : function (args) {
            var recordCache = this.getRecordCache(args.recordId);
            recordCache.childIds = args.childIds;
            if (args.childIds && args.childIds.length) {
                recordCache.hasChildren = true;
            } else {
                recordCache.hasChildren = false;
            }
        },

        setParent : function (args) {
            var recordCache = getRecordCache.call(this, args.recordId);
            recordCache.parentId = args.parentId;
        },

        getRecordCache : function (recordId, args) {
            if (recordId == null) {
                return null;
            }
            if (args && args.fill) {
                this.getRecordAttributes(recordId, args);
                this.getRecordPlacement(recordId, args);
            }
            return getRecordCache.call(this, recordId);
        },

        getRecordAttributes : function (recordId, args) {
            var recordCache = getRecordCache.call(this, recordId);
            var doClear = args && args.clear;

            if (recordCache.attributes == null || doClear) {
                delete recordCache.attributes;
                var record = this.widget._getRecord(recordId);
                if (record != null) {
                    recordCache.attributes = this.widget._attrs.tree(record);
                    // in the event the attrs abstraction held onto the old value
                    delete recordCache.attributes.rolledUp;
                    delete recordCache.attributes.parentId;
                }
            }
            return recordCache.attributes;
        },

        getRecordPlacement : function (recordId, args) {
            var recordCache = getRecordCache.call(this, recordId);
            var doClear = args && args.clear;

            if (recordCache.placement == null || doClear) {
                delete recordCache.placement;
                delete recordCache.parentId;
                var record = this.widget._getRecord(recordId);
                if (record != null) {
                    recordCache.placement = this.widget._getRecordPlacement(record);
                    recordCache.parentId = recordCache.placement.parentId;
                }
            }
            return recordCache.placement;
        }

    };

    return DataCache;
});
