define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.grid/constants',
    'markup!gridapi.grid/templates/grid.htm:.htm-fill-down-border'
], function ($, _, keyCode, constants, cellFillBorderTemplate) {
    'use strict';

    return {
        change : function () {
            var that = this;
            return function (event) {
                var $target = $(event.target);
                var checked = false;
                var quickAdd = false;
                var args = {};
                // check box column
                var $tdCheckColumn = $target.closest('.cell-renderer-checkbox');
                if ($tdCheckColumn.length) {
                    checked = $tdCheckColumn.find('input').is(':checked');
                    var column = that.columnDefinitions[$tdCheckColumn.attr('data-index')];
                    var value = checked;
                    if (_.isFunction(column.getCheckValue)) {
                        value = column.getCheckValue(value);
                    }
                    var recordIdsToChange = that.widget.selection.recordIds;
                    _.forEach(recordIdsToChange, function (recordId) {
                        var record = that.widget._getRecord(recordId);
                        if (recordId === constants.DUMMY_RECORD_ID) {
                            record = that.widget._getProxyRecord();
                            quickAdd = true;
                        }
                        if (that.widget.canEditCell(recordId, $tdCheckColumn.attr('data-index'), that.widget._attrs.all(record), null, true) === true) {
                            args = {
                                record : record,
                                value : value,
                                valid : true,
                                column : column,
                                dataIndex : column.dataIndex
                            };
                            if (quickAdd) {
                                that.widget._newRecordCellUpdated(args);
                            } else {
                                that.widget._cellUpdated(args);
                            }
                        }
                    });
                } else {
                    // selector column
                    var $tdSelectorColumn = $target.closest('.cell-renderer-selector-checkbox');
                    if ($tdSelectorColumn.length) {
                        checked = $tdSelectorColumn.find('input').is(':checked');
                        that.widget.checkRows(that.widget.selection.recordIds, checked);
                    }
                }
            };
        },
        doubleClick : function () {
            var that = this;
            return function (event) {
                var $target = $(event.target);
                if ($target.is('.loading-film,.toolbar-item,.cell-error-indicator,a')) {
                    event.preventDefault();
                    event.stopPropagation();
                    return;
                }
                var $td = $target.closest('td');

                if ($target.hasClass('cell-selection-overlay') || $target.parent().hasClass('cell-selection-overlay')) {
                    $target.addClass('hidden');
                    var trueEl = document.elementFromPoint(event.pageX, event.pageY);
                    event.target = trueEl;
                    $(trueEl).closest('td').trigger(event);
                    $target.removeClass('hidden');
                }

                if ($td.length) {
                    // no editor for check column
                    var recordId = that.fixRecordIdType($td.parent().attr('data-id'));
                    that.widget.editRow(
                        {
                            recordId : recordId,
                            columnId : $td.attr('data-index'),
                            subRow : $td.attr('data-sub-row')
                        }
                    );
                }
            };
        },
        scroll : function () {
            return function () {
                return false;
            };
        },
        keypress : function () {
            var that = this;
            that.currentlyTyping = '';
            return function (event) {
                if (that.isLoading || that.options.disabled) {
                    return;
                }
                // If it is a shift+char or just a character
                if (!event.ctrlKey && !event.metaKey && !event.altKey &&
                    /[\u0021-\u007E\u00A1-\u02FE]/.test(String.fromCharCode(event.which))) {
                    // allow for automatic typing to occur in cell
                    if (that.widget.focus.record != null) {
                        if (!that.options.multiCellEdit) {
                            that.widget.selectCell();
                        }

                        that.currentlyTyping += String.fromCharCode(event.which);

                        // Due to the scrollTop function we need to give a break in execution as it closes the editor immediately otherwise
                        setTimeout(function () {
                            that.widget.editRow({
                                recordId : that.widget.focus.recordId,
                                columnId : that.widget.focus.columnId,
                                subRow : that.widget.focus.subRow,
                                currentlyTyping : true
                            });
                        }, 1);
                    }
                } else if (event.keyCode === keyCode.F2) { // Firefox does not trigger function keyup events
                    // F2
                    that.editSelectedCell();
                    event.stopPropagation();
                    event.preventDefault();
                }
            };
        },
        keydown : function () {
            var that = this;
            var UNDO_REDO_TIMER = 250;
            var SELECT_ALL_TIMER = 250;
            var MOVE_TIMER = 75;
            var moveLeftFn = _.throttle(function (fromTab, appendSelection) {
                that.selectionModel.cell.moveLeft(fromTab, appendSelection);
                that.resetRowHovered();
            }, MOVE_TIMER);
            var moveRightFn = _.throttle(function (fromTab, appendSelection) {
                that.selectionModel.cell.moveRight(fromTab, appendSelection);
                that.resetRowHovered();
            }, MOVE_TIMER);
            var moveUpFn = _.throttle(function (appendToSelection, moveFocusOnly) {
                that.selectionModel.cell.moveUp(appendToSelection, moveFocusOnly);
                that.resetRowHovered();
            }, MOVE_TIMER);
            var moveDownFn = _.throttle(function (appendToSelection, moveFocusOnly) {
                that.selectionModel.cell.moveDown(appendToSelection, moveFocusOnly);
                that.resetRowHovered();
            }, MOVE_TIMER);
            var moveRowLeftFn = _.throttle(function () {
                that.widget.moveRowLeft();
            }, MOVE_TIMER);
            var moveRowRightFn = _.throttle(function () {
                that.widget.moveRowRight();
            }, MOVE_TIMER);
            var selectAllFn = _.throttle(function () {
                that.widget.selectRows(
                    that.virtualRows[that.selectionModel.focus.rowsetId].records,
                    false, null, that.selectionModel.focus.columnId);
            }, SELECT_ALL_TIMER);

            var undoFn = _.throttle(function () {
                that.widget.undo();
            }, UNDO_REDO_TIMER);

            var redoFn = _.throttle(function () {
                that.widget.redo();
            }, UNDO_REDO_TIMER);

            return function (event) {
                if (that.isLoading || that.options.disabled) {
                    return;
                }
                var $td, node, expanded;
                // if in escape mode exit escape mode for any button press ignore enter as that will exit escape mode in the keyup
                if (that.escapeMode) {
                    switch (event.keyCode) {
                    case keyCode.ESCAPE:
                    case keyCode.SHIFT:
                    case keyCode.CTRL:
                    case keyCode.TAB:
                    case keyCode.F2 :
                    case keyCode.ENTER:
                        break;
                    default :
                        that.setEscapeMode(false);
                    }
                }
                if (_.includes(that.options.keybindings.disable, event.keyCode)) {
                    return;
                }
                switch (event.keyCode) {
                case keyCode.ENTER :
                case keyCode.F2 :
                    if (that.escapeMode) {
                        that.setEscapeMode(false);
                        break;
                    }
                    that.editSelectedCell();
                    event.stopPropagation();
                    event.preventDefault();
                    break;
                case keyCode.F10:
                    if (event.shiftKey) {
                        that.contextMenuKeyDown = true;
                    }
                    break;
                case keyCode.SELECT_KEY:// context menu button
                    that.contextMenuKeyDown = true;
                    break;
                case keyCode.LETTER_A:
                    if (event.ctrlKey || event.metaKey) {
                        event.preventDefault();
                        selectAllFn();
                    }
                    break;
                case keyCode.LETTER_V :// ctrl v (paste)
                    if (event.ctrlKey || event.metaKey) {
                        if (that.widget.clipboard.scope === 'cell') {
                            that.widget.pasteCells();
                        } else if (that.options.editable) {
                            that.widget.pasteRows();
                        }
                    }
                    break;
                case keyCode.LETTER_X : // ctrl x (cut)
                    if (event.ctrlKey || event.metaKey) {
                        var cutCell = that.options.shortCutLevel.cutCopyPaste === 'cell';
                        if (event.shiftKey || event.altKey) {
                            event.preventDefault();
                            cutCell = !cutCell;
                        }
                        if (cutCell) {
                            that.widget.cutCells();
                        } else if (that.options.editable) {
                            that.widget.cutRows();
                        }
                    }
                    break;
                case keyCode.LETTER_C :// ctrl c (copy)
                    if (event.ctrlKey || event.metaKey) {
                        var copyCell = that.options.shortCutLevel.cutCopyPaste === 'cell';
                        if (event.shiftKey || event.altKey) {
                            event.preventDefault();
                            copyCell = !copyCell;
                        }
                        if (copyCell) {
                            that.widget.copyCells();
                        } else if (that.options.editable) {
                            that.widget.copyRows();
                        }
                    }
                    break;
                case keyCode.TAB:
                    if (!that.escapeMode) {
                        if (event.shiftKey) {
                            moveLeftFn(true);
                        } else {
                            moveRightFn(true);
                        }
                        if (that.options.editOnClick) {
                            that.widget.editRow({
                                recordId : that.widget.focus.recordId,
                                columnId : that.widget.focus.columnId,
                                subRow : that.widget.focus.subRow
                            }
                                );
                        }
                        event.stopPropagation();
                        event.preventDefault();
                    }
                    break;
                case keyCode.RIGHT:
                    event.preventDefault();
                    if (event.ctrlKey && (that.options.tree || that.widget.groupingApplied)) {
                        if (that.widget.focus.record != null) {
                            that.widget.selectCell();
                            $td = that.selectionModel.focus.$td;
                            if ($td.is('.cell-expandable')) {
                                node = that.virtualRows[that.selectionModel.focus.rowsetId].nodes[that.selectionModel.focus.recordId];
                                if (node != null && !node.expanded) {
                                    expanded = that.toggleChildren({
                                        recordId : that.selectionModel.focus.recordId,
                                        rowsetId : that.selectionModel.focus.rowsetId,
                                        toggle : true
                                    });
                                    if (_.isBoolean(expanded)) {
                                        that.widget.requestBrowserFocus(true);// IE is losing focus somehow, added to try and fix
                                        that.widget._checkExpandCollapseState();
                                    } else {
                                        expanded.done(function () {
                                            that.widget.requestBrowserFocus(true);// IE is losing focus somehow, added to try and fix
                                            that.widget._checkExpandCollapseState();
                                        });
                                    }
                                }
                            }
                        }
                    } else if (event.altKey) {
                        moveRowRightFn();
                    } else {
                        moveRightFn(false, event.shiftKey);
                    }
                    break;
                case keyCode.LEFT:
                    event.preventDefault();
                    if (event.ctrlKey && (that.options.tree || that.widget.groupingApplied)) {
                        if (that.widget.focus.record != null) {
                            that.widget.selectCell();
                            $td = that.selectionModel.focus.$td;
                            if ($td.is('.cell-expandable')) {
                                node = that.virtualRows[that.selectionModel.focus.rowsetId].nodes[that.selectionModel.focus.recordId];
                                if (node != null && node.expanded) {
                                    expanded = that.toggleChildren({
                                        recordId : that.selectionModel.focus.recordId,
                                        rowsetId : that.selectionModel.focus.rowsetId,
                                        toggle : false
                                    });
                                    if (_.isBoolean(expanded)) {
                                        that.widget.requestBrowserFocus(true);// IE is losing focus somehow, added to try and fix
                                        that.widget._checkExpandCollapseState();
                                    } else {
                                        expanded.done(function () {
                                            that.widget.requestBrowserFocus(true);// IE is losing focus somehow, added to try and fix
                                            that.widget._checkExpandCollapseState();
                                        });
                                    }
                                }
                            }
                        }
                    } else if (event.altKey) {
                        moveRowLeftFn();
                    } else {
                        moveLeftFn(false, event.shiftKey);
                    }
                    break;
                case keyCode.UP:
                    event.preventDefault();
                    if (event.altKey || event.metaKey) {
                        that.widget.moveRowUp();
                    } else {
                        moveUpFn(event.shiftKey && that.options.multiSelect, event.ctrlKey);
                    }
                    break;
                case keyCode.DOWN:
                    event.preventDefault();
                    if (event.altKey || event.metaKey) {
                        that.widget.moveRowDown();
                    } else {
                        moveDownFn(event.shiftKey && that.options.multiSelect, event.ctrlKey);
                    }
                    break;
                case keyCode.HOME:
                    event.preventDefault();
                    break;
                case keyCode.END:
                    event.preventDefault();
                    break;
                case keyCode.PAGEUP:
                    event.preventDefault();
                    var pageUpRowset = that.virtualRows[that.selectionModel.focus.rowsetId];
                    var pageUpRecordIdIndex = pageUpRowset.renderedTop - pageUpRowset.visibleRows > 0 ?
                            pageUpRowset.renderedTop - pageUpRowset.visibleRows : 0;
                    var pageUpRecordId = pageUpRowset.records[pageUpRecordIdIndex];

                    if (event.shiftKey) {
                        that.widget.selectRows(
                                that.selectionModel.getShiftSelection(that.selectionModel.focus.rowsetId, pageUpRecordId),
                                false, null, that.selectionModel.focus.columnId);
                        if (that.renderFocus(pageUpRecordId,
                                that.selectionModel.focus.rowsetId,
                                that.selectionModel.focus.columnId, 'top')) {
                            that.updateRowsetScrollbar(that.selectionModel.focus.rowsetId);
                        }
                    } else if (that.renderRecord(pageUpRecordId,
                                    that.selectionModel.focus.rowsetId,
                                    'top', pageUpRecordIdIndex)) {
                        that.updateRowsetScrollbar(that.selectionModel.focus.rowsetId);
                    }
                    break;
                case keyCode.PAGEDOWN:
                    event.preventDefault();
                    var pageDownRowset = that.virtualRows[that.selectionModel.focus.rowsetId];
                    var pageDownRecordIdIndex = pageDownRowset.renderedBottom + pageDownRowset.visibleRows < pageDownRowset.records.length - 1 ?
                            pageDownRowset.renderedBottom + pageDownRowset.visibleRows : pageDownRowset.records.length - 1;
                    var pageDownRecordId = pageDownRowset.records[pageDownRecordIdIndex];

                    if (event.shiftKey) {
                        that.widget.selectRows(
                                that.selectionModel.getShiftSelection(that.selectionModel.focus.rowsetId, pageDownRecordId),
                                false, null, that.selectionModel.focus.columnId);
                        if (that.renderFocus(pageDownRecordId,
                                    that.selectionModel.focus.rowsetId,
                                    that.selectionModel.focus.columnId, 'bottom')) {
                            that.updateRowsetScrollbar(that.selectionModel.focus.rowsetId);
                        }
                    } else if (that.renderRecord(pageDownRecordId,
                                    that.selectionModel.focus.rowsetId,
                                    'bottom', pageDownRecordIdIndex)) {
                        that.updateRowsetScrollbar(that.selectionModel.focus.rowsetId);
                    }
                    break;
                case keyCode.ESCAPE:
                    // stop focus moving away from table
                    event.preventDefault();
                    var cancelledMenus = that.closeMenus();
                    var cancelledClipboard = that.clearClipboardStyles();
                    if (!cancelledMenus && !cancelledClipboard) {
                        that.setEscapeMode(!that.escapeMode);
                    }
                    break;
                case keyCode.SPACE:
                    event.preventDefault();
                    break;
                case keyCode.LETTER_Y :
                        // ctrl y (redo)
                    if (event.ctrlKey) {
                        redoFn();
                    }
                    break;
                case keyCode.LETTER_Z :
                        // ctrl z (undo)
                    if (event.ctrlKey) {
                        if (event.shiftKey) {
                            redoFn();
                        } else {
                            undoFn();
                        }
                    }
                    break;
                case keyCode.BACKSPACE: // Prevent browser from going back.
                    event.preventDefault();
                    event.stopPropagation();
                    break;
                case keyCode.NUMPAD_ADD:
                case keyCode.EQUALS:
                case keyCode.FF_EQUALS:
                    if (event.keyCode === keyCode.NUMPAD_ADD || event.shiftKey) {
                        var groupId = that.selectionModel.focus.groupId;
                        var group = that._getGroup(groupId);
                        that.openColumnGroupMenu(group, true);
                    }
                    break;
                }
            };
        },
        keyup : function () {
            var that = this;
            return function (event) {
                if (that.isLoading || that.options.disabled) {
                    return;
                }
                var $td = $();
                switch (event.keyCode) {
                case keyCode.INSERT:
                    var customOpts;

                    if (event.altKey) {
                        customOpts = that.options.shortcuts.insert.secondary;
                    } else {
                        customOpts = that.options.shortcuts.insert.primary;
                    }

                    var insertOpts = _.defaults({}, _.isFunction(customOpts) ? customOpts() : customOpts, {
                        skipEdit : true,
                        isChild : event.ctrlKey || event.metaKey
                    });

                    that.widget.newRow(insertOpts);
                    break;
                case keyCode.SELECT_KEY:// context menu button
                    that.contextMenuKeyDown = false;
                    break;
                case keyCode.BACKSPACE:
                case keyCode.DELETE :
                    if (!(event.ctrlKey || event.altKey)) {
                        var deleteCell = that.options.shortCutLevel.delete === 'cell';
                        if (event.shiftKey || event.altKey) {
                            deleteCell = !deleteCell;
                        }
                        if (deleteCell) {
                            that.widget.deleteCells();
                        } else {
                            that.widget.deleteRows();
                        }
                    }
                    break;

                case keyCode.SPACE :
                    var triggerCheckbox = function ($cell) {
                        var $checkBox = $cell.find('input[type="checkbox"]');
                        if ($checkBox.length && $cell.length) {
                            if (!$checkBox.is(':disabled')) {
                                var isChecked = $checkBox.is(':checked');
                                if (isChecked) {
                                    $checkBox.prop('checked', false);
                                } else {
                                    $checkBox.prop('checked', true);
                                }
                                $checkBox.trigger('change');
                            }
                            return true;
                        }
                        return false;
                    };
                    if (that.selectionModel.focus.rowsetId === 'headers') {
                        if (that.widget._trigger('header-action', event, { dataIndex : that.selectionModel.focus.columnId }) !== false) {
                            $td = that.selectionModel.focus.$td;
                            if (!triggerCheckbox($td)) { // First try and trigger the checkbox. if there is no checkbox sort
                                var columnMenuOpened = that.openColumnMenu(that.selectionModel.focus.$td, event, true);
                                if (!columnMenuOpened) {
                                    that.reselectHeaderAfterSort = true;
                                    that.widget.sortRows(that.selectionModel.focus.columnId, event.ctrlKey);
                                    that.recalculateHeaderHeight = true;
                                    that.widget.resize();
                                }
                                event.stopPropagation();
                                event.preventDefault();
                            }
                        }
                    } else {
                        if (that.selectionModel.focus.columnId === 'menu-button-column') {
                            if (that.selectionModel.focus.recordId === constants.DUMMY_RECORD_ID) {
                                that.widget._commitNewRecord({});
                            } else {
                                that.openContextMenu({ record : that.widget.focus.record, fromKeyboard : true });
                            }
                            return;
                        }
                        if (event.ctrlKey || event.altKey) {
                            var selected = that.widget.getSelectionIds();
                            var rowsToSelect = [that.widget.focus.recordId];
                            if (!_.includes(selected, that.widget.focus.recordId)) {
                                that.widget.selectRows(rowsToSelect, that.options.multiSelect);
                            } else {
                                that.widget.deselectRows(rowsToSelect);
                            }
                        } else {
                                // stop focus moving away from table
                            event.preventDefault();
                            event.preventDefault();
                            if (that.widget.focus.record != null) {
                                $td = that.selectionModel.focus.$td;

                                if ($td.length === 1) {
                                    if (!triggerCheckbox($td)) { // First try and trigger the checkbox. if there is no checkbox do the other action
                                        if ($td.is('.cell-invalid')) {
                                            if (!that.$errorBox.parent().length) {
                                                var showCellErrorIndex = that.selectionModel.focus.columnId;
                                                if (that.selectionModel.focus.subRow != null) {
                                                    showCellErrorIndex += '$$' + that.selectionModel.focus.subRow;
                                                }
                                                that.showCellErrors(that.selectionModel.focus.recordId, showCellErrorIndex);
                                            } else {
                                                that.closeMenus();
                                            }
                                        } else {
                                                // All data triggers will use these args
                                            var args = {
                                                record : that.widget.focus.record,
                                                dataIndex : $td.attr('data-index')
                                            };

                                                // Find all data-triggers and fire them.
                                            _.forEach($td.find('[data-trigger]'), function (el) {
                                                var $el = $(el);
                                                that.widget._trigger($el.attr('data-trigger'), event, args);
                                            });
                                            var clickableLinks = $td.find('a[href]');
                                            if (clickableLinks.length) {
                                                clickableLinks[0].click();
                                            }

                                            if (that.widget.options.selectionMode === 'check' && (!clickableLinks.length && !$td.find('[data-trigger]').length)) {
                                                var checked = _.includes(that.widget.checkedRecordIds, that.widget.focus.recordId);
                                                that.widget.checkRows(that.widget.selection.recordIds, !checked);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    break;
                case keyCode.END :
                    if (event.ctrlKey) {
                        that.selectionModel.cell.lastCellInGrid();
                    } else {
                        that.selectionModel.cell.lastCellInRow();
                    }

                    break;
                case keyCode.HOME :
                    if (event.ctrlKey) {
                        that.selectionModel.cell.firstCellInGrid();
                    } else {
                        that.selectionModel.cell.firstCellInRow();
                    }
                    break;
                case keyCode.ESCAPE :
                    that.closeEditor();

                    event.stopPropagation();
                    event.preventDefault();
                    break;
                case keyCode.UP :
                case keyCode.DOWN :
                case keyCode.LEFT :
                case keyCode.RIGHT :
                    event.stopPropagation();
                    event.preventDefault();
                    break;

                }
            };
        },

        contextMenu : function () {
            var that = this;
            return function (event) {
                if (that.escapeMode) { // Do not do anything if the grid is in escape mode.
                    event.preventDefault();
                    event.stopPropagation();
                    return;
                }
                var $target = $(event.target);

                if (!that.contextMenuKeyDown && $target.is('a,.viewport,.pseudo-scroller')) { // If it is an anchor allow them to right click to open in new tab.
                    return;
                }

                // if the $target area is scrolly-grid then it is a keyboard event that was fired
                if (that.contextMenuKeyDown) {
                    if (that.selectionModel.focus.$td.is('th')) {
                        that.openColumnMenu(that.selectionModel.focus.$td, event, true);
                    } else if (that.widget.focus.record != null) {
                        that.openContextMenu({
                            record : that.widget.focus.recordId != null ? that.widget._getRecord(that.widget.focus.recordId) : null,
                            openedFromCell : true,
                            dataIndex : that.selectionModel.focus.columnId,
                            $relativeElement : that.selectionModel.focus.$td,
                            fromKeyboard : true
                        });
                    }
                } else {
                    var $cell = $target.closest('td');
                    that.selectionModel.cell.cellSelected($cell, {
                        append : $cell.closest('tr').is('.' + that.options.classes.rowSelected),
                        event : event
                    });
                    that.openContextMenu(
                        {
                            record : that.widget.focus.recordId != null ? that.widget._getRecord(that.widget.focus.recordId) : null,
                            dataIndex : that.selectionModel.focus.columnId,
                            openedFromCell : true,
                            $relativeElement : $cell,
                            fromKeyboard : false
                        });
                }
                // Only if there is a menu should we stop the inspector, nicer for development.
                if (that.contextMenu.isOpen) {
                    event.stopPropagation();
                    event.preventDefault();
                }
            };
        },

        clickTrigger : function () {
            var that = this;

            return function (event) {
                var $target = $(this);
                var recordId;
                var record = null;

                recordId = that.fixRecordIdType($target.parents('tr').first().attr('data-id'));
                if (recordId != null) {
                    record = that.widget._getRecord(recordId);
                }

                if ($target.is('[data-trigger]')) {
                    var args = {
                        record : record,
                        requestNewTab : event.which === 2 // if from middle click set this
                    };

                    args.dataIndex = $target.parents('td').first().attr('data-index');
                    that.widget._trigger($target.attr('data-trigger'), event, args);
                }

                if ($target.is('[data-grid-trigger]')) {
                    var action = $target.attr('data-grid-trigger');
                    switch (action) {
                    case 'newRow' :
                    case 'newChildRow':
                        that.widget[action].call(that.widget, { focus : record });
                        break;
                    case 'deleteRows' :
                        that.widget[action].call(that.widget, { records : [record] });
                        break;
                    }
                }
            };
        },

        clickMenu : function () {
            var that = this;
            return function () {
                var $target = $(this);
                if ($target.parents('tr').is('.row-menu-open')) {
                    that.contextMenu.close();
                } else {
                    that.openContextMenu({ recordId : that.fixRecordIdType($target.parents('tr').first().attr('data-id')) });
                }
            };
        },
        mousemove : function () {
            var that = this;
            that.$currentHoveredVirtualRow = $();
            return _.throttle(
                function (event) {
                    if (that.contextMenu == null || !that.contextMenu.isOpen) {
                        var $target = $(event.target);
                        var $closestTr = $target.closest('tr');

                        // find the thing and make it do this as well.
                        if ($closestTr.parent('tbody').length && $closestTr.length) {
                            var recordId = $closestTr.attr('data-id');
                            if (recordId !== that.currentHoverId) {
                                that.currentHoverId = recordId;
                                that.$currentHoveredVirtualRow.removeClass(that.options.classes.rowHovered);
                                if (recordId != null) {
                                    that.$currentHoveredVirtualRow = that.buildVirtualRow(that.fixRecordIdType(recordId), $closestTr.attr('data-rowset-id'));
                                    that.$currentHoveredVirtualRow.addClass(that.options.classes.rowHovered);
                                    that.$grid.addClass('grid-state-row-hovered');
                                }
                            }
                        } else {
                            that.$currentHoveredVirtualRow.removeClass(that.options.classes.rowHovered);
                            that.$grid.removeClass('grid-state-row-hovered');
                            that.currentHoverId = null;
                        }
                    }
                    event.stopPropagation();
                    event.preventDefault();
                }, 50);
        },

        mouseleave : function () { // When the mouse leaves the grid
            var that = this;
            return function (event) {
                _.delay(function () {
                    if (that.contextMenu == null || !that.contextMenu.isOpen) {
                        that.resetRowHovered();
                    }
                }, 60);
            };
        },

        mouseenter : function () {
            var that = this;
            return function (event) {
                that.$currentHoveredVirtualRow.removeClass(that.options.classes.rowHovered);
                that.$currentHoveredVirtualRow = $();
                that.currentHoverId = null;
            };
        },

        mouseDownCellDrag : function () {
            var that = this;

            return function (event) {
                var dragHandle = $(this);
                var cellSelection = dragHandle.parent();
                var $viewPort = cellSelection.closest('.viewport');

                var rows = 0;

                var focusedRecord = that.widget.focus.record;
                var allRecords = that.virtualRows[that.widget._getRecordPlacement(focusedRecord).rowsetId].records;
                var pseudoScroller = that.pseudoScrollers.vertical[that.widget._getRecordPlacement(focusedRecord).rowsetId].$scroller;
                var pseudoScrollerTop = pseudoScroller.offset().top;
                var focusedIndex = _.indexOf(allRecords, that.widget._getRecordId(focusedRecord));
                var selectedIds = that.widget.selection.recordIds;

                var selectionBorder = $(cellFillBorderTemplate());

                var origTop = cellSelection.position().top + that.options.layout.rowHeight;

                // Records we will be copying to
                var records = [];

                // Keep mousedown event from propagating. Only this delegated event handler should be used for processing that
                // mousedown.
                event.stopPropagation();

                // Start drag operation by appending the selection element to the DOM
                $viewPort.find('.viewport-overlay').append(selectionBorder);

                // Give the selection element proper positioning and dimensions
                selectionBorder.css({
                    width : cellSelection.outerWidth(),
                    height : that.options.layout.rowHeight,
                    top : origTop,
                    left : cellSelection.position().left
                });
                var scrollFn = function (amount) {
                    pseudoScroller.scrollTop(pseudoScroller.scrollTop() + amount);
                };

                var scrollTimer;

                // Put on temporary namespaced (mousemove, leave and up) events on the document for listening to the mouse
                // anywhere on the document
                var filldownDragFn = _.throttle(function (event) {
                    if (scrollTimer != null) {
                        clearTimeout(scrollTimer);
                        scrollTimer = null;
                    }
                    var newTop = event.clientY - pseudoScrollerTop + pseudoScroller.scrollTop();
                    var top, height;

                    if (newTop < origTop) {
                        top = Math.floor(newTop / that.options.layout.rowHeight) * that.options.layout.rowHeight;
                        height = origTop - top - that.options.layout.rowHeight;
                        selectionBorder.addClass('reverse');
                        rows = -(height / that.options.layout.rowHeight);
                        // scroll and update if its above
                        if (event.clientY < pseudoScrollerTop + that.options.layout.rowHeight) {
                            scrollTimer = setInterval(function () {
                                scrollFn(-15);
                                newTop = event.clientY - pseudoScrollerTop + pseudoScroller.scrollTop();
                                top = Math.floor(newTop / that.options.layout.rowHeight) * that.options.layout.rowHeight;
                                height = origTop - top - that.options.layout.rowHeight;
                                rows = -(height / that.options.layout.rowHeight);
                                selectionBorder.css({
                                    top : top,
                                    height : height
                                });
                            }, 100);
                        }
                    } else {
                        top = origTop;
                        height = Math.ceil((newTop - origTop) / that.options.layout.rowHeight) * that.options.layout.rowHeight;
                        selectionBorder.removeClass('reverse');
                        rows = height / that.options.layout.rowHeight + 1;
                        // Scroll and update if its below
                        if (event.clientY > pseudoScrollerTop + pseudoScroller.height() - that.options.layout.rowHeight) {
                            scrollTimer = setInterval(function () {
                                scrollFn(15);
                                height = Math.ceil((event.clientY - pseudoScrollerTop + pseudoScroller.scrollTop() - origTop) / that.options.layout.rowHeight) * that.options.layout.rowHeight;
                                rows = height / that.options.layout.rowHeight + 1;
                                selectionBorder.css({
                                    top : top,
                                    height : height
                                });
                            }, 100);
                        }
                    }

                    // Update border with what would be affected by the DONE/up operation
                    selectionBorder.css({
                        top : top,
                        height : height
                    });
                }, 50);
                $(document).on('mousemove.fillDownDrag', filldownDragFn);

                $(document).on('mouseleave.fillDownDrag mouseup.fillDownDrag', function (event) {
                    // clear the scrolling timeout
                    if (scrollTimer != null) {
                        clearTimeout(scrollTimer);
                        scrollTimer = null;
                    }
                    // Clean up namespaced events and remove the selection element
                    $(document).off('.fillDownDrag');
                    selectionBorder.remove();

                    // Get the records we will be copying values to
                    var spliced = allRecords.slice(Math.min(focusedIndex, focusedIndex + rows), Math.max(focusedIndex + rows, focusedIndex));
                    spliced = _.difference(spliced, selectedIds);

                    records = _.map(spliced, function (id) {
                        return that.widget._getRecord(id);
                    });

                    if (records.length > 0) {
                        that.selectionModel.setPivotIndex();
                        var shouldCopy = that.widget._trigger('before-filldown', event, {
                            record : focusedRecord,
                            dataIndex : that.selectionModel.focus.columnId,
                            recordsToFill : records
                        }) !== false;
                        if (shouldCopy) {
                            // Copy and then paste the values to the remaining records
                            that.widget.copyCells();
                            that.widget.pasteCells({
                                records : records,
                                appendToSelection : true,
                                strictSelection : true
                            });
                        }
                        that.widget.selectRows(that.getRecordIds(records), true);
                        records.push(focusedRecord);
                        var lastRecord = _.last(that.orderByViewIndex(records));
                        that.selectionModel.cell._updateFocusModelFromIds(that.widget._getRecordId(lastRecord), that.widget.focus.columnId);
                        that.widget.selectCell(that.widget._getRecordId(lastRecord), that.widget.focus.columnId, 'bottom', true);
                    }
                });
            };
        },

        mousedown : function () {
            var that = this;

            return function (event) {
                var $target = $(event.target);
                that.setEscapeMode(false, true);

                // fire only on left click
                if (event.which === 1) { // left click only, right click selection handled in context menu
                    if (!$target.is('.input')) {
                        var $td = $target.closest('td');

                        if ($target.is('.toolbar-item') || $target.parents().is('.menu-button-cell') || $target.is('.pseudo-scroller')) {
                            that.widget.requestBrowserFocus(true);
                            event.preventDefault();
                            return;
                        }
                        if ($td.length) {
                            if (!$target.is('th')) {
                                that.rowDragTimeout = setTimeout(function () {
                                    if (that.canDragRows()) {
                                        var $tr = $td.parents('tr').first();
                                        if ($tr.length && $tr.is('.draggable')) {
                                            that.rowMover.initMoveRow($td.closest('div.grid-rowset'), $td.parent(), event.clientY);
                                            var throttled =  _.throttle(that.rowMover.mousemoveFn(60), 20);
                                            var mouseupFn = that.rowMover.mouseupFn(throttled);

                                            that.$grid.on('mousemove.row-move', throttled);
                                            $(document.body).on('mouseup.row-move', mouseupFn);
                                        }
                                    }
                                }, 200);

                                $(document.body).on('mouseup.grid-row-drag', function () {
                                    clearTimeout(that.rowDragTimeout);
                                    $(document.body).off('mouseup.grid-row-drag');
                                });
                                event.preventDefault();
                            }

                            var recordId;
                            // if not already editing
                            if (!$td.is('.editing') && !$target.is('.cell-grabber')) {
                                // select the cell and row
                                var $tr = $td.parent();
                                recordId = that.fixRecordIdType($tr.attr('data-id'));
                                if ((!event.shiftKey && !event.ctrlKey) || !that.options.multiSelect) { // If regular click or no multi-select
                                    var appendSelection = that.options.multiSelect &&
                                        $td.closest('tr').is('.row-selected') && !that.$grid.is(':focus');
                                    that.selectionModel.cell.cellSelected($td, {
                                        append : appendSelection,
                                        event : event
                                    });
                                } else {
                                    var rowsToSelect = [];
                                    rowsToSelect.push(recordId);
                                    if (event.ctrlKey && $tr.is('.' + that.options.classes.rowSelected)) { // If unselecting a row with control
                                        that.widget.deselectRows(rowsToSelect, event);
                                        if (that.options.selectionMode === 'check' && that.options.checkOnSelect && !$target.is('input')) {
                                            that.widget.checkRows(rowsToSelect, false);
                                        }
                                    } else {
                                        if (event.shiftKey) { // Shift add
                                            var viewport = $tr.closest('div.viewport').data().viewport;
                                            rowsToSelect = that.selectionModel.getShiftSelection(viewport.rowset, recordId);
                                            that.widget.selectRows(rowsToSelect, false, null, that.selectionModel.focus.columnId);
                                            that.selectionModel.cell._updateFocusModel($td);
                                        }
                                        that.selectionModel.cell.cellSelected($td, {
                                            append : true,
                                            skipPivotReset : event.shiftKey,
                                            event : event
                                        }); // Fix focus for shift and append control selection


                                        if (that.options.selectionMode === 'check' && that.options.checkOnSelect && !$target.is('input')) {
                                            that.widget.checkRows(rowsToSelect, true);
                                        }
                                    }
                                }
                            }
                            if ($target.is('.toggle-children') || $target.parents('.toggle-children').length) {
                                var $closestTr = $target.parentsUntil('tbody', 'tr');
                                recordId = that.fixRecordIdType($closestTr.attr('data-id'));

                                var rowsetId = $closestTr.attr('data-rowset-id');
                                var expanded = that.toggleChildren({
                                    recordId : recordId,
                                    rowsetId : rowsetId,
                                    toggle : !that.virtualRows[rowsetId].nodes[recordId].expanded
                                });
                                if (_.isBoolean(expanded)) {
                                    that.widget.requestBrowserFocus(true);// IE is losing focus somehow, added to try and fix
                                    // that.widget._checkExpandCollapseState();
                                } else {
                                    expanded.done(function () {
                                        that.widget.requestBrowserFocus(true);// IE is losing focus somehow, added to try and fix
                                        // that.widget._checkExpandCollapseState();
                                    });
                                }
                            }

                            if ($target.is('.cell-error-indicator')) {
                                var dataIndex = $td.attr('data-index');
                                recordId = that.fixRecordIdType($td.closest('tr').attr('data-id'));
                                var subRow = $td.attr('data-sub-row');
                                if (subRow != null) {
                                    dataIndex = dataIndex + '$$' + subRow;
                                }
                                that.showCellErrors(recordId, dataIndex, $target);
                                return;
                            }
                            if (that.options.editOnClick) {
                                var $tdCheckColumn = $td.find('div.cell-renderer-checkbox');
                                if (!$tdCheckColumn.length) {
                                    // no editor for check column
                                    recordId = that.fixRecordIdType($td.parent().attr('data-id'));

                                    that.widget.editRow(
                                        {
                                            recordId : recordId,
                                            columnId : $td.attr('data-index'),
                                            subRow : $td.attr('data-sub-row')
                                        });
                                    event.stopPropagation();
                                    event.preventDefault();
                                }
                            }
                        } else if (document.activeElement === that.$grid[0]) {
                            // IE - Keep focus where it is.
                            event.preventDefault();
                        }
                        // close open context/column menu
                        that.closeMenus();
                    }
                    if (that.options.inModal === true) { // ojDialog tries to focus itself on mousedown which breaks grid eventHandlers.
                        event.stopPropagation();
                    }
                }
            };
        }
    };
});
