define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils'
], function ($, _, utils) {
    'use strict';
    // Calculate the group widths by summing up all the visible column widths
    function calculateGroupColumnWidths(group, gridWidth, grid) {
        var groupCalcDeduction = 0;
        if (grid.options.contextMenu.enabled && grid.options.contextMenu.enableRowMenuButton) {
            groupCalcDeduction = grid.options.contextMenu.rowMenuButtonWidth;
        }
        calculateColumnWidthFromPercent(group, gridWidth - groupCalcDeduction, grid);
        var combinedColumnWidth = 0;
        _.forEach(_.filter(group.columns, function (column) {
            return column.visible;
        }), function (column) {
            column.positionLeft = combinedColumnWidth;
            column.renderedWidth = parseInt(column.width, 10);
            combinedColumnWidth += column.renderedWidth;
        });
        group.combinedColumnWidth = combinedColumnWidth;
        return combinedColumnWidth;
    }

    function calculateColumnWidthFromPercent(group, gridWidth, grid) {
        _.forEach(group.columns, function (column) {
            if (column.percent != null) {
                var newWidth = Math.floor((parseFloat(column.percent) / 100) * gridWidth) + 'px';
                if (newWidth !== column.width) {
                    grid.viewports['$group-' + group.id].find('th[data-index="' + column.dataIndex + '"]').attr('width', newWidth);
                    column.width = newWidth;
                    column.renderedWidth = newWidth;
                }
            }
        });
    }

    /*
     * Called when group resizing occurs from dragging the blue line. This will
     * readjust the widths
     */
    function changeGroupSize(groups, groupToChange, newWidth) {
        var relativeGroup = null;
        var groupIdx = -1;
        var relativeWidth;
        var totalWidth;

        _.forEach(groups, function (group, idx) {
            if (group.type === 'flex') {
                if (group.id === groupToChange.id) {
                    groupIdx = idx;
                }
                if (groupIdx !== -1) {
                    relativeGroup = group;
                }
            }
        });

        if (groupToChange.combinedColumnWidth <= newWidth) {
            newWidth = groupToChange.combinedColumnWidth;
        } else if (newWidth < 25) {
            newWidth = 25;
        }

        totalWidth = groupToChange.width + relativeGroup.width;
        relativeWidth = totalWidth - newWidth;
        if (relativeWidth >= relativeGroup.combinedColumnWidth) {
            relativeWidth = relativeGroup.combinedColumnWidth;
        } else if (relativeWidth < 25) {
            relativeWidth = 25;
        }
        newWidth = totalWidth - relativeWidth;
        groupToChange.width = newWidth;
        relativeGroup.width = relativeWidth;
    }

    function changeRowsetSize(rowsets, rowsetToChange, newHeight) {
        var heightChange = newHeight - rowsetToChange.height;
        rowsetToChange.height = newHeight;
        var flexRowsets = _.filter(rowsets, function (rowset) {
            return rowset.type === 'flex' && rowset.id !== rowsetToChange.id;
        });
        var totalHeight = 0;
        _.forEach(_.pluck(flexRowsets, 'height'), function (height) {
            totalHeight += height;
        });
        _.forEach(flexRowsets, function (rowset) {
            var percent = rowset.height / totalHeight;
            rowset.height -= parseInt(heightChange * percent, 10);
        });
    }

    function getRowsetHeight(virtualRows, ROW_HEIGHT) {
        return (virtualRows.records.length + (virtualRows.headerRow ? 1 : 0)) * ROW_HEIGHT;
    }

    return {


        resizeRowsetsFn : function (grid, requiresVerticalShim) {
            var usedHeight = 0;
            if (requiresVerticalShim && grid.options.statusBar.enabled) {
                grid.$statusBar.css('margin-right', grid.options.scrollbar.extraWidth);
            }
            _.forEach(grid.config.rowsets, function (rowset) {
                if (!_.isNull(rowset.height) && !_.isUndefined(rowset.height)) {
                    var originalVisibleRows = grid.virtualRows[rowset.id].visibleRows;
                    var hasHeaderRow = (grid.virtualRows[rowset.id].headerRow ? 1 : 0);   // LUX-266 Part 2 & 3
                    var $rowset = grid.$rowsets[rowset.id];
                    $rowset.css('height', rowset.height);
                    if (requiresVerticalShim && rowset.type !== 'manual') {
                        $rowset.css('margin-right', grid.options.scrollbar.extraWidth);
                    } else {
                        $rowset.css('margin-right', '0');
                    }
                    if (rowset.type === 'flex') {
                        var resizer = grid.resizers.rowset[rowset.id];
                        if (resizer != null) {
                            resizer.$selector.css('top', usedHeight + rowset.height - 2); // 2 pixel margin for larger hover target
                        }
                        var pseudoScroller = grid.pseudoScrollers.vertical[rowset.id];
                        if (pseudoScroller.originalScroller != null) {
                            pseudoScroller.originalScroller.$scroller.css('height', rowset.height);
                            pseudoScroller.originalScroller.$scroller.css('top', usedHeight);
                            pseudoScroller.originalScroller.$content.css('height', rowset.contentHeight + 10);
                        }
                        pseudoScroller.$scroller.css('height', rowset.height - (hasHeaderRow ? grid.ROW_HEIGHT : 0));  // LUX-266 Part 3
                        pseudoScroller.$scroller.css('top', usedHeight + (hasHeaderRow ? grid.ROW_HEIGHT : 0));  // LUX-266 Part 3

                        pseudoScroller.$content.css('height', rowset.contentHeight + 10);
                        pseudoScroller.$scroller.scrollTop(grid.virtualRows[rowset.id].displayTop * grid.ROW_HEIGHT);
                        grid.widget._trigger('view-sync-content-height-sync', null, { newHeight : rowset.contentHeight + 10 });
                    }
                    usedHeight += rowset.height;
                    var newVisibleRows = Math.floor(rowset.height / grid.ROW_HEIGHT) - hasHeaderRow;  // // LUX-266 Part 2
                    grid.virtualRows[rowset.id].visibleRows = (newVisibleRows || 1);
                    if (originalVisibleRows !== newVisibleRows) {
                        grid.renderRowset(rowset.id);
                    }
                }
            });
        },

        resizeGroupsFn : function (grid, gridHeight) {
            var usedWidth = 0;
            _.forEach(grid.config.groups, function (group) {
                if (!_.isNull(group.width) && !_.isUndefined(group.width)) {
                    grid.viewports['$group-' + group.id].css('width', group.width).css('left', usedWidth);
                    // Attempt to fix issue where an empty group is set. The borders will cause things to look weird and not fit
                    if (group.width === 0) {
                        grid.viewports['$group-' + group.id].css('display', 'none');
                        group.forcedHidden = true;
                    } else if (group.forcedHidden) {
                        grid.viewports['$group-' + group.id].css('display', 'inline-block');
                        group.forcedHidden = false;
                    }
                    if (group.type === 'flex') {
                        var resizer = grid.resizers.group[group.id];
                        if (resizer != null) {
                            resizer.$selector.css('left', usedWidth + group.width - 2); // 2 pixel margin for larger hover target
                        }

                        var pseudoScroller = grid.pseudoScrollers.horizontal[group.id];
                        if (pseudoScroller != null) {
                            pseudoScroller.$scroller.css('width', group.width);
                            grid.viewports['$group-' + group.id].find('.viewport-overlay').css('width', group.combinedColumnWidth);
                            pseudoScroller.$scroller.css('left', usedWidth);
                            pseudoScroller.$content.css('width', group.combinedColumnWidth);
                            pseudoScroller.$scroller.css('top', gridHeight - grid.options.scrollbar.extraHeight - 1); // scrollbar height -  1 for the pseudoscroller
                        }
                    }
                    usedWidth += group.width;
                }
            });
        },

        calculateRowsetHeights : function (grid, gridHeight, requiresShim) {
            var fixedHeightSpace = 0,
                requiresVerticalShim = false;

            if (requiresShim) {
                gridHeight -= grid.options.scrollbar.extraHeight;
            }

            // Calculate the space required for the header and any fixed rowsets
            var fixedRowsets = _.filter(grid.config.rowsets, function (rowset) {
                return rowset.type === 'fixed' || rowset.type === 'manual';
            });

            _.forEach(fixedRowsets, function (rowset) {
                var height = 0;
                if (rowset.height != null && !rowset.squash) {
                    if (isNaN(rowset.height)) {
                        if (rowset.height.indexOf('%') === -1) {
                            height = parseInt(rowset.height, 10);
                        } else {
                            rowset.recalculateHeight = true;
                            rowset.heightPercent = (parseFloat(rowset.height) / 100);
                        }
                    } else {
                        height = rowset.height;
                    }
                } else {
                    rowset.recalculateHeight = true;
                }

                if (rowset.recalculateHeight) {
                    if (rowset.heightPercent == null) {
                        height = getRowsetHeight(grid.virtualRows[rowset.id], grid.ROW_HEIGHT);
                        if (height > 0) {
                            height += 2; // account for borders for fixed rows.
                        }
                    } else {
                        height = Math.floor(gridHeight * rowset.heightPercent);
                    }
                }
                rowset.height = height;
                fixedHeightSpace += height;
            });


            // Calculate the proper percentages for the flexible rowsets
            var flexRowsets = _.filter(grid.config.rowsets, function (rowset) {
                if (rowset.squash) {
                    rowset.contentHeight = getRowsetHeight(grid.virtualRows[rowset.id], grid.ROW_HEIGHT);
                }
                // exclude squashed rowsets that have been set to their correct sizes, this is for when it recalcs due to chagning scrollshims
                return rowset.type === 'flex';
            });

            var remainingHeight = (gridHeight - fixedHeightSpace);
            var allocatedSpace = 0;
            var flexRowsetHeight = 0;
            _.forEach(flexRowsets, function (rowset) {
                var rowsetHeight = 0;
                if (rowset.height != null) {
                    if (isNaN(rowset.height)) {
                        if (rowset.height.indexOf('%') === -1) {
                            rowset.height = parseInt(rowset.height, 10);
                        } else {
                            rowset.height = Math.floor(remainingHeight * (parseFloat(rowset.height) / 100));
                        }
                    }
                    rowsetHeight = rowset.height;
                }

                flexRowsetHeight += rowsetHeight;
            }
            );

            var percentPerRowset = 1.0 / flexRowsets.length;
            _.forEach(flexRowsets, function (rowset, index) {
                var newHeight;
                if (rowset.height == null) {
                    newHeight = parseInt(percentPerRowset * remainingHeight, 10);
                } else {
                    if (rowset.squashed) {
                        newHeight = rowset.height;
                    } else {
                        newHeight = Math.round((rowset.height / flexRowsetHeight) * remainingHeight);
                    }
                    if (isNaN(newHeight)) {
                        newHeight = 0;
                    }
                }
                var contentHeight = getRowsetHeight(grid.virtualRows[rowset.id], grid.ROW_HEIGHT);
                if (rowset.squash && newHeight > contentHeight) {
                    if (contentHeight !== 0) {
                        newHeight = contentHeight;
                        rowset.squashed = true;
                    }
                }
                if (contentHeight > newHeight) {
                    requiresVerticalShim = true;
                }
                allocatedSpace += newHeight;
                // Take up additional space if it is not told to squash
                if (index + 1 === flexRowsets.length && (rowset.squash == null || !rowset.squash)) {
                    newHeight += (remainingHeight - allocatedSpace);
                }
                rowset.height = newHeight;
                rowset.contentHeight = contentHeight;
            });
            return requiresVerticalShim;
        },


        // Makes sure things can actually fit in places. Accounts for fixed Widths changing.
        calculateGroupWidths : function (grid, gridWidth, requiresShim) {
            var fixedWidth = 0,
                flexGroupWidths = 0,
                usedWidth,
                requiresHorizontalShim = false;

            if (requiresShim) {
                gridWidth -= grid.options.scrollbar.extraWidth;
            }
            var fixedGroups = _.filter(grid.config.groups, function (group) {
                return group.type === 'fixed';
            });
            var flexGroups = _.filter(grid.config.groups, function (group) {
                return group.type === 'flex';
            });
            // Fixed groups sizing
            _.forEach(fixedGroups, function (group) {
                var width = calculateGroupColumnWidths(group, gridWidth, grid);
                if (!group.ignoreBorders) {
                    width += grid.options.borders.viewportLeftRight;
                }
                fixedWidth += width;
                group.width = width;
            });

            var remainingWidth = gridWidth - fixedWidth;
            _.forEach(flexGroups, function (group) {
                var groupWidth = 0;
                calculateGroupColumnWidths(group, gridWidth, grid);
                if (group.width != null) {
                    if (isNaN(group.width)) {
                        if (_.isString(group.width) && group.width.indexOf('%') === -1) {
                            group.width = parseInt(group.width, 10);
                        } else {
                            group.width = Math.floor(remainingWidth * (parseFloat(group.width) / 100));
                        }
                    }
                    groupWidth = group.width;
                }

                flexGroupWidths += groupWidth;
            }
            );

            usedWidth = fixedWidth;

            var percentPerGroup = 1.0 / flexGroups.length;
            _.forEach(flexGroups, function (group, index) {
                var groupWidth;
                if (group.width == null || group.width === 0) { // Do not listen to width 0
                    groupWidth = parseInt(percentPerGroup * remainingWidth, 10);
                } else {
                    groupWidth = Math.floor((group.width / flexGroupWidths) * remainingWidth);
                    if (isNaN(groupWidth)) {
                        groupWidth = 0;
                    }
                }
                if (group.maxWidth != null) {
                    if (isNaN(group.maxWidth)) {
                        if (group.maxWidth.indexOf('%') === -1) {
                            group.maxWidth = parseInt(group.maxWidth, 10);
                        } else {
                            group.maxWidth = gridWidth * (parseFloat(group.maxWidth) / 100);
                        }
                    }
                    if (groupWidth > group.maxWidth) {
                        groupWidth = group.maxWidth;
                    }
                }

                usedWidth += groupWidth;
                // Make sure missed pixels get cleaned up from calculation
                if (index === flexGroups.length - 1 && usedWidth < gridWidth) {
                    groupWidth += gridWidth - usedWidth;
                    usedWidth += gridWidth - usedWidth;
                }
                // Don't let the group be larger than the data underneath
                if (group.combinedColumnWidth < groupWidth) {
                    if (group.combinedColumnWidth !== 0) { // Dont add extra space from empty groups as they are not used in the calculation to size the other groups
                        usedWidth -= (groupWidth - group.combinedColumnWidth);
                    } else {
                        usedWidth -= groupWidth;
                    }
                    groupWidth = _.clone(group.combinedColumnWidth);
                }
                // Reset if needed
                group.width = groupWidth;

                if (group.width < group.combinedColumnWidth) {
                    requiresHorizontalShim = true;
                }
            });
            var extraSpace = 0;
            if (usedWidth < gridWidth) {
                extraSpace = gridWidth - usedWidth;
            }
            // Redistrubute freespace to groups that have scrolling;
            if (extraSpace > 0) {
                _.forEach(flexGroups, function (group) {
                    var scrollSpace = group.combinedColumnWidth - group.width;
                    if (scrollSpace > 0 && extraSpace > 0) {
                        var addToGroup = extraSpace < scrollSpace ? extraSpace : scrollSpace;
                        if (group.maxWidth != null && !isNaN(group.maxWidth)) {
                            if (group.width < group.maxWidth) {
                                addToGroup = 0;
                            } else {
                                var allowableExtra = group.maxWidth - group.width;
                                if (addToGroup > allowableExtra) {
                                    addToGroup = allowableExtra;
                                }
                            }
                        }
                        group.width += addToGroup;
                        extraSpace -= addToGroup;
                    }
                });

                if (extraSpace > 0) {
                    grid.$grid.find('th[data-index="empty-column"]').attr('width', extraSpace + 'px');
                    var lastGroup = _.last(grid.config.groups);
                    lastGroup.width += extraSpace;
                }
            }

            return requiresHorizontalShim;
        },


        resizeDraggedGroupFn : function (grid, resizer) {
            return function () {
                grid.$grid.off('mousemove.group-resize');
                $(document.body).off('mouseup.group-resize');
                grid.$grid.removeClass('resizing-column');
                resizer.$selector.removeClass('dragging');
                var viewport = grid.viewports[_.first(grid.config.rowsets).id][resizer.group.id],
                    leftPosition = viewport.$viewport.position().left;
                // Potential call to giggity
                changeGroupSize(grid.config.groups, resizer.group,
                    resizer.$selector.position().left - leftPosition);
                grid.widget._trigger('view-updated', null, { action : 'group-resized', group : resizer.group.id, userAction : true });
                grid.resize();
            };
        },

        onGroupDragFn : function (grid, resizer, gridOffset) {
            var that = this;
            return function (event) {
                // left click button
                if (event.which === 1) {
                    that.moveGroupResizeBarFn(grid, resizer, event.clientX - gridOffset);
                }
            };
        },

        moveGroupResizeBarFn : function (grid, resizer, position) {
            if (resizer.$selector.is('.dragging')) {
                resizer.$selector.css('left', position);

                var viewport = grid.viewports[_.first(grid.config.rowsets).id][resizer.group.id],
                    leftPosition = viewport.$viewport.position().left;
                // Potential call to giggity
                changeGroupSize(grid.config.groups, resizer.group,
                    resizer.$selector.position().left - leftPosition);
                grid.resize();
            }
        },

        resizeDraggedRowsetFn : function (grid, resizer) {
            return function () {
                grid.$grid.off('mousemove.rowset-resize');
                $(document.body).off('mouseup.rowset-resize');
                resizer.$selector.removeClass('dragging');
                var topPosition = grid.$rowsets[resizer.rowset.id].position().top;
                // Potential call to giggity
                changeRowsetSize(grid.config.rowsets, resizer.rowset,
                    resizer.$selector.position().top - topPosition);
                grid.widget._trigger('view-updated', null, { action : 'rowset-resized', rowset : resizer.rowset.id, userAction : true });
                grid.resize();
                grid.widget._forceViewRender();
            };
        },

        onRowsetDragFn : function (resizer, gridOffset) {
            var that = this;
            return function (event) {
                // left click button
                if (event.which === 1) {
                    that.moveRowsetResizeBarFn(resizer, event.clientY - gridOffset);
                }
            };
        },
        moveRowsetResizeBarFn : function (resizer, position) {
            if (resizer.$selector.is('.dragging')) {
                resizer.$selector.css('top', position);
            }
        },

        calculateScrollbarWidth : utils.calculateScrollbarWidth
    };
});
