define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.moment',
    'gridapi.logger',
    'gridapi.grid/constants',
    'gridapi.grid/timephasedHandlers',
    'ganttapi.gantt/defaultTimescale',
    'ganttapi.gantt/periodTimescale',
    'gridapi.grid/configuration',
    'gridapi.grid/utils',
    'markup!gridapi.grid/templates/timephased.htm:.htm-layout',
    'markup!gridapi.grid/templates/timephased.htm:.htm-bucket',
    'markup!gridapi.grid/templates/timephased.htm:.htm-range',
    'markup!gridapi.grid/templates/timephased.htm:.htm-context-menu',
    'markup!gridapi.grid/templates/timephased.htm:.htm-context-menu-item',
    'markup!gridapi.grid/templates/timephased.htm:.htm-viewport-buttons'
], function ($, _, moment, logger, constants, timephasedMixins, DefaultTimescale, PeriodTimescale, config, utils,
            layoutTemplate, bucketTemplate, rangeTemplate, contextMenuTemplate, contextMenuItemTemplate, buttonTemplate) {
    'use strict';

    var formatString = utils.formatString;
    var TimephasedColumnGroup = function (args) {
        var that = this;
        this.$contextMenu = $();
        this.widget = args.widget;
        this.$timephaseContainer = $(layoutTemplate());
        this.$rangeContainer = this.$timephaseContainer.find('.range-container');
        this.$buttons = $(buttonTemplate({ locale : this.widget.options.locale }));
        this.$shiftLeftBtn = this.$buttons.filter('.shift-scale-left');
        this.$shiftRightBtn = this.$buttons.filter('.shift-scale-right');
        var start, end;
        if (this.widget.options.timephased.range.start != null) {
            start = this.widget.options.timephased.range.start.clone().startOf('day');
        } else {
            start = moment();
        }
        if (this.widget.options.timephased.range.end != null) {
            end = this.widget.options.timephased.range.end.clone().startOf('day');
        } else {
            end = start.add(1, 'year');
        }

        var viewStart = this.widget.options.timephased.view.start || start.clone().startOf('day');
        if (viewStart.isAfter(end)) {
            viewStart = start.clone().startOf('day');
        }

        this.widget.options.timephased.view.start = viewStart;

        this.settings = {
            step : 1,
            minValue : 'day',
            midValue : 'week',
            maxValue : 'month',
            range : {
                start : start,
                end : end
            },
            width : 0,
            view : {
                start : this.widget.options.timephased.view.start
            }
        };
        var setViewRange = function (range) {
            var newStart = range.start.clone()[config.startOf](this.options.value.min);
            if (newStart.isBefore(that.settings.range.start)) {
                newStart = that.settings.range.start.clone()[config.startOf](this.options.value.min);
            }

            var newEnd = moment(newStart).add(that.widget.options.timephased.bucketCount, this.options.value.min);
            if (newEnd.isAfter(that.settings.range.end)) {
                newEnd = that.settings.range.end.clone()[config.endOf](this.options.value.min);
            }
            this.options.range.start = newStart;
            this.options.range.end = newEnd;
        };
        this.scales = {
            WEEK_DAY : new DefaultTimescale({
                buffer : 0,
                title : this.widget.options.locale.timephased.weekDay,
                range : this.settings.range,
                rangeBounds : { end : this.settings.range.end.clone(), start : this.settings.range.start.clone() },
                granularity : constants.GRANULARITY.DAILY,
                granularitySize : 1,
                value : {
                    min : 'day',
                    mid : 'week',
                    max : 'month'
                },
                timescaleFns : {
                    getMidLabel : function (bucketStart) {
                            // exempt from PRIMCR-3724 - just a label, which may read better for foreign users if it says
                            // "week of <when their week starts>", rather than assuming sunday
                        return 'Week of ' + bucketStart.clone().startOf('week').format(this.options.format.mid);
                    },
                    setViewRange : setViewRange
                },

                format : {
                    min : 'ddd D',
                    mid : 'MMMM DD',
                    max : 'MMMM YYYY',
                    column : 'MMMM DD YYYY'
                },
                bucketSize : this.widget.options.timephased.bucketWidth,
                locale : this.widget.options.locale
            }),
            MONTH_WEEK : new DefaultTimescale({
                buffer : 0,
                title : this.widget.options.locale.timephased.monthWeek,
                range : this.settings.range,
                rangeBounds : this.settings.range,
                granularity : constants.GRANULARITY.WEEKLY,
                granularitySize : 7,
                value : {
                    min : 'week',
                    mid : 'month',
                    max : 'year'
                },
                format : {
                    min : 'DD',
                    mid : 'MMMM',
                    max : 'YYYY',
                    column : 'MMMM DD YYYY'
                },
                timescaleFns : {
                    setViewRange : setViewRange
                },
                bucketSize : this.widget.options.timephased.bucketWidth,
                locale : this.widget.options.locale
            }),
            QUARTER_MONTH : new DefaultTimescale({
                buffer : 0,
                title : this.widget.options.locale.timephased.quarterMonth,
                range : this.settings.range,
                rangeBounds : this.settings.range,
                granularity : constants.GRANULARITY.MONTHLY,
                granularitySize : 30,
                value : {
                    min : 'month',
                    mid : 'quarter',
                    max : 'year'
                },
                format : {
                    min : 'MMMM',
                    mid : '\\QQ',
                    max : 'YYYY',
                    column : 'MMMM DD YYYY'
                },
                timescaleFns : {
                    setViewRange : setViewRange
                },
                bucketSize : this.widget.options.timephased.bucketWidth,
                locale : this.widget.options.locale
            }),
            YEAR_MONTH : new DefaultTimescale({
                buffer : 0,
                title : this.widget.options.locale.timephased.yearMonth,
                range : this.settings.range,
                rangeBounds : this.settings.range,
                granularity : constants.GRANULARITY.MONTHLY,
                granularitySize : 30,
                value : {
                    min : 'month',
                    mid : 'year'
                },
                format : {
                    min : 'MMMM',
                    mid : 'YYYY',
                    column : 'MMMM DD YYYY'
                },
                timescaleFns : {
                    setViewRange : setViewRange
                },
                bucketSize : this.widget.options.timephased.bucketWidth,
                locale : this.widget.options.locale
            }),
            YEAR_QUARTER : new DefaultTimescale({
                buffer : 0,
                title : this.widget.options.locale.timephased.yearQuarter,
                range : this.settings.range,
                rangeBounds : this.settings.range,
                granularity : constants.GRANULARITY.QUARTERLY,
                granularitySize : 120,
                value : {
                    min : 'quarter',
                    mid : 'year'
                },
                format : {
                    min : '\\QQ',
                    mid : 'YYYY',
                    column : 'MMMM DD YYYY'
                },
                timescaleFns : {
                    setViewRange : setViewRange
                },
                bucketSize : this.widget.options.timephased.bucketWidth,
                locale : this.widget.options.locale
            })
        };
        if (this.widget.options.timephased.reportingPeriod != null && this.widget.options.timephased.reportingPeriod.periods != null && this.widget.options.timephased.reportingPeriod.periods.length) {
            this.scales[constants.GRANULARITY.REPORTING_PERIOD] = new PeriodTimescale({
                buffer : 0,
                range : this.settings.range,
                rangeBounds : { end : this.settings.range.end.clone(), start : this.settings.range.start.clone() },
                periodOptions : this.widget.options.timephased.reportingPeriod,
                granularity : constants.GRANULARITY.REPORTING_PERIOD,
                granularitySize : this.widget.options.timephased.reportingPeriod.size,
                bucketCount : this.widget.options.timephased.bucketCount,
                value : {
                    min : 'day',
                    mid : 'month',
                    max : 'year'
                },
                format : {
                    min : 'ddd D',
                    mid : 'MMMM',
                    max : 'YYYY',
                    column : 'MMMM DD YYYY',
                    reportingPeriod : 'ddd D',
                    reportingPeriodMonthly : 'MMM D'
                },
                usePeriodLabels : this.widget.options.timephased.usePeriodLabels,
                bucketSize : this.widget.options.timephased.bucketWidth,
                title : formatString(this.widget.options.locale.timephased.reportingPeriod, this.widget.options.timephased.reportingPeriod.name),
                locale : this.widget.options.locale
            });
        }
        _.forEach(_.values(this.scales), function (timescale) {
            timescale.setRange(that.settings.range);
            timescale.allMinBuckets = timescale.getBuckets().min;
        });
        this.setViewRange(this.settings.view);
        this.generateColumnDefinitions();
    };

    TimephasedColumnGroup.prototype = {
        constructor : TimephasedColumnGroup,

        applyToView : function () {
            if (this.widget.options.timephased.enabled) {
                var $viewport = this.widget.view.viewports['$group-timephased'].filter('.viewport-headers');
                $viewport.prepend(this.$buttons);
                $viewport.prepend(this.$timephaseContainer);
            }
        },

        getColumnDefinitions : function () {
            return this.columnDefinitions;
        },

        generateColumnDefinitions : function () {
            var that = this;
            var colGroupOpts = this.widget.options.timephased;

            var columns = [];
            _.forEach(this.scales, function (timescale, timescaleKey) {
                _.forEach(timescale.allMinBuckets, function (bucket) {
                    var bucketIdentifiers = that.getBucketIdentifiers(bucket, timescaleKey);
                    var columnTooltip;
                    if (constants.GRANULARITY.DAILY === timescale.options.granularity) {
                        columnTooltip = bucket.start.format(colGroupOpts.tooltipFormat);
                    } else {
                        columnTooltip = bucket.start.format(colGroupOpts.tooltipFormat) + ' - ' + bucket.end.format(colGroupOpts.tooltipFormat);
                    }
                    columns.push({
                        displayType : 'timephased',
                        title : bucket.label,
                        dataIndex : bucketIdentifiers.dataIndex,
                        bucketId : bucketIdentifiers.id,
                        width : that.widget.options.timephased.bucketWidth,
                        granularity : timescale.options.granularity,
                        tooltip : columnTooltip,
                        readOnly : !that.widget.options.timephased.canEditTimephasedBucket({
                            start : bucket.start,
                            end : bucket.end,
                            bucketId : bucketIdentifiers.id
                        }),
                        start : bucket.start,
                        end : bucket.end,
                        visible : false,
                        timephased : true
                    });
                });
            });

            this.columnDefinitions = columns;
        },

        shiftLeft : function () {
            var start = this.appliedTimescale.options.range.start.clone();
            var currentStartIndex = _.findIndex(this.appliedTimescale.allMinBuckets, function (bucket) {
                return bucket.start.isSame(start);
            });

            var newStartIndex = currentStartIndex - this.widget.options.timephased.shiftCount;
            if (newStartIndex < 0) {
                newStartIndex = 0;
            }

            var newStart = this.appliedTimescale.allMinBuckets[newStartIndex].start.clone();
            if (this.widget._trigger('before-timephased-bounds-change', null, {
                newStart : newStart
            }) === false) {
                return;
            }

            this.setViewRange({ start : newStart });
            this.widget._trigger('timephased-bounds-change', null, {
                scale : this.appliedTimescaleId,
                min : this.appliedTimescale.options.range.start,
                max : this.appliedTimescale.options.range.end,
                granularity : this.appliedTimescale.options.granularity
            });
            this.widget.view.changeTimescale(this.appliedTimescaleId);
            this.widget.selectCell(null, this.firstVisibleTimephasedColumn, null, null);
        },

        shiftRight : function () {
            var start = this.appliedTimescale.options.range.start.clone();
            var currentStartIndex = _.findIndex(this.appliedTimescale.allMinBuckets, function (bucket) {
                return bucket.start.isSame(start);
            });

            var newStartIndex = currentStartIndex + this.widget.options.timephased.shiftCount;
            var newEndIndex = newStartIndex + this.widget.options.timephased.bucketCount;
            if (newEndIndex >= this.appliedTimescale.allMinBuckets.length) {
                newStartIndex = this.appliedTimescale.allMinBuckets.length - 1 - this.widget.options.timephased.bucketCount;
            }

            var newStart = this.appliedTimescale.allMinBuckets[newStartIndex].start.clone();
            if (this.widget._trigger('before-timephased-bounds-change', null, {
                newStart : newStart
            }) === false) {
                return;
            }

            this.setViewRange({ start : newStart });
            this.widget._trigger('timephased-bounds-change', null, {
                scale : this.appliedTimescaleId,
                min : this.appliedTimescale.options.range.start,
                max : this.appliedTimescale.options.range.end,
                granularity : this.appliedTimescale.options.granularity
            });
            this.widget.view.changeTimescale(this.appliedTimescaleId);

            this.widget.selectCell(null, this.lastVisibleTimephasedColumn, null, null);
        },

        createTimescale : function () {
            this.applyTimescale(this.widget.options.timephased.scale);
            this.initListeners();
        },

        initListeners : function () {
            var that = this;
            this.$timephaseContainer.on('mousedown', that.onMouseDown());
            this.$timephaseContainer.on('contextmenu', that.onContextMenu());
            this.$shiftLeftBtn.on('click', that.onShiftLeft());
            this.$shiftRightBtn.on('click', that.onShiftRight());
        },

        closeContextMenu : function () {
            this.$contextMenu.remove();
            $(document.body).off('mousedown.close-context');
        },

        openContextMenu : function (event) {
            this.closeContextMenu();
            var that = this;
            var timescales = [];
            _.forEach(this.widget.options.timephased.allowedTimescales, function (allowedScale) {
                if (that.scales[allowedScale] != null) {
                    timescales.push({
                        title : that.scales[allowedScale].options.title,
                        timescale : allowedScale,
                        granularitySize : that.scales[allowedScale].options.granularitySize
                    });
                }
            });
            timescales = _.sortBy(timescales, 'granularitySize');
            var $menu = $(contextMenuTemplate({ locale : that.widget.options.locale }));
            var $menuItems = $menu.find('.menu-items');
            _.forEach(timescales, function (timescale) {
                $menuItems.append(contextMenuItemTemplate(timescale));
            });
            $menu.find('.grid-menu-item[data-timescale="' + that.appliedTimescaleId + '"]').addClass('timescale-selected');
            $menu.css('left', event.pageX + 'px');
            $menu.css('top', event.pageY - $(window).scrollTop() + 'px');

            this.$contextMenu = $menu;
            $(document.body).append($menu);
            $menu.find('.set-timescale').on('mousedown', function () {
                var timescale = $(this).data('timescale');
                if (that.widget._trigger('before-timephased-scale-change', null, {
                    currentScale : that.appliedTimescaleId,
                    scale : timescale,
                    min : that.scales[timescale].options.range.start,
                    max : that.scales[timescale].options.range.end,
                    granularity : that.scales[timescale].options.granularity
                }) === false) {
                    return;
                }
                that.widget.view.changeTimescale(timescale);
                that.widget.selectCell(null, that.firstVisibleTimephasedColumn, null, null);
            });
            $(document.body).on('mousedown.close-context', function () {
                that.closeContextMenu();
            });
        },

        setTimescaleRange : function (range) {
            this.settings.view.start = range.start;
            range.end = this.settings.range.end;
            this.widget.options.timephased.view.start = range.start;
            _.forEach(_.values(this.scales), function (timescale) {
                timescale.setRange({ start : range.start, end : range.end });
            });
            if (this.appliedTimescale != null) {
                this.widget.options.timephased.view.end = this.appliedTimescale.options.range.end;
            }
        },

        setViewRange : function (newRange) {
            if (newRange == null || newRange.start == null) {
                return;
            }
            // Don't let them try to go to a date outside of the range
            if (newRange.start.isAfter(this.settings.range.end)) {
                newRange.start = this.settings.range.start.clone();
            }
            this.settings.view.start = newRange.start;
            newRange.end = this.settings.range.end;
            this.widget.options.timephased.view.start = newRange.start;
            _.forEach(_.values(this.scales), function (timescale) {
                timescale.setViewRange({ start : newRange.start, end : newRange.end });
            });
            if (this.appliedTimescale != null) {
                this.widget.options.timephased.view.end = this.appliedTimescale.options.range.end;
            }
        },

        getBucketIdentifiers : function (bucket, prefix) {
            var id = bucket.start.format(this.widget.options.timephased.bucketIdFormat);
            return {
                id : id,
                dataIndex : prefix + '_' + id
            };
        },

        applyTimescale : function (timescale) {
            var that = this;
            if (this.scales[timescale] == null) {
                timescale = this.widget.options.timephased.scale || 'QUARTER_MONTH';
            }
            if (this.scales[timescale] != null) {
                this.emptyTimescale();
                this.appliedTimescale = this.scales[timescale];
                this.appliedTimescaleId = timescale;
                var buckets = this.appliedTimescale.getBuckets();
                _.forEach(that.widget.columnDefinitions, function (definition) {
                    if (definition.timephased) {
                        definition.visible = false;
                    }
                });
                var columnsToRollup = [];
                _.forEach(buckets.min, function (bucket, index) {
                    var bucketIdentifiers = that.getBucketIdentifiers(bucket, that.appliedTimescaleId);
                    if (index === 0) {
                        that.firstVisibleTimephasedColumn = bucketIdentifiers.dataIndex;
                    }
                    that.lastVisibleTimephasedColumn = bucketIdentifiers.dataIndex;
                    that.widget.columnDefinitions[bucketIdentifiers.dataIndex].visible = true;
                    columnsToRollup.push(bucketIdentifiers.dataIndex);
                });

                if (_.first(buckets.min).start.isSame(_.first(this.appliedTimescale.allMinBuckets).start)) {
                    this.$shiftLeftBtn.hide();
                } else {
                    this.$shiftLeftBtn.show();
                }

                if (_.last(buckets.min).start.isSame((_.last(this.appliedTimescale.allMinBuckets).start))) {
                    this.$shiftRightBtn.hide();
                } else {
                    this.$shiftRightBtn.show();
                }

                this.createRange({
                    buckets : buckets.mid,
                    rangeClasses : 'mid-range',
                    bucketClasses : ['mid-value']
                });
                this.createRange({
                    buckets : buckets.max,
                    rangeClasses : 'max-range',
                    bucketClasses : ['max-value']
                });

                this.settings.numberOfBuckets = buckets.min.length;
                this.settings.width = this.appliedTimescale.options.bucketSize * this.settings.numberOfBuckets;
                this.widget.options.timephased.granularity = this.appliedTimescale.options.granularity;
                this.$timephaseContainer.css('width', this.settings.width + 'px');
                this.widget.options.timephased.view.end = this.appliedTimescale.options.range.end;
                setTimeout(function () {
                    that.widget._trigger('timephased-scale-change', null, {
                        scale : that.appliedTimescaleId,
                        min : that.appliedTimescale.options.range.start,
                        max : that.appliedTimescale.options.range.end,
                        granularity : that.appliedTimescale.options.granularity
                    });
                    that.widget.recordGrouper.forceAllRollups({ dataIndices : columnsToRollup });
                }, 10);
            }
        },

        createRange : function (options) {
            var buckets = options.buckets;
            var $range = $(rangeTemplate({
                classes : options.rangeClasses
            }));
            this.$rangeContainer.prepend($range);

            if (buckets != null && buckets.length > 0) {
                var bucketMarkup = '';
                for (var i = 0; i < buckets.length; i++) {
                    var bucket = buckets[i];
                    var bucketClasses = options.bucketClasses;

                    if (bucket.available === false) {
                        bucketClasses = bucketClasses.concat(['unavailable']);
                    }

                    bucketMarkup += bucketTemplate({
                        resize : options.resize,
                        classes : bucketClasses.join(' '),
                        width : bucket.width,
                        label : bucket.label,
                        units : bucket.units
                    });
                }
                $range.append(bucketMarkup);
            }
        },

        emptyTimescale : function () {
            this.$rangeContainer.empty();
        },

        resizeTimescale : function (newBucketSize) {
            _.forEach(this.widget.columnDefinitions, function (definition) {
                if (definition.timephased) {
                    definition.width = newBucketSize;
                }
            });
            this.widget.options.timephased.bucketWidth = newBucketSize;
            this.appliedTimescale.options.bucketSize = newBucketSize;
            this.settings.width = this.appliedTimescale.options.bucketSize * this.settings.numberOfBuckets;


            this.$rangeContainer.find('.mid-value,.max-value').each(function () {
                $(this).css({
                    'padding-left' : '',
                    'padding-right' : ''
                });
                $(this).width(parseFloat($(this).data('units')) * newBucketSize);
            });

            this.$timephaseContainer.css('width', this.settings.width + 'px');
        },

        centerLabels : function (selector, timescaleOffset, timescaleVisibleWidth) {
            var $left, $right, leftPosition, leftWidth, rightPosition, rightWidth;

            var labels = this.$timephaseContainer.find(selector);

            // Wipe out any paddings that were originally on them
            labels.filter('.centered-label').css({
                'padding-left' : '',
                'padding-right' : ''
            }).removeClass('centered-label');

            _.forEach(labels, function (midValue) {
                var bucketLeft, bucketWidth;

                // Once found both left and right - can break out early
                if (!$left || !$right) {
                    bucketLeft = midValue.offsetLeft;
                    bucketWidth = midValue.offsetWidth;
                    if (timescaleOffset + bucketLeft < 0 &&
                        timescaleOffset + bucketLeft + bucketWidth > 0) {
                        $left = $(midValue);
                        leftPosition = bucketLeft;
                        leftWidth = bucketWidth;
                    }

                    if (timescaleOffset + bucketLeft + bucketWidth > timescaleVisibleWidth &&
                        timescaleOffset + bucketLeft < timescaleVisibleWidth) {
                        $right = $(midValue);

                        rightPosition = bucketLeft;
                        rightWidth = bucketWidth;
                    }
                }

                // https://lodash.com/docs#forEach "iterators may exit earlier by explicitly returning false"
                return !($left && $right);
            });

            if ($left &&
                (Math.abs(timescaleOffset) - leftPosition < leftWidth)) {
                $left.addClass('centered-label');
                $left.css({
                    'padding-left' : (Math.abs(timescaleOffset) - leftPosition) + 'px'
                });
            }

            if ($right &&
                (timescaleOffset + rightPosition + rightWidth - timescaleVisibleWidth) < rightWidth) {
                $right.addClass('centered-label');
                $right.css({
                    'padding-right' : (timescaleOffset + rightPosition + rightWidth) - timescaleVisibleWidth + 'px'
                });
            }
        },

        updateBucketLabelPositions : function () {
            var timescaleOffset = this.$timephaseContainer.position().left;
            var timescaleVisibleWidth = this.$timephaseContainer.parent().width();

            this.centerLabels('.mid-value', timescaleOffset, timescaleVisibleWidth);
            this.centerLabels('.max-value', timescaleOffset, timescaleVisibleWidth);
        }

    };

    _.assign(TimephasedColumnGroup.prototype, timephasedMixins);

    return TimephasedColumnGroup;
});
