define([
    'gridapi.jquery',
    'gridapi.lodash'
], function ($, _) {
    'use strict';

    function ViewManager(widget) {
        this.widget = widget;
        this._paused = false;
        this.reset();
    }

    ViewManager.prototype = _.assign({}, {
        constructor : ViewManager,

        pause: function () {
            this._paused = true;
        },

        resume: function () {
            this._paused = false;
        },

        reset : function () {
            this.queue = {};
            this.args = {};
        },

        isPaused : function () {
            return this._paused;
        },

        refresh : function () {
            if (this.isPaused()) {
                return;
            }

            var queue = this.queue;
            var viewConfig;
            if (queue.resetView) {
                this.pause();
                if (queue.applyViewConfig) {
                    viewConfig = this.widget.getViewConfig();
                    if (queue.deleteColumnGroups) {
                        delete viewConfig.columnGroups;
                    }
                    this.widget.applyViewConfig(viewConfig);
                }
                this.widget._resetView();
                this.resume();
            }
            if (queue.group) {
                this.widget.groupRows(this.widget.config.groupBy);
            } else if (queue.sortBy) {
                this.widget._applySortConfig({
                    surpressTrigger : false,
                    userAction : false
                });
            } else if (queue.repopulate) {
                this.widget.repopulate(_.get(this.args.repopulate, 'record'), _.get(this.args.repopulate, 'dataIndex'));
            } else if (queue.redrawRows) {
                this.widget._forceViewRender();
            } else if (queue.resize) {
                this.widget.resize();
            }
            if (queue.setTimescaleView) {
                this.widget.view.setTimescaleView();
            }
            this.reset();
        }
    });

    return ViewManager;
});
