define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.logger',
    'gridapi.grid/utils/keyCode'
], function ($, _, logger, keyCode) {
    'use strict';

    var api = {
        allowRefocusOn : function (ctx, givenFn) {
            //>>excludeStart("validation", pragmas.production)
            logger.assert(ctx && ctx.timeouts, 'given ctx must have a timeouts instance (from utils#timeoutFactory');
            //>>excludeEnd("validation")
            ctx.allowRefocus = function allowRefocus() {
                ctx._internalFocus = true;
                ctx.timeouts.setTimeout(function () {
                    ctx._internalFocus = false;
                }, 500);
            };
            var method = givenFn || 'destroy';
            ctx[method] = _.wrap(ctx[method], function (origFn, args) {
                if (ctx._internalFocus && args && args.fromBlur) {
                    ctx._internalFocus = false;
                } else {
                    origFn.call(ctx, args);
                }
            });
        },
        initDestroyEvents : function (args) {
            args.$el.on('keydown', function (ev) {
                if (ev.keyCode === keyCode.ESCAPE) {
                    ev.preventDefault();
                    args.destroyFn();
                    args.grid.view.requestFocus();
                }
            });
            var attachId = _.uniqueId('grid-menu-');
            var $w = $(window).on('resize.' + attachId, function () {
                args.destroyFn();
            });
            var $b = $('body').on('mousedown.' + attachId, function (ev) {
                var $target = $(ev.target);
                if (!$target.parents(args.rootSelector).length) {
                    args.destroyFn();
                }
            });
            return {
                destroy : function () {
                    $w.off('.' + attachId);
                    $b.off('.' + attachId);
                }
            };
        }
    };
    return api;
});
