define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/constants',
    'gridapi.grid/exporter',
    'markup!gridapi.grid/templates/grid.htm:.htm-row-template',
    'gridapi.columns/textColumn',
    'gridapi.grid/utils'
], function ($, _, constants, exporter, rowTemplate, textColumn, utils) {
    'use strict';

    var formatString = utils.formatString;
    var RowRenderer = function (args) {
        var that = this;
        var rendererFns = args.options.rows.rendererFns;
        this.options = args.view.options;
        this.view = args.view;
        this.widget = args.widget;
        // override mediator functions from optionsmn
        _.forEach(rendererFns, function (fnc, fncName) {
            that[fncName] = fnc || that[fncName];
        });
    };

    RowRenderer.prototype = {
        constructor : RowRenderer,

        makeRow : function (recordId, rowsetId) {
            var groups = this.view.config.groups;
            var record;
            if (recordId === constants.DUMMY_RECORD_ID) {
                record = this.widget._getProxyRecord();
            } else {
                record = this.widget._getRecord(recordId);
            }
            // Grab args ahead of time to reduce the widget calls each time a row segment/cell is made
            var args = {
                recordId : recordId,
                rowsetId : rowsetId,
                record : record,
                node : this.view.virtualRows[rowsetId].nodes[recordId],
                lastRecordAtLevel : false,
                recordAttributes : this.widget._attrs.all(record),
                isGroup : this.widget._isGroupRecord(record)
            };

            var row = {
                RECORD_ID : recordId
            };

            if (args.node != null) {
                var recordBelow = this.view.recordBelow(recordId);

                if (recordBelow != null) {
                    var nextNode = this.view.virtualRows[rowsetId].nodes[recordBelow];
                    if (nextNode != null && args.node != null && (nextNode.level < args.node.level)) {
                        args.lastRecordAtLevel = true;
                    }
                } else {
                    args.lastRecordAtLevel = true;
                }
            }
            for (var g = 0, glen = groups.length; g < glen; g++) {
                var group = groups[g];
                if (group.showStackedCells) {
                    var stackedRows = '';
                    var cellHeight;
                    var stackedCells = group.stackedCells;
                    for (var i = 0, sclen = stackedCells.length; i < sclen; i++) {
                        if (i === 0 || i === sclen - 1) {
                            cellHeight = this.widget.options.stackedCells.rowHeight;
                        } else {
                            cellHeight = this.widget.options.stackedCells.rowHeight - 1;
                        }
                        stackedRows += this.makeRowGroupSegment(_.defaults({
                            group : group,
                            subRow : group.stackedCells[i],
                            rowHeight : sclen > 1 ? this.widget.options.stackedCells.rowHeight : this.view.ROW_HEIGHT,
                            cellHeight : sclen > 1 ? cellHeight : this.view.CELL_HEIGHT
                        }, args));
                    }
                    row[group.id] = stackedRows;
                } else {
                    row[group.id] = this.makeRowGroupSegment(_.defaults({
                        group : group,
                        rowHeight : this.view.ROW_HEIGHT,
                        cellHeight : this.view.CELL_HEIGHT
                    }, args));
                }
            }
            return row;
        },

        makeRowGroupSegment : function (args) {
            var columns = args.group.columns;
            var parentId = null;
            if (args.node != null && args.node.parent != null) {
                parentId = args.node.parent.id;
            }

            // Create the base row
            var tr = rowTemplate({
                id : this.getRowId(args),
                recordId : args.recordId,
                rowsetId : args.rowsetId,
                groupId : args.group.id,
                parentId : parentId,
                ariaAttributes : this.getAriaAttributes(args),
                rowClasses : this.getRowClasses(args),
                dataAttributes : this.getDataAttributes(args),
                inlineStyles : this.getInlineStyles(args),
                rowAttributes : this.getRowAttributes(args)
            });


            // Add each column if it is visible
            for (var c = 0; c < columns.length; c++) {
                var column = columns[c];
                if (args.recordId === constants.DUMMY_RECORD_ID) {
                    if (column.visible) {
                        tr += this.getDummyRecordCell(_.defaults({
                            column : column,
                            printable : false
                        }, args));
                    }
                } else if (column.visible) {
                    if (args.isGroup) {
                        if (!column.hideGroupCell) {
                            tr += this.makeGroupCell(_.defaults({
                                column : column
                            }, args), args.recordId, args.node, column, args.recordAttributes, args.rowsetId, args.group.id, false, args.lastRecordAtLevel);
                        }
                    } else {
                        tr += this.makeCell(_.defaults({
                            column : column,
                            printable : false
                        }, args));
                    }
                }
            }
            tr += '</tr>';

            return tr;
        },

        getRowId : function (args) {
            var rowId = this.widget.uuid + 'grid-row-' + args.recordId + '-group-' + args.group.id;

            if (args.subRow != null) {
                rowId += '-' + args.subRow;
            }
            return rowId;
        },

        getDataAttributes : function (args) {
            var dataIdAttribute = ' data-id="' + args.recordId + '"',
                dataRowsetIdAttribute = ' data-rowset-id="' + args.rowsetId + '"',
                dataGroupAttribute = ' data-group-id="' + args.group.id + '"',
                parentIdAttribute = '';

            return dataIdAttribute + dataRowsetIdAttribute + dataGroupAttribute + parentIdAttribute;
        },

        getInlineStyles : function (args) {
            return '';
        },

        getAriaAttributes : function (args) {
            var ariaAttributes = ' role="row" ';
            if (args.node != null) {
                ariaAttributes += ' aria-level="' + (args.node.level + 1) + '" ';
                if (args.node.loading) {
                    ariaAttributes += ' aria-busy="true"';
                } else if (this.options.lazyload) {
                    ariaAttributes += ' aria-busy="false"';
                }
            }
            return ariaAttributes;
        },

        getRowAttributes : function (args) {
            return this.options.rows.attributes(args.record, args.group.id, args.rowsetId);
        },

        getDummyRecordCell : function (args) {
            var markup = '';
            var data = this.widget.store.getValueFromRecord(args.record, args.column.dataIndex);
            args.isDummyRecord = true;
            if (args.column.dataIndex === constants.MENU_BUTTON_COLUMN) {
                args.showQuickAdd = true;
                args.quickAddHint = formatString(this.widget.options.locale.quickAddHint, this.widget.options.quickAdd.getQuickAddType({
                    options : this.options,
                    grid : this.widget,
                    mediator : this.widget.store
                }));
                markup += this.makeCell(args);
            } else if (args.column.dataIndex === this.view.quickAddHintColumn &&
                    this.options.locale.quickAddHint != null &&
                    (data == null || data === '')) {
                var cellClasses = 'pgbu-cell ';
                if (args.column.dataIndex === this.view.firstVisibleColumn) {
                    cellClasses += 'first-visible-cell ';
                }
                if (args.column.dataIndex === this.view.lastVisibleColumn) {
                    cellClasses += 'last-visible-cell ';
                }
                markup += '<td class="' + cellClasses + '" data-index="' + args.column.dataIndex;
                markup += '" id="' + this.view.getCellId(args.recordId, args.column.dataIndex);
                markup += '"><div class="cell-positioner" style="height: ' + args.cellHeight + 'px"><div class="cell-renderer" style="line-height: ' + this.view.CELL_HEIGHT + 'px">';
                markup += '<span aria-hidden="true" role="presentation" class="cell-hint-message">';
                markup += formatString(this.widget.options.locale.quickAddHint, this.widget.options.quickAdd.getQuickAddType({
                    options : this.options,
                    grid : this.widget,
                    mediator : this.widget.store
                }));
                markup += '</span>';
                markup += '</td>';
            } else {
                markup += this.makeCell(args);
            }
            return markup;
        },


        getRowClasses : function (args) {
            var userClasses = this.options.rows.classes(args.record, args.group.id, args.rowsetId),
                classes = [];

            if (args.recordAttributes.newRow) {
                classes.push('row-new');
            }
            if (args.recordAttributes.selected) {
                classes.push(this.options.classes.rowSelected);
            }
            if (args.recordAttributes.rowSelectedClasses && args.recordAttributes.rowSelectedClasses.indexOf('row-selected-top') > -1) {
                classes.push(this.options.classes.rowSelectedTop);
            }
            if (args.recordAttributes.rowSelectedClasses && args.recordAttributes.rowSelectedClasses.indexOf('row-selected-bottom') > -1) {
                classes.push(this.options.classes.rowSelectedBottom);
            }
            if (args.recordAttributes.focused) {
                classes.push(this.options.classes.rowFocused);
            }
            if (this.widget.clipboard.scope === 'row' && args.recordAttributes.cutCopyClasses != null) {
                classes.push(args.recordAttributes.cutCopyClasses);
            }
            if (args.recordAttributes.checked) {
                classes.push('row-checked');
            }
            if (args.node != null && args.node.isNode) {
                var bandClassIndex = (parseInt(args.node.level, 10)) % this.view.groupBandClasses.length;
                classes.push(this.view.groupBandClasses[bandClassIndex]);
                if (this.options.grouping.boldBands && !args.group.showStackedCells) {
                    classes.push('group-text-bold');
                }
            }
            if (args.recordAttributes.draggable && this.view.canDragRows()) {
                classes.push('draggable');
            }
            classes.push(userClasses);

            if (args.group.showStackedCells) {
                var subRowPosition = args.group.stackedCells.indexOf(args.subRow);
                if (args.group.hideStackedCellBorders) {
                    if (subRowPosition - 1 <= args.group.stackedCells.length) {
                        classes.push('hide-row-bottom-border');
                    }
                    if (subRowPosition > 0) {
                        classes.push('hide-row-top-border');
                    }
                }
                if (subRowPosition === 0) {
                    classes.push('top-stacked-row');
                } else if (subRowPosition === (args.group.stackedCells.length - 1)) {
                    classes.push('bottom-stacked-row');
                } else {
                    classes.push('middle-stacked-row');
                }
            }

            classes = _.without(classes, undefined, null, '');
            return classes.join(' ');
        },


        makePrintRow : function (recordId, rowsetId, stacked) {
            var args = {
                recordId : recordId,
                rowsetId : rowsetId,
                record : this.widget._getRecord(recordId),
                node : this.view.virtualRows[rowsetId].nodes[recordId],
                lastRecordAtLevel : false
            };
            args.recordAttributes = this.widget._attrs.all(args.record);
            args.isGroup = this.widget._isGroupRecord(args.record);
            var groups = this.view.config.groups;

            var userClasses = this.options.rows.classes(args.record, 'print', rowsetId),
                classes = 'class="' + (args.recordAttributes.newRow ? 'row-new ' : ''),
                dataIdAttribute = ' data-id="' + recordId + '"',
                dataGroupAttribute = ' data-group-id="print"',
                parentIdAttribute = '';

            classes += args.node != null && args.node.isNode ? this.view.groupBandClasses[parseInt(args.node.level, 10)] : '';
            classes += userClasses + '"';

            if (classes.length === 8) {
                classes = '';
            }

            if (args.node != null && args.node.parent != null) {
                parentIdAttribute = ' data-parent-id="' + args.node.parent.id + '" ';
            }

            if (recordId === constants.DUMMY_RECORD_ID) {
                return '';
            }
            var printRow = {
                row : '<tr ' + classes + this.options.rows.attributes(args.record, 'print', rowsetId) +
                    dataIdAttribute + dataGroupAttribute + parentIdAttribute + '>'
            };

            for (var g = 0; g < groups.length; g++) {
                var group = groups[g];

                var columns = group.columns,
                    groupId = group.id;

                // Add each column if it is visible
                for (var c = 0; c < columns.length; c++) {
                    var column = columns[c];
                    var columnIsShown = column.visible && (!column.internalDisplayOnly || column.displayType === 'joinedIcon');

                    if (columnIsShown && stacked && !group.showStackedCells) {
                        // stacked > 0 means we are generating a stacked row that is not the first in the group,
                        // and therefore we do not need to repeat values that are not stacked in this row
                        printRow.row += '<td></td>';
                    } else if (columnIsShown) {
                        if (group.showStackedCells) {
                            args.subRow = group.stackedCells[stacked];
                            printRow.stacked = (stacked + 1) % group.stackedCells.length;
                        }

                        if (args.isGroup) {
                            if (!column.hideGroupCell) {
                                printRow.row += this.makeGroupCell(_.defaults({
                                    group : group,
                                    column : column,
                                    cellHeight : args.rowHeight - this.widget.options.borders.row
                                }, args), args.recordId, args.node, column, args.recordAttributes, args.rowsetId, groupId, true, args.lastRecordAtLevel);
                            }
                        } else {
                            printRow.row += this.makeCell(_.defaults({
                                column : column,
                                group : group,
                                printable : true
                            }, args));
                        }
                    }
                }
            }
            printRow.row += '</tr>';
            return printRow;
        },

        makeExcelRow : function (recordId, rowsetId, rowNum, stacked) {
            var that = this,
                record = that.widget._getRecord(recordId),
                recordAttributes = that.widget._attrs.all(record),
                node = that.view.virtualRows[rowsetId].nodes[recordId],
                groups = that.view.config.groups,
                isGroup = that.widget._isGroupRecord(record);

            if (recordId === constants.DUMMY_RECORD_ID) {
                return '';
            }
            var excelRow = {
                attrs : 'x14ac:dyDescent="0.25"',
                cells : []
            };
            if (node != null && node.level != null && node.level > 0) {
                excelRow.outlineLevel = node.level;
                excelRow.attrs += ' outlineLevel="' + node.level + '"';
            }
            var columnNum = 1;

            for (var g = 0; g < groups.length; g++) {
                var group = groups[g];

                var columns = group.columns,
                    groupId = group.id;

                // Add each column if it is visible
                for (var c = 0; c < columns.length; c++) {
                    var column = columns[c];
                    var columnIsShown = column.visible && !column.internalDisplayOnly;

                    if (columnIsShown && stacked && !group.showStackedCells) {
                        // stacked > 0 means we are generating a stacked row that is not the first in the group,
                        // and therefore we do not need to repeat values that are not stacked in this row
                        excelRow.cells.push({
                            name : exporter.getExcelCellName(rowNum, columnNum),
                            mergedCell : true
                        });
                        columnNum++;
                    } else if (columnIsShown) {
                        var data, renderer;
                        var args = {
                            group : group,
                            column : column,
                            recordId : recordId,
                            record : record,
                            recordAttributes : recordAttributes
                        };

                        if (group.showStackedCells) {
                            args.subRow = group.stackedCells[stacked];
                            excelRow.stacked = (stacked + 1) % group.stackedCells.length;
                        }

                        var expandable = (column.dataIndex === that.view.expandableDataIndex);
                        var renderOptions = {
                            cellHeight : this.view.CELL_HEIGHT,
                            attributes : recordAttributes,
                            column : column,
                            record : record,
                            recordId : recordId,
                            modified : _.includes(recordAttributes.modified, column.dataIndex),
                            readOnly : column.readOnly || _.includes(recordAttributes.readOnly, column.dataIndex),
                            editable : this.options.editable,
                            focused : recordAttributes.focused,
                            selected : column.dataIndex === that.view.selectionModel.focus.columnId,
                            clickableNode : expandable && recordAttributes.isNode,
                            expandable : expandable,
                            hasChildren : recordAttributes.hasChildren,
                            expanded : (node != null && node.expanded != null) ? node.expanded : false,
                            level : node != null ? node.level : 0,
                            loading : node != null ? node.loading : false,
                            groupBandClasses : that.view.groupBandClasses,
                            cellSelectedClasses : recordAttributes.cellSelectedClasses,
                            found : false,
                            matcher: this.widget.searchMatcher,
                            modifiedRow : column.dataIndex === that.view.firstVisibleColumn && recordAttributes.modified.length,
                            invalid : recordAttributes.invalid != null ? recordAttributes.invalid[column.dataIndex] : null,
                            rowsetId : rowsetId,
                            groupId : groupId,
                            addSpacer : expandable && (that.options.tree || that.widget.groupingApplied),
                            locked : _.includes(recordAttributes.locked, column.dataIndex),
                            markup : this.options.markup,
                            id : this.view.getCellId(recordId, column.dataIndex),
                            labels : this.getCellLabels(recordId, column.dataIndex),
                            customAttributes : recordAttributes.custom,
                            locale : this.options.locale,
                            cellLocationClasses : '',
                            cutCopyClasses : this.widget.clipboard.cellIndex === column.dataIndex ? recordAttributes.cutCopyClasses : '',
                            descriptionIds : this.view.descriptionIds,
                            forExport : true,
                            excelCellName : exporter.getExcelCellName(rowNum, columnNum)
                        };

                        if (isGroup) {
                            data = this.getGroupCellData(args, renderOptions);
                            renderer = this.getGroupCellRenderer(args);
                        } else {
                            data = this.getCellData(args, renderOptions);
                            renderer = this.getRenderer(args);
                        }
                        renderOptions.data = data;

                        if (renderOptions.invalid != null && renderOptions.invalid.invalidFormat) {
                            renderOptions.formattedValue = renderOptions.invalid.invalidFormat.invalidData;
                        } else if (!renderOptions.formattedValue) {
                            renderOptions.formattedValue = renderer.format(renderOptions, data);
                        }

                        var cell = {
                            name : renderOptions.excelCellName
                        };

                        _.assign(cell, renderer.getExcelCell(renderOptions));

                        if (excelRow.outlineLevel && columnNum === 1) {
                            // only indent cell content of the first column
                            cell.outlineLevel = excelRow.outlineLevel;
                        }

                        excelRow.cells.push(cell);
                        columnNum++;
                    }
                }
            }

            return excelRow;
        },

        getCellId : function (recordId, dataIndex, subRow) {
            return this.widget.uuid + 'grid-row-' + recordId + '-col-' + dataIndex + (subRow != null ? ('-' + subRow) : '');
        },

        getCellLabels : function (recordId, dataIndex, group, skip) {
            var headers = [];
            if (!_.isUndefined(_.where(this.view.widget.options.layout.rowsets, { id: this.view.widget._getRecordRowset(recordId) })[0].header)) {
                headers.push(this.view.widget.uuid + '-' + 'rowsetId-' + this.view.widget._getRecordRowset(recordId));
            }
            if (this.view.descriptionIds.cellState.rowHeaderLabel && !skip) {
                headers.push(this.view.descriptionIds.cellState.rowHeaderLabel);
            }
            for (var i = 0; i < this.view.rowHeaders.length; i++) {
                var rowHeader = this.view.rowHeaders[i];

                headers.push(this.view.getCellId(recordId, rowHeader));
            }
            if (dataIndex !== constants.MENU_BUTTON_COLUMN) {
                if (this.view.descriptionIds.cellState.columnHeaderLabel && !skip) {
                    headers.push(this.view.descriptionIds.cellState.columnHeaderLabel);
                }
                headers.push(this.view.getHeaderId(dataIndex));


                if (this.view.descriptionIds.cellState.cellLabel && !skip) {
                    headers.push(this.view.descriptionIds.cellState.cellLabel);
                }
            }
            return headers.join(' ');
        },
        getGroupCellLabels : function (recordId, dataIndex) {
            var headers = [];
            if (this.view.descriptionIds.cellState.groupRowHeaderLabel) {
                headers.push(this.view.descriptionIds.cellState.groupRowHeaderLabel);
            }
            for (var i = 0; i < this.view.rowHeaders.length; i++) {
                var rowHeader = this.view.rowHeaders[i];

                headers.push(this.widget.uuid + 'grid-group-' + recordId + '-col-' + rowHeader);
            }
            if (dataIndex !== constants.MENU_BUTTON_COLUMN) {
                if (this.view.descriptionIds.cellState.columnHeaderLabel) {
                    headers.push(this.view.descriptionIds.cellState.columnHeaderLabel);
                }
                headers.push(this.view.getHeaderId(dataIndex));


                if (this.view.descriptionIds.cellState.cellLabel) {
                    headers.push(this.view.descriptionIds.cellState.cellLabel);
                }
            }
            return headers.join(' ');
        },

        getCellDescriptions : function (dataIndex, isDummyRecord) {
            if (dataIndex === constants.MENU_BUTTON_COLUMN) {
                return isDummyRecord ? [this.view.descriptionIds.cellState.quickAddButtonDescription] : [this.view.descriptionIds.cellState.menuButtonDescription];
            }
        },

        getRenderer : function (args) {
            if (args.group.showStackedCells) {
                if (!args.column.stackedCellHeader) {
                    return this.view.columnDefinitions[args.subRow].renderer;
                }
            }
            var columnDef = this.view.getCombinedDefinition(args);
            return columnDef.renderer;
        },

        checkReadOnly : function (column, args) {
            if (column.readOnly) {
                return true;
            }
            if (_.includes(args.recordAttributes.readOnly, column.dataIndex)) {
                return true;
            }
            if (!args.recordAttributes.editable) {
                return true;
            }
            if (args.recordAttributes.rollup && column.rollupRole != null) {
                return true;
            }
            if (args.group.timephased) {
                if (args.column.readOnly) { // Bucket column is readOnly
                    return true;
                }
                // Stacked Row is readOnly for this row
                if (_.includes(args.recordAttributes.timephased.readOnlyStacked, column.dataIndex)) {
                    return true;
                }
                // Bucket is readOnly for this row
                if (_.includes(args.recordAttributes.timephased.readOnlyBuckets, args.column.bucketId)) {
                    return true;
                }
                // Bucket is read only for this subRow
                if (args.recordAttributes.timephased.readOnlyMatrix.bySubRow[column.dataIndex] != null &&
                        _.includes(args.recordAttributes.timephased.readOnlyMatrix.bySubRow[column.dataIndex], args.column.bucketId)) {
                    return true;
                }
                // SubRow is readOnly for this bucket
                if (args.recordAttributes.timephased.readOnlyMatrix.byBucket[args.column.bucketId] != null &&
                        _.includes(args.recordAttributes.timephased.readOnlyMatrix.byBucket[args.column.bucketId], column.dataIndex)) {
                    return true;
                }
            } else if (args.group.showStackedCells) {
                if (args.column.readOnly) { // Bucket column is readOnly
                    return true;
                }
                // Sub Row is readOnly for this row
                if (_.includes(args.recordAttributes.stackedCells.readOnlyStacked, column.dataIndex)) {
                    return true;
                }
                // Column is readOnly for this row
                if (_.includes(args.recordAttributes.stackedCells.readOnlyColumns, args.column.dataIndex)) {
                    return true;
                }
                // Column is read only for this subRow
                if (args.recordAttributes.stackedCells.readOnlyMatrix.bySubRow[column.dataIndex] != null &&
                        _.includes(args.recordAttributes.timephased.readOnlyMatrix.bySubRow[column.dataIndex], args.column.dataIndex)) {
                    return true;
                }
                // SubRow is readOnly for this bucket
                if (args.recordAttributes.stackedCells.readOnlyMatrix.byColumn[args.column.dataIndex] != null &&
                        _.includes(args.recordAttributes.timephased.readOnlyMatrix.byColumn[args.column.dataIndex], column.dataIndex)) {
                    return true;
                }
            }


            return false;
        },

        checkModified : function (column, args) {
            if (args.group.timephased) {
                return args.recordAttributes.timephased != null && args.recordAttributes.timephased.modified[args.column.bucketId] != null &&
                        _.includes(args.recordAttributes.timephased.modified[args.column.bucketId], column.dataIndex);
            } else if (args.group.showStackedCells && args.recordAttributes.stackedCells != null) {
                return args.recordAttributes.stackedCells.modified[args.column.dataIndex] != null &&
                        _.includes(args.recordAttributes.stackedCells.modified[args.column.dataIndex], column.dataIndex);
            } else {
                return _.includes(args.recordAttributes.modified, column.dataIndex);
            }
        },


        checkLocked : function (column, args) {
            if (column.locked) {
                return true;
            }
            if (_.includes(args.recordAttributes.locked, column.dataIndex)) {
                return true;
            }

            if (args.group.timephased) {
                if (args.column.locked) { // Bucket column is readOnly
                    return true;
                }
                // Stacked Row is readOnly for this row
                if (args.recordAttributes.timephased.lockedStacked != null && _.includes(args.recordAttributes.timephased.lockedStacked, column.dataIndex)) {
                    return true;
                }
                // Bucket is readOnly for this row
                if (args.recordAttributes.timephased.lockedBuckets != null && _.includes(args.recordAttributes.timephased.lockedBuckets, args.column.bucketId)) {
                    return true;
                }
            } else if (args.group.showStackedCells) {
                if (args.column.locked) { // Bucket column is readOnly
                    return true;
                }
                // Sub Row is readOnly for this row
                if (args.recordAttributes.stackedCells.lockedStacked != null && _.includes(args.recordAttributes.stackedCells.lockedStacked, column.dataIndex)) {
                    return true;
                }
                // Column is readOnly for this row
                if (args.recordAttributes.stackedCells.lockedColumns != null && _.includes(args.recordAttributes.stackedCells.lockedColumns, args.column.dataIndex)) {
                    return true;
                }
            }


            return false;
        },


        getInvalid : function (column, args) {
            var invalid = null;
            if (args.group.timephased || args.group.showStackedCells) {
                invalid = _.get(args.recordAttributes, 'timephased.invalid.' + args.column.dataIndex + '$$' + column.dataIndex, null);
                if (invalid == null) {
                    invalid = _.get(args.recordAttributes, 'invalid.' + args.column.dataIndex + '$$' + column.dataIndex, null);
                }
            } else {
                invalid = _.get(args.recordAttributes, 'invalid.' + column.dataIndex, null);
            }
            return invalid;
        },


        makeCell : function (args) {
            var that = this;
            var column = this.view.getCombinedDefinition(args);
            var expandable = (column.dataIndex === that.view.expandableDataIndex);
            var isDummyRecord = _.get(args, 'isDummyRecord', false);
            var renderOptions = {
                isDummyRecord : isDummyRecord,
                cellHeight : args.cellHeight,
                attributes : args.recordAttributes,
                column : column,
                record : args.record,
                recordId : args.recordId,
                isGroup : args.isGroup,
                showQuickAdd : args.showQuickAdd,
                modified : this.checkModified(column, args),
                readOnly : this.checkReadOnly(column, args),
                editable : this.options.editable && !this.options.disabled,
                focused : args.recordAttributes.focused,
                selected : column.dataIndex === that.view.selectionModel.focus.columnId,
                clickableNode : expandable && args.recordAttributes.isNode && this.options.tree,
                expandable : expandable,
                hasChildren : args.recordAttributes.hasChildren,
                groupBandClasses : that.view.groupBandClasses,
                cellSelectedClasses : args.recordAttributes.cellSelectedClasses,
                found : _.includes(args.recordAttributes.found, column.dataIndex),
                searcher : column.stringReplacer ? column.stringReplacer : this.widget.stringReplacer, // this.widget.stringReplacer, // LUX-506
                matcher: column.searchMatcher ? column.searchMatcher : this.widget.searchMatcher,
                modifiedRow : column.dataIndex === that.view.firstVisibleColumn && args.recordAttributes.modified.length,
                invalid : this.getInvalid(column, args),
                rowsetId : args.rowsetId,
                groupId : args.group.id,
                printable : args.printable,
                addSpacer : expandable && (that.options.tree || that.widget.groupingApplied),
                locked : this.checkLocked(column, args),
                markup : this.options.markup,
                id : this.view.getCellId(args.recordId, args.column.dataIndex, args.subRow),
                labels : this.getCellLabels(args.recordId, args.column.dataIndex),
                descriptionIds : this.view.descriptionIds,
                customAttributes : args.recordAttributes.custom || {},
                locale : this.options.locale,
                cellLocationClasses : '',
                cutCopyClasses : this.widget.clipboard.cellIndex === column.dataIndex ? args.recordAttributes.cutCopyClasses : '',
                describedBy : this.getCellDescriptions(column.dataIndex, isDummyRecord),
                lastRecordAtLevel : args.lastRecordAtLevel,
                cellToolbarActions : this.getCellToolbarOptions(args),
                iconMarkup : this.getCellIconMarkup(args),
                dataIndex : args.column.dataIndex,
                subRow : args.subRow,
                quickAddHint : args.quickAddHint,
                timephased : args.group.timephased,
                autoEscape : column.escaping.autoEscapeRender,
                escapeFn : column.escaping.escape
            };
            renderOptions.data = this.getCellData(args, renderOptions);

            if (args.node != null) {
                renderOptions.expanded = args.node.expanded != null ? args.node.expanded : false;
                renderOptions.level = args.node.level;
                renderOptions.loading = args.node.loading;
                args.node.loaded = args.recordAttributes.childrenLoaded;

                if (!renderOptions.hasChildren) { // Reset expanded if there are no children
                    args.node.expanded = false;
                    renderOptions.expanded = false;
                }
            }

            if (args.column.dataIndex === that.view.firstVisibleColumn) {
                renderOptions.cellLocationClasses += 'first-visible-cell ';
            }
            if (args.column.dataIndex === that.view.lastVisibleColumn) {
                renderOptions.cellLocationClasses += 'last-visible-cell ';
            }

            if (args.recordAttributes.editable != null && renderOptions.editable) {
                renderOptions.editable = args.recordAttributes.editable;
            }

            if (args.group.timephased) {
                renderOptions.timephasedColumnDefinition = args.column;
            }

            if (that.view.canDragRows() && args.recordAttributes.draggable) {
                if (args.column.dataIndex === that.view.firstVisibleColumn && !that.options.tree) {
                    if (that.view.firstVisibleColumn === that.view.lastVisibleColumn) {
                        renderOptions.grabber = 'both';
                    } else {
                        renderOptions.grabber = 'before';
                    }
                } else if (args.column.dataIndex === that.view.lastVisibleColumn) {
                    renderOptions.grabber = 'after';
                }
            }

            return this.getRenderer(args).create(renderOptions); // selected column
        },

        getGroupCellIconMarkup : function (args) {
            var iconMarkup = '';
            if (_.get(args, 'recordGroup.groupConfig.iconMarkup') != null) {
                iconMarkup = args.recordGroup.groupConfig.iconMarkup;
            } else if (_.get(args, 'recordGroup.groupConfig.getIconMarkup') != null) {
                iconMarkup = args.recordGroup.groupConfig.getIconMarkup({
                    group : args.recordGroup,
                    groupConfig : args.recordGroup.groupConfig
                });
            }
            return iconMarkup;
        },

        getCellIconMarkup : function (args) {
            var iconMarkup = null;
            if (args.recordAttributes.dataIcons != null) {
                iconMarkup = args.recordAttributes.dataIcons[args.column.dataIndex] || args.recordAttributes.dataIcons.default;
            }
            if (iconMarkup == null) {
                iconMarkup = this.options.recordIcons[args.recordAttributes.type] || this.options.recordIcons.default;
            }
            if (iconMarkup == null) {
                iconMarkup = '';
            }
            return iconMarkup;
        },

        getCellToolbarOptions : function (args) {
            var cellToolbarActions = null;
            if (this.options.cellToolbar.enabled && ((this.options.cellToolbar.column || this.view.firstVisibleColumn) === args.column.dataIndex)) {
                cellToolbarActions = [];
                var actionOrder = this.options.cellToolbar.actionOrder;
                for (var i = 0; i < actionOrder.length; i++) {
                    var action = this.options.cellToolbar.actions[actionOrder[i]];
                    if (action.enabled) {
                        if (action.gridAction) {
                            if (this.widget._checkAction(action.trigger, this.widget._preCheckActions([args.record], args.record))) {
                                cellToolbarActions.push({
                                    dataTrigger : 'data-grid-trigger',
                                    trigger : action.trigger,
                                    title : action.title || this.options.locale['cellToolbar_' + action.trigger],
                                    icon : action.iconClass
                                });
                            }
                        } else if (args.recordAttributes.cellToolbar[action.trigger]) {
                            cellToolbarActions.push({
                                dataTrigger : 'data-trigger',
                                trigger : action.trigger,
                                title : action.title,
                                icon : action.iconClass
                            });
                        }
                    }
                }
            }
            return cellToolbarActions;
        },

        getCellData : function (args, renderOptions) {
            var data = null;
            if (args.column.dataIndex === constants.ROW_SELECTOR) {
                return args.recordAttributes.checked;
            }
            if (args.group.showStackedCells && args.column.stackedCellHeader) {
                return this.view.columnDefinitions[args.subRow].title;
            }
            if (!args.column.displayColumn) {
                if (args.group.timephased) {
                    data = this.widget._getTimephasedValueFromRecord({
                        recordId : args.recordId,
                        record : args.record,
                        dataIndex : args.subRow,
                        bucketId : args.column.bucketId,
                        granularity : args.column.granularity
                    });
                    renderOptions.timephasedData = this.widget._getTimephasedDataFromRecord({
                        recordId : args.recordId,
                        record : args.record,
                        dataIndex : args.subRow,
                        bucketId : args.column.bucketId,
                        granularity : args.column.granularity
                    });
                    if (this.widget.options.timephased.hideZeroValues && data === 0) {
                        data = null;
                    }
                } else if (args.group.showStackedCells) {
                    data = this.widget._getStackedValueFromRecord(args.record, args.column.dataIndex, args.subRow);
                } else {
                    data = this.widget.store.getValueFromRecord(args.record, args.column.dataIndex);
                }

                if (args.recordAttributes.rollup && args.column.rollupRole != null && this.widget.options.dataRollup.enabled) {
                    if (args.group.timephased) {
                        data = _.get(this.widget.dataCache.getRecordCache(args.recordId).timephasedRollupValues, args.column.granularity + '-' + args.column.bucketId + '.' + args.subRow);
                        if (this.widget.options.timephased.hideZeroValues && data === 0) {
                            data = null;
                        }
                        renderOptions.timephasedRollupValues = this.widget.dataCache.getRecordCache(args.recordId).timephasedRollupValues;
                    } else if (args.group.showStackedCells) {
                        renderOptions.stackedRollupValues = this.widget.dataCache.getRecordCache(args.recordId).stackedRollupValues;
                        data = renderOptions.stackedRollupValues[args.column.dataIndex + '-' + args.subRow];
                    } else {
                        renderOptions.rollupValues = this.widget.dataCache.getRecordCache(args.recordId).rollupValues;
                        data = renderOptions.rollupValues[args.column.dataIndex];
                    }
                }
            }

            return data;
        },

        getGroupCellData : function (args, renderOptions) {
            var data = null;
            var recordGroup = this.widget.recordGrouper.recordGroups[args.recordId];
            recordGroup.grouping = this.options.grouping;
            if (args.column.dataIndex === constants.ROW_SELECTOR) {
                data = args.recordAttributes.checked;
            } else if (args.group.showStackedCells && args.column.stackedCellHeader) {
                return this.view.columnDefinitions[args.subRow].title;
            } else if (this.view.expandableDataIndex === args.column.dataIndex) {
                var columnDefinition = this.view.columnDefinitions[recordGroup.dataIndex];

                var columnTitle = columnDefinition != null ? columnDefinition.title : '';
                var formattedValue = '';

                if (renderOptions.forExport && _.has(args, 'record.groupConfig.formatExcelValue')) {
                    formattedValue = args.record.groupConfig.formatExcelValue(args, renderOptions);
                } else if (recordGroup.formatValue != null) {
                    formattedValue = recordGroup.formatValue(recordGroup.dataValue, recordGroup, this.view.columnDefinitions[recordGroup.dataIndex], this.options.locale);
                } else {
                    formattedValue = this.view.columnDefinitions[recordGroup.dataIndex].renderer.format(_.assign(_.clone(renderOptions), {
                        column : this.view.columnDefinitions[recordGroup.dataIndex],
                        data : recordGroup.dataValue
                    }), recordGroup.dataValue);
                }
                if (formattedValue === '' || formattedValue == null) {
                    formattedValue = recordGroup.noGroupValue;
                }
                if (this.options.grouping.showColumnTitle && !recordGroup.hideColumnTitle) {
                    formattedValue = columnTitle + ' - ' + formattedValue;
                }
                renderOptions.data = formattedValue;
                renderOptions.formattedValue = formattedValue;
                recordGroup.displayName = formattedValue;
                renderOptions.clickableNode = true;
                if (recordGroup.isRollupGroup) {
                    renderOptions.clickableNode = false;
                }
            } else if (args.column.rollupRole != null || args.column.internalDisplayOnly || args.column.timephased || args.column.useStackedContext) {
                if (!args.column.displayColumn) {
                    if (args.group.timephased) {
                        data = _.get(recordGroup.timephasedRollupValues, args.column.granularity + '-' + args.column.bucketId + '.' + args.subRow);
                        if (this.widget.options.timephased.hideZeroValues && data === 0) {
                            data = null;
                        }
                        renderOptions.timephasedRollupValues = _.get(recordGroup.timephasedRollupValues, args.column.granularity + '-' + args.column.bucketId);
                    } else if (args.group.showStackedCells) {
                        data = recordGroup.stackedRollupValues[args.column.dataIndex + '-' + args.subRow];
                    } else {
                        data = recordGroup.rollupValues[args.column.dataIndex];
                    }
                }
            }
            return data;
        },

        getGroupCellRenderer : function (args) {
            var renderer = new textColumn.renderer();
            if (args.column.internalDisplayOnly) {
                renderer = args.column.renderer;
            } else if (args.column.rollupRole != null) {
                if (!args.column.displayColumn) {
                    renderer = args.column.renderer;
                }
            } else if (args.group.showStackedCells) {
                if (!args.column.stackedCellHeader) {
                    renderer = this.view.columnDefinitions[args.subRow].renderer;
                }
            }
            return renderer;
        },


        makeGroupCell : function (args, tacos, node, column, recordAttributes, rowsetId, groupId, printable, lastRecordAtLevel) {
            var that = this,
                recordGroup = that.widget.recordGrouper.recordGroups[args.recordId];
            args.recordGroup = recordGroup;
            column = this.view.getCombinedDefinition(args);
            var expandable = (column.dataIndex === that.view.expandableDataIndex);
            var renderOptions = {
                cellHeight : args.cellHeight,
                attributes : recordAttributes,
                column : column,
                modified : false,
                readOnly : true,
                selected : false,
                expandable : expandable,
                hasChildren : recordGroup.records.length > 0 || recordGroup.childGroups.length > 0 || recordGroup.hierarchyGroups.length > 0,
                expanded : node != null ? node.expanded : false,
                level : node != null ? node.level : null,
                groupBandClasses : that.view.groupBandClasses,
                cellSelectedClass : this.options.classes.cellSelected,
                found : _.includes(args.recordAttributes.found, args.column.dataIndex),
                matcher: this.widget.searchMatcher,
                rowsetId : rowsetId,
                groupId : groupId,
                printable : printable,
                images : this.options.images,
                markup : this.options.markup,
                addSpacer : expandable && (that.options.tree || that.widget.groupingApplied),
                id : this.view.getCellId(args.recordId, args.column.dataIndex, args.subRow),
                labels : this.getGroupCellLabels(args.recordId, column.dataIndex) + '',
                locale : this.options.locale,
                descriptionIds : this.view.descriptionIds,
                describedBy : this.getCellDescriptions(column.dataIndex),
                customAttributes : args.recordAttributes.custom || {},
                colSpan : this.view.expandableDataIndex === column.dataIndex ? column.colSpan : null,
                lastRecordAtLevel : lastRecordAtLevel,
                dataIndex : args.column.dataIndex,
                subRow : args.subRow,
                record : recordGroup,
                timephased : args.group.timephased,
                cellLocationClasses : '',
                isGroup : true,
                iconMarkup : this.getGroupCellIconMarkup(args)
            };
            renderOptions.data = this.getGroupCellData(args, renderOptions);

            if (args.group.timephased) {
                renderOptions.timephasedColumnDefinition = args.column;
            }

            if (!renderOptions.hasChildren && node != null) { // Reset expanded if there are no children
                node.expanded = false;
                renderOptions.expanded = false;
            }

            return this.getGroupCellRenderer(args, renderOptions).create(renderOptions); // selected column
        }
    };

    return RowRenderer;
});
