/*global define */
/**
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * This base widget is inspired from knockout's component bindings. Knockout's component binding allows for clear distinction between
 * the ViewModel and View (tempalte) and provides for clear MVVM pattern in the compoent. jQueryUI widget factory on the other hand does
 * not provide this advantage. Developers who want to create jQueryUI widgets (with all jQueryUI standards) but still like Knockout's
 * MVVM, can inherit their widget from this as a base widget.
 *
 * Implimentation details:
 * ViewModel's constructor is put on widget_factory's prototype, this would insure only one instace of the VM constructor.
 * VM's observables are put on widget_factory_options object, this would insure instance specific observables for VM
 * Note that
 * ko.applyBindings(vmObjInstance, tem ); ..........is called by widget_Factory, so WidgetFactory has the resposibility
 * of storing VM's state, and only way that WidgetFactory can store any state is in it's options obejct.
 *
 * @author Shailesh Zirpe
 */

define(['jquery',
    'underscore',
    'knockout',
    'jqueryui-amd/widget'
],
function($, _, ko) {

    $.widget('GBU.koWidget', {
        options: {
            /*
                 * widget factory creates options object per widget not on the prototype of the widget,
                 * therefore this is good place to put widget's private observables
                 */
            pvtObservables: {}
        },
        _create: function() {
            /*
                 * widgetObj will be the one stored on element's data-* , widgetObj would have it's instance specific options obj.
                 */
            var widgetObj = this;
            widgetObj.uuid = 'widgetId_' + this.uuid;
            var vmObj = widgetObj._vm();
            _.each(widgetObj._vm_prototype, function(value, key, list) {
                vmObj.constructor.prototype[key] = _.bind(value, vmObj);
            }, widgetObj);
            vmObj.widgetObj = widgetObj;
            vmObj.options = widgetObj.options;

            widgetObj.options.pvtObservables = vmObj;
            widgetObj._apply_bindings(vmObj);
            this._super();
        },
        _apply_bindings: function(vmObjInstance) {
            var widgetObj = this;
            //myLocalRequire(['text!./luxbreadcrumbs_template.html'], function(template){          //using local scoped require for relative path mapping.
            require([widgetObj._template], function(template) {
                var tem = $(template).addClass('myClass').get(0);
                widgetObj.options.pvtObservables = vmObjInstance;
                ko.applyBindings(vmObjInstance, tem);
                var node = widgetObj.element.append(tem);
            });
        }
    });

});
