/**
 * Copyright (c) 2016 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements some useful Knockout custom bindings for LUX use.
 *
 * @author Daniel Hoang
 */

define(['knockout'], function(ko) {

    /*
     * Custom binding called luxAllowBindings that allows descendant bindings to be applied only if its value
     * is true. If the value is false, then allowBindings tells Knockout that it is responsible for
     * descendant bindings so they won’t be bound as usual.
     * Taken from http://knockoutjs.com/documentation/custom-bindings-controlling-descendant-bindings.html
     */
    ko.bindingHandlers.luxAllowBindings = {
        init: function(elem, valueAccessor) {
            // Let bindings proceed as normal *only if* my value is false
            var shouldAllowBindings = ko.unwrap(valueAccessor());
            return {controlsDescendantBindings: !shouldAllowBindings};
        }
    };

});
