/*
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements a default adapter for lux Dashboard pattern.
 * storeAdapter is the only module in luxDashboard pattern who  knows about the store (ojCollection),
 *  and instantiates the store and orchestrates the store.
 * @author Shailesh Zirpe
 *
 */
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'underscore',
    '../defaultStore/ctrlCollection',
    'ojs/ojknockout'
], function(oj, ko, $, _, CtrlCollection) {
    'use strict';

    var ctrlStoreAdapter = function(widgetOptions) {
        var self = this;
        var DeveloperExtendedCtrlCollection;
        // use customStore provided in widgetOptions.ctrlStore if don't want to use the defalutStore.
        if ((widgetOptions.ctrlStore === 'useDefault') || _.isUndefined(widgetOptions.ctrlStore)) {
            DeveloperExtendedCtrlCollection = CtrlCollection.extend(widgetOptions.defaultCtrlStoreExtender);
            self.ctrlStore = new DeveloperExtendedCtrlCollection();
        } else {
            self.ctrlStore = widgetOptions.ctrlStore;
        }

        //adapt the adapter using ctrlStoreAdapterOverrides if using customStore
        _.each(widgetOptions.ctrlStoreAdapterOverrides, function(value, key) {
            self[key] = value || self[key];
        });

    };

    var prototypeObj = {

        'initAdapter': function($deferred, defaultCtrlStoreURL) {
            var self = this;
            var $dfd = $.Deferred();
            var ctrlArray = ko.observableArray();
            $dfd.done(function(resolveWithCollection) {
                ctrlArray = oj.KnockoutUtils.map(self.ctrlStore, null, true);
                $deferred.resolve(ctrlArray);
            });

            // store kick-starter progresses from VM->adapter->store
            self.ctrlStore.setURL_and_fetch(defaultCtrlStoreURL, $dfd);
        },

        'update': function(tileId, key, newVal) {
            var self = this;
            var model = self.ctrlStore.get(tileId);
            model.set(key, newVal);
            model.save();
        }

    };

    $.extend(ctrlStoreAdapter.prototype, prototypeObj);
    return ctrlStoreAdapter;
});
