/*
 * Copyright (c) 2013 - 2018 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements a Dashboard pattern that can be used to organize and display contents in tiles.
 * @author Daniel Hoang, Shailesh Zirpe
 */
/* eslint-disable max-params */
/* eslint-disable max-statements */
// jshint maxstatements: 100
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'lodash',
    './utils/luxSubscriptionStoreFactory',
    './storeAdapter/metaStoreAdapter',
    './storeAdapter/ctrlStoreAdapter',
    './utils/tileFlipHelper',
    'knockout-mapping',
    'ojs/ojknockout',
    'ojs/ojmasonrylayout',
    'ojs/ojbutton',
    'ojs/ojmodel',
    'ojs/ojknockout-model',
    'ojs/ojdialog',
    'ojs/ojmenu',
    'ojs/ojcheckboxset',
    'promise',
    'ojs/ojlistview'
], function(oj, ko, $, _, SubscriptionStoreFactory, CR_metaStoreAdapter, CR_ctrlStoreAdapter, tileFlipHelper, komapping) {

    'use strict';

    var refresh_ojMasonryLayout_debounce = _.debounce(function(self_getMasonryLayoutId) {
        $('#' + self_getMasonryLayoutId).ojMasonryLayout('refresh');
        $('#' + self_getMasonryLayoutId).ojMasonryLayout('option', 'reorderHandle', '.oj-panel-drag-handle');
    }, 300);

    var refresh_ojListView_debounce = _.debounce(function(self_getListViewId) {
        $('#' + self_getListViewId).ojListView('refresh');
        if ($('#' + self_getListViewId).children(':not(.oj-listview-empty-text)').length){
            $('.oj-listview-empty-text').hide();
        }
    }, 100);

    var viewInfoAllUpdate_debounce = _.debounce(function($ele, tileId) {
        var ele = $ele.get(0);
        var params = {
            'bubbles': true,
            'detail':  {'value':  ele.value,
                'tileId': tileId
            }
        };
        ele.dispatchEvent(new CustomEvent('viewInfoAllUpdate', params));
    }, 100);

    var ViewModelMetaData = function(dbObj) {
        var self = this;
        var subStoreFactory = new SubscriptionStoreFactory();
        var startMetaAdapter = function(def, metaURL) {
            // 1) the adapter would own the store (ojCollection)
            self.metaStoreAdapter = new CR_metaStoreAdapter(self.options);
            // 2) initAdapter is store kick-starter,  resolve the $meta_deferred with metaArray
            self.metaStoreAdapter.initAdapter(def, metaURL);
        };
        var startCtrlAdapter = function(def, ctrlURL) {
            // 1) the adapter would own the store (ojCollection)
            self.ctrlStoreAdapter = new CR_ctrlStoreAdapter(self.options);
            // 2) initAdapter is store kick-starter,  resolve the $ctrl_deferred with ctrlArray
            self.ctrlStoreAdapter.initAdapter(def, ctrlURL);
        };

        // step 3 (step 1 is to create adapter, step2 is to kick-start loading the store)
        var metaStoreResolved = function(metaArray) {
            self.metaArray = metaArray;
            // Auctual metaArray is not used in ko foreach! Althoug this is a new array, The objects in this array
            // are references to the original metaArray which means, if you update a property in this object, metaArray
            // is updated and store.update is fired!
            self.filteredMetaArray = ko.utils.arrayFilter(self.metaArray(), function(obj) {
                return (obj.inserted() === true);
            });
            // metaArray should be sorted by store, but this is done for consistency after clien side re-ordering and re-render
            self.filteredMetaArray = self.filteredMetaArray.sort(function(left, right) {
                return left.order() === right.order() ? 0 : (left.order() < right.order() ? -1 : 1);
            });

            // clean previously created subscriptions. Remember metaStoreResolved can happen more than once
            if (!_.isUndefined(self.meta_subStore)) {
                self.meta_subStore.setWriteBack(false);
            }
            // No subscriptions are created at this point. It only happens when user sets the writeBack option to true.
            self.meta_subStore = subStoreFactory.createSubscriptionStore(self.metaArray, self.metaStoreAdapter, 'tileId');
            // create subscriptions if writeBack is true
            self.meta_subStore.setWriteBack(self.metaWriteBack());
        };
        var ctrlStoreResolved = function(ctrlArray) {
            self.ctrlArray = ctrlArray;
            // clean previously created subscriptions. Remember ctrlStoreResolved can happen more than once
            if (!_.isUndefined(self.ctrl_subStore)) {
                self.ctrl_subStore.setWriteBack(false);
            }
            // 3) No subscriptions are created at this point. It only happens when user sets the writeBack option to true.
            self.ctrl_subStore = subStoreFactory.createSubscriptionStore(self.ctrlArray, self.ctrlStoreAdapter, 'tileId');
            self.ctrl_subStore.setWriteBack(self.ctrlWriteBack());
        };
        var bothCallsComplete = function(metaArray, ctrlArray) {
            self.dataReady(true);
            //tileFlipped event
            var ele = dbObj.element.get(0);
            var params = {
                'bubbles': true,
                'detail':  {
                    uuid: self.dbObj.uuid
                }
            };
            ele.dispatchEvent(new CustomEvent('dataReady', params));
        };
        var $meta_deferred = $.Deferred();
        var $ctrl_deferred = $.Deferred();
        $meta_deferred.done(metaStoreResolved); // 3.1) $meta_deferred.done is resolved by adapter with metaArray
        $ctrl_deferred.done(ctrlStoreResolved); // 3.1) $ctrl_deferred.done is resolved by adapter with ctrlArray

        self.dbObj = dbObj;
        self.handledCollapse = false;
        /* Need to declare metaWriteBack as own observable of this VM (can not hook into app VM, becuase there might
         * not be one if they used standard widgetFactory syntax and not luxBindingBrige)
         * Set initial values of this obserable
         */
        self.metaWriteBack = ko.observable(self.options.metaWriteBack);
        self.defaultMetaStoreURL = ko.observable(self.options.defaultMetaStoreURL);
        // ctrl info
        self.ctrlWriteBack = ko.observable(self.options.ctrlWriteBack);
        self.defaultCtrlStoreURL = ko.observable(self.options.defaultCtrlStoreURL);

        self.orderObserver = ko.observable(self.options.orderObserver);

        self.dataReady = ko.observable(false);
        self.metaArray = ko.observableArray();
        self.ctrlArray = ko.observableArray();

        // step1) kick start
        startMetaAdapter($meta_deferred, self.defaultMetaStoreURL());
        startCtrlAdapter($ctrl_deferred, self.defaultCtrlStoreURL());

        // time to turn on bindings
        $.when($meta_deferred, $ctrl_deferred).done(bothCallsComplete);

        self.reloadMetaStore = function(metaURL) {
            var $metaDeferred = $.Deferred();
            var $ctrlDeferred = $.Deferred();

            $('.childOfPreventChildrenBinding').each(function(i, ele){
                //debugger;
                ko.cleanNode(ele);
                $(ele).empty();
            });
            self.dataReady(false);

            $metaDeferred.done(metaStoreResolved);
            $ctrlDeferred.done(ctrlStoreResolved);

            startMetaAdapter($metaDeferred, metaURL); // true resolve
            $ctrlDeferred.resolve(self.ctrlArray); //fake resolve with the array that you already have (no trip adapter and server)

            $.when($metaDeferred, $ctrlDeferred).done(bothCallsComplete);
        };

        self.reloadCtrlStore = function(ctrlURL) {
            var $metaDeferred = $.Deferred();
            var $ctrlDeferred = $.Deferred();
            self.dataReady(false);

            $metaDeferred.done(metaStoreResolved);
            $ctrlDeferred.done(ctrlStoreResolved);

            $metaDeferred.resolve(self.metaArray); //fake resolve with the array that you already have (no trip adapter and server)
            startCtrlAdapter($ctrlDeferred, ctrlURL); // true resolve

            $.when($metaDeferred, $ctrlDeferred).done(bothCallsComplete);
        };

        self.reloadStores = function() {
            // Use the stores we cached in the component, and that consumers
            // can modify from their application's viewModel.
            var _ctrlStore = self.properties.ctrlStore;
            var _metaStore = self.properties.metaStore;

            // Update the component's internal stores
            self._internalStoreUpdate(_ctrlStore, _metaStore);

            // Below is basically the same code as in the old widget's reloadBothStores API.
            var $metaDeferred = $.Deferred();
            var $ctrlDeferred = $.Deferred();
            self.dataReady(false);

            $metaDeferred.done(metaStoreResolved);
            $ctrlDeferred.done(ctrlStoreResolved);

            startMetaAdapter($metaDeferred, self.options.defaultMetaStoreURL); // true resolve
            startCtrlAdapter($ctrlDeferred, self.options.defaultCtrlStoreURL); // true resolve

            $.when($metaDeferred, $ctrlDeferred).done(bothCallsComplete);
        };

        self.defaultMetaStoreURL.subscribe(function(newVal) { // option update 3
            self.reloadMetaStore(newVal);
        });

        self.defaultCtrlStoreURL.subscribe(function(newVal) {
            self.reloadCtrlStore(newVal);
        });

        // subscriptions for meta
        self.metaWriteBack.subscribe(function(newVal) {
            self.meta_subStore.setWriteBack(newVal);
        });
        // subscriptions for Ctrl
        self.ctrlWriteBack.subscribe(function(newVal) {
            self.ctrl_subStore.setWriteBack(newVal);
        });

        self.getAllTileIds = function() {
            return _.pluck(ko.toJS(self.metaArray), 'tileId');
        };
        self.getInsertedTileIds = function() {
            return _.pluck(ko.toJS(self.filteredMetaArray), 'tileId');
        };

        // dashboardTileAdder pattern
        self.openTheDialogLazyLoader = ko.observable(false);
        self.dashboardTileAdder_buttonClick = function(data, event) {
            $('#' + self.getTileAdderDialogId()).ojDialog('open');
            self.openTheDialogLazyLoader(true);
            //refresh_ojListView_debounce(self.getListViewId());
        };
        self.closeDialogEvent = function() {
            self.openTheDialogLazyLoader(false);
        };

        self.ctrlInfoCurrentStateOfCtrlDialog = {};
        self.ConfigureTileDialogLazyLoder = ko.observable(false);
        self.ctrlInfo_bothEnable = false;
        self.ctrlInfo_both = false;
        self.ctrlInfo_stateValidation = false;

        self.getConfigureTileDialogId = function() {
            var self = this;
            return 'dashUUID' + self.dbObj.uuid + '-ConfigureTileDialogId';
        };

        self.closeConfigureTileDialog = function(event, ui) {
            self.ConfigureTileDialogLazyLoder(false);
            self.ctrlInfoCurrentStateOfCtrlDialog = {};
            //only computed observables have subscriptions, normal observables can not be disposed!
            if (ko.isComputed(self.ctrlInfo_bothEnable)) {
                self.ctrlInfo_bothEnable.dispose();
            }
            if (ko.isComputed(self.ctrlInfo_both)) {
                self.ctrlInfo_both.dispose();
            }
            if (ko.isComputed(self.ctrlInfo_stateValidation)) {
                self.ctrlInfo_stateValidation.dispose();
            }
        };

        //i18n  LUX-573
        self.setI18nStringsKOMapping = function(value){
            komapping.fromJS(value, self.i18nStrings);
            return komapping.toJS(self.i18nStrings);
        };

        self.i18nStrings = komapping.fromJS(self.options.translations); // LUX-573

        self.refreshLocale = function(newVal) {
            $.each(self.i18nStrings, function(key, obFn) {
                if (ko.isObservable(obFn)) {
                    obFn(oj.Translations.getTranslatedString('luxDashboard' + '.' + key));
                }
            });
            // get_tile_title & get_tile_description will not be updated since they are not observables
            self.dataReady(false);
            self.dataReady(true);
        };

        self.refreshLocaleAndMetaStore = function(metaURL) { // LUX-597
            $.each(self.i18nStrings, function(key, obFn) {
                if (ko.isObservable(obFn)) {
                    obFn(oj.Translations.getTranslatedString('luxDashboard' + '.' + key));
                }
            });
            // get_tile_title & get_tile_description will not be updated since they are not observables,
            // but self.dataReady would be turned on and off by reloadMetaStore.
            self.reloadMetaStore(metaURL);
        };

        self.get_tile_title = function(tileId) {
            if (self.options.titleDescriptionInMetaStore){ // LUX-597
                var obj =  _.findWhere(ko.toJS(self.metaArray), {'tileId': tileId()});
                return obj.title;
            } else { // look inside nls bundle
                var nlsStr = 'luxDashboard' + '.' + self.dbObj.element.attr('id') + '_' + tileId() + '_title';
                var val = oj.Translations.getTranslatedString(nlsStr);
                return (val === nlsStr) ? 'please define ' + val + ' in nls bundle' : val;
            }
        };
        self.get_tile_description = function(tileId) {
            if (self.options.titleDescriptionInMetaStore){ // LUX-597
                var obj =  _.findWhere(ko.toJS(self.metaArray), {'tileId': tileId()});
                return obj.description;
            } else { // look inside nls bundle
                var nlsStr = 'luxDashboard' + '.' + self.dbObj.element.attr('id') + '_' + tileId() + '_description';
                var val = oj.Translations.getTranslatedString(nlsStr);
                return (val === nlsStr) ? 'please define ' + val + ' in nls bundle' : val;
            }
        };
        self.contextMenuSingle = function(){
            // the template is in the 'script' tag, so it shouldn't be accounted for.
            return ($(document.body).find('#luxDashboardContextMenu').length === 0);
        };

    };

    var prototypeObj = {
        /*
         * exapand $data With more info
         * ctrlInfo: admin control related functionality
         * ViewInfo: so that it can be easily used by the bluePrint developer to react to tile state changes such has flipping etc.
         * events: tile level events to go up to widget
         */
        'expand_ko_data': function($data, $parent) {
            var self = $parent;
            // Part 1) $data.ctrlInfo
            // note that $data.ctrlInfo is not an observable, so even if, $data is in
            // context of metaArray, subscriptionStore won't fire updates on metaStore
            var b = _.pluck(ko.toJS($parent.ctrlArray), 'tileId');
            var i = _.indexOf(b, $data.tileId());
            var ctrlInfo = $parent.ctrlArray()[i];
            $data.ctrlInfo = ctrlInfo;

            // Part 2) $data.viewInfo
            $data.viewInfo = {};

            $data.viewInfo.normalView = ko.computed(function() {
                return (!!((!$data.expanded() && !$data.flipped())));
            });

            $data.viewInfo.flippedView = ko.computed(function() {
                return (!!((!$data.expanded() && $data.flipped())));
            });

            $data.viewInfo.expandedView = ko.computed(function() {
                return (!!(($data.expanded() && !$data.flipped())));
            });

            $data.viewInfo.flippedExpandedView = ko.computed(function() {
                return (!!(($data.expanded() && $data.flipped())));
            });

            // Check the context's busy state, so we can set up a computed task to handle
            // tile's view info update only when dashboard component is ready.
            var busyContext = oj.Context.getContext(self.ce).getBusyContext();
            busyContext.whenReady().then(function() {
                var temp = ko.computed(function() {
                    if (self.options !== undefined) {
                        var ele = self.dbObj.element.get(0);

                        // Update the tile viewInfoAll property via "setProperty" so application
                        // can getProperty when handling viewInfoAllUpdate event.
                        var val = ele.getProperty('viewInfoAll');
                        val[$data.tileId()] = {
                            'normalView':          $data.viewInfo.normalView(),
                            'flippedView':         $data.viewInfo.flippedView(),
                            'expandedView':        $data.viewInfo.expandedView(),
                            'flippedExpandedView': $data.viewInfo.flippedExpandedView()
                        };
                        ele.setProperty('viewInfoAll', val);

                        var params = {
                            'bubbles': true,
                            'detail':  {
                                'value':  ele.value,
                                'tileId': $data.tileId()
                            }
                        };
                        if (self.customBindingsApplied) {
                            // Send consumer the event so they can update the viewInfo accordingly.
                            ele.dispatchEvent(new CustomEvent('viewInfoAllUpdate', params));
                        }
                    }
                });
            });

            // Part 3) For eventing functinality
            $data.eventsChannel = self.options.eventsChannel;

            return $data;
        },

        // default value -> bluePrintPathPrefix: 'luxDashboardTileBluePrints'
        'getBluePrintId': function(bluePrintId) {
            var self = this;
            return self.options.bluePrintPathPrefix + bluePrintId();
        },

        //this is needed after defaultMetaStoreURL is changed. The foreach binding pulls new data, but the masonryLayout needs to be refreshed.
        'refresh_ojMasonryLayout': function(element, index, data) {
            var self = this;
            refresh_ojMasonryLayout_debounce(self.getMasonryLayoutId());
        },

        /*
         * 1)remove tile step1->  button click
         */
        'removeTile': function(data, event) {
            var target = event.target;
            var tile = $(target).closest('.lux-tile');
            var masonryLayout = tile.closest(':oj-masonrylayout');

            // beforeTileRemoved event
            var context = ko.contextFor(tile[0]);
            var ele = context.$parent.dbObj.element.get(0);
            var params = {
                'bubbles': true,
                'detail':  {
                    tileId: data.tileId()
                }
            };
            ele.dispatchEvent(new CustomEvent('beforeTileRemoved', params));

            masonryLayout.ojMasonryLayout('removeTile', '#' + tile.attr('id'));
        },

        /* 2)remove tile step2->
         * ojMasonryLayout remove event (Triggered immediately after a tile is removed.)
         * The idea is that, immediately after the tile is removed, set it's 'inserted=false', so that
         * the removed tile get's rendered in the tileAdder.
         */
        'handleRemove': function(event, ui) {
            var tile = ui.tile;
            var context = ko.contextFor(tile[0]);
            var $data = context.$data; // this is a good pattern for gettting to $data from $parent.handleRemove
            $data.inserted(false);
            // remove the tile from filteredMetaArray
            var b = _.pluck(ko.toJS(context.$parent.filteredMetaArray), 'tileId');
            var i = _.indexOf(b, $data.tileId());
            context.$parent.filteredMetaArray.splice(i, 1);
            context.$parent.orderObserverBackgroundProcess();

            //tileRemoved event
            var ele = context.$parent.dbObj.element.get(0);
            var params = {
                'bubbles': true,
                'detail':  {
                    tileId: $data.tileId(),
                    uuid:   context.$parent.dbObj.uuid
                }
            };
            ele.dispatchEvent(new CustomEvent('tileRemoved', params));
        },

        sorter: function(left, right) {
            var leftOrder = left.order();
            var rightOrder = right.order();
            return (leftOrder === rightOrder ? 0 : (leftOrder < rightOrder ? -1 : 1));
        },

        /*
         * inserting a tile
         * Can not use insterTile API, because I want to the tile to be derivative of array, that way I
         * can observe changes in the tile and fire REST calls.
         */
        'selectionListViewOptionChange': function(event, data) {
            var tile;
            var context;
            var $data;

            if (data.option === 'selection') {
                tile = $('#' + data.value[0]).find('.lux-tile');
                context = ko.contextFor(tile[0]);
                $data = context.$data;
                context.$parent.dataReady(false);
                context.$parent.filteredMetaArray.sort(this.sorter).push($data);
                $data.inserted(true);
                context.$parent.dataReady(true);
                refresh_ojMasonryLayout_debounce(context.$parent.getMasonryLayoutId());
                context.$parent.orderObserverBackgroundProcess();

                //tileInserted event
                var ele = context.$parent.dbObj.element.get(0);
                var params = {
                    'bubbles': true,
                    'detail':  {
                        tileId: $data.tileId(),
                        uuid:   context.$parent.dbObj.uuid
                    }
                };
                ele.dispatchEvent(new CustomEvent('tileInserted', params));
            }
        },

        /*
         * API for programtically inserting a tile.
         * Note: A tile must already be there as part of the metaArray. This mehtod only moves already present
         * tile into the filteredMetaArray.
        */
        'insertTile': function(options) {
            var self = this;
            var tileId = options.tileId;
            var index = options.index;
            var $data = ko.utils.arrayFirst(self.metaArray(), function(obj) {
                return (obj.inserted() === false && obj.tileId() === tileId);
            });

            try {
                if (!_.isNull($data)){
                    self.dataReady(false);
                    self.filteredMetaArray.sort(this.sorter).splice(index, 0, $data);
                    $data.inserted(true);
                    self.dataReady(true);
                    refresh_ojMasonryLayout_debounce(self.getMasonryLayoutId());
                    self.orderObserverBackgroundProcess();

                    //tileInserted event
                    var ele = self.dbObj.element.get(0);
                    var params = {
                        'bubbles': true,
                        'detail':  {
                            tileId: tileId,
                            uuid:   self.dbObj.uuid
                        }
                    };
                    ele.dispatchEvent(new CustomEvent('tileInserted', params));
                } else {
                    throw new Error('luxDashboard: insertTile error: tile already inserted or does not exist');
                }
            } catch (e) {
                oj.Logger.error(e);
            }

        },

        /*
         * resize tile 1) button click
         */
        'resizeTile': function(data, event) {
            var target = event.target;
            var tile = $(target).closest('.lux-tile');
            var masonryLayout = tile.closest(':oj-masonrylayout');
            var context = ko.contextFor(tile[0]);

            if (data.expanded()) {
                //data.expanded()=true -->  collapse the tile
                masonryLayout.ojMasonryLayout('resizeTile', '#' + tile.attr('id'), data.collapsedSizeClass());
                data.expanded(false);
            } else {
                //data.expanded()=false -->  expand the tile
                masonryLayout.ojMasonryLayout('resizeTile', '#' + tile.attr('id'), data.expandedSizeClass());
                data.expanded(true);
            }

            //tileResized event
            var ele = context.$parent.dbObj.element.get(0);
            var params = {
                'bubbles': true,
                'detail':  {
                    tileId: data.tileId(),
                    uuid:   context.$parent.dbObj.uuid
                }
            };
            ele.dispatchEvent(new CustomEvent('tileResized', params));
        },

        /*
         * Flipping a tile
         */
        'flipTile': function(data, event) {
            var target = event.target;
            var tile = $(target).closest('.lux-tile');
            var context = ko.contextFor(tile[0]);
            var flipDirection = 180;
            if (!tile.hasClass('lux-tile-flipping')) {
                tile.addClass('lux-tile-flipping');
                var rtl = tileFlipHelper.isRtl();
                if ((!rtl && data.flipped()) ||
                    (rtl && !data.flipped())) {
                    tileFlipHelper.flipPos180(tile);
                    flipDirection = 180; // event data
                } else {
                    tileFlipHelper.flipNeg180(tile);
                    flipDirection = -180; // event data
                }
            }

            //tileFlipped event
            var ele = context.$parent.dbObj.element.get(0);
            var params = {
                'bubbles': true,
                'detail':  {
                    tileId:        data.tileId(),
                    uuid:          context.$parent.dbObj.uuid,
                    flipDirection: flipDirection
                }
            };
            ele.dispatchEvent(new CustomEvent('tileFlipped', params));
        },

        'getDragHandleId': function(data) {
            var self = this;
            return self.getDashUniqueId() + '-tileId-' + data.tileId() + '-dragHandle';
        },

        'getDragHandleLabelledBy': function(index) {
            var self = this;
            return self.getDragHandleId(index) + ' ' + self.getLabelId(index);
        },

        /*
         * using 1-based ordering (set of positive intergers)
         */
        'handleReorder': function(e, ui) {
            var tile = ui.tile;
            var context = ko.contextFor(tile[0]);
            var $data = context.$data;
            var vm = context.$parent;
            vm.orderObserverBackgroundProcess();
        },

        // orderObserverBackgroundProcess
        'orderObserverBackgroundProcess': function() {
            var self = this;
            if (self.orderObserver()) {
                // observer the DOM order of tiles and assign it to $data associated with them
                $('#' + self.getMasonryLayoutId()).find('.lux-tile').each(function(i, tile) {
                    var context = ko.contextFor(tile);
                    var $data = context.$data;
                    $data.order(i + 1);
                });

                //set order zero for removed tiles
                var filteredMetaArrayIds = _.pluck(ko.toJS(self.filteredMetaArray), 'tileId');
                $.each(self.metaArray(), function(i, val) {
                    if (!_.contains(filteredMetaArrayIds, val.tileId())) {
                        val.order(0);
                    }
                });
            }
        },

        'openConfigureTileDialog': function($data, event) {

            var target = event.target;
            var tile = $(target).closest('.lux-tile');
            var context = ko.contextFor(tile[0]);
            var self = context.$parent;

            self.ctrlInfoCurrentStateOfCtrlDialog = $data.ctrlInfo;
            $data = $data; //LUX-380 - part 2

            self.ctrlInfo_bothEnable = ko.computed(function(){ //LUX-380 - part 1.1
                return (!!(self.ctrlInfoCurrentStateOfCtrlDialog.allow_flip() && self.ctrlInfoCurrentStateOfCtrlDialog.allow_resize()));
            });

            self.ctrlInfo_both = ko.computed(function(){ //LUX-380 - part 1.2
                if (!self.ctrlInfoCurrentStateOfCtrlDialog.allow_flip() || !self.ctrlInfoCurrentStateOfCtrlDialog.allow_resize()){
                    self.ctrlInfoCurrentStateOfCtrlDialog.allow_flipExpand(false);
                }
            });

            // flipping disabled while tile was flipped
            self.ctrlInfoCurrentStateOfCtrlDialog.allow_flip.subscribe(function(newVal) {
                if ($data.flipped() && !newVal){
                    $data.flipped(false);
                }
            });

            // resize disabled while tile was expanded
            self.ctrlInfoCurrentStateOfCtrlDialog.allow_resize.subscribe(function(newVal) {
                if ($data.expanded() && !newVal){
                    $('#' + self.getMasonryLayoutId()).ojMasonryLayout('resizeTile', '#' + self.getTileId($data.tileId()), $data.collapsedSizeClass());
                    $data.expanded(false);
                }
            });

            //allow_flipExpand disabled when tile was both flipped and expanded
            self.ctrlInfoCurrentStateOfCtrlDialog.allow_flipExpand.subscribe(function(newVal) {
                if ($data.flipped() && $data.expanded() && !newVal){
                    $data.flipped(false);
                    $('#' + self.getMasonryLayoutId()).ojMasonryLayout('resizeTile', '#' + self.getTileId($data.tileId()), $data.collapsedSizeClass());
                    $data.expanded(false);
                }
            });

            self.ConfigureTileDialogLazyLoder(true);
            $('#' + self.getConfigureTileDialogId()).ojDialog('open');
        },

        /*
         * Flipping a tile
         */
        'getFlipButtonId': function(data) {
            var self = this;
            return self.getDashUniqueId() + '-tileId-' + data.tileId() + '-flipButton';
        },

        /*
         * Flipping a tile
         */
        'getFlipButtonTitle': function(data) {
            var self = this;
            return (data.flipped() ? self.i18nStrings.flipForward() : self.i18nStrings.flipBack());
        },

        /*
         * Resize a tile
         */
        'getResizeButtonTitle': function(data) {
            var self = this;
            return (data.expanded() ? self.i18nStrings.resizeCollapse() : self.i18nStrings.resizeExpand());
        },

        /*
         * Flipping a tile
         */
        'getFlipButtonStartIcon': function(data) {
            return (data.flipped() ? 'oj-fwk-icon-caret-start' : 'oj-fwk-icon-caret-end') + ' oj-component-icon';
        },

        /*
         * Flipping a tile
         */
        'getFlipButtonCss': function(data) {
            return data.flipped() ? 'lux-tile-back-flip-icon' : 'lux-tile-front-flip-icon';
        },

        getCurrentSizeClass: function(data) {
            return (data.expanded() ? data.expandedSizeClass() : data.collapsedSizeClass());
        },

        'getDashUniqueId': function() {
            var self = this;
            if (self.dbObj.element.attr('id') !== ''){
                return 'dashId-' + self.dbObj.element.attr('id');
            } else {
                return 'dashUUID-' + self.dbObj.uuid;
            }
        },

        'getTileId': function(tileId) {
            var self = this;
            return self.getDashUniqueId() + '-tileId-' + tileId;
        },

        'getTileIdClassAndFrontBackClass': function(tileId, flipped) {
            var self = this;
            var frontBackClass = flipped ? 'lux-tile-back' : 'lux-tile-front';
            return self.getDashUniqueId() + '-tileId-' + tileId + ' ' + frontBackClass;
        },

        'getDragDropContextMenuId': function() {
            var self = this;
            return self.getDashUniqueId() + '-DragDropContextMenuId';
        },

        'getLabelId': function(index) {
            return 'label' + (index + 1);
        },

        'getResizeButtonId': function(data) {
            var self = this;
            return self.getDashUniqueId() + '-tileId-' + data.tileId() + '-resizeButton';
        },

        'getRemoveButtonId': function(data) {
            var self = this;
            return self.getDashUniqueId() + '-tileId-' + data.tileId() + '-removeButton';
        },

        'getButtonStartIcon': function(data) {
            return (data.expanded() ? 'oj-panel-collapse-icon' : 'oj-panel-expand-icon');
        },

        getMasonryLayoutId: function() {
            var self = this;
            return 'dashUUID' + self.dbObj.uuid + '-ojMasonryLayoutId';
        },

        getTileAdderDialogId: function() {
            var self = this;
            return 'dashUUID' + self.dbObj.uuid + '-tileAdderDialogId';
        },

        getListViewId: function() {
            var self = this;
            return 'dashUUID' + self.dbObj.uuid + '-ojListViewId';
        },

        getAddTilesButtonId: function() {
            var self = this;
            return 'dashUUID' + self.dbObj.uuid + '-dashboardTilesDialogButtonOpener';
        },
        getConfigureButtonId: function(data) {
            var self = this;
            return self.getDashUniqueId() + '-tileId-' + data.tileId() + '-configureButton';
        }

    };

    $.extend(ViewModelMetaData.prototype, prototypeObj);

    return ViewModelMetaData;
});
