
# LUX - Basic grid component

The **`<oj-lux-grid-basic>`** is a composite component with basic data grid functionality. The component provides an easy way for users to configure a data grid with the common functionalities out of the box.

Basic functionality includes:
- Various column data and display types
- Column filters
- Row selection
- Cell editing
- A basic toolbar
- Paging

## Configuration Requirements

The component has a dependency on the LUX library lux/v5.1.0, and you can download this version of the LUX library from the [GBU LUX Home](https://confluence.oraclecorp.com/confluence/display/gbulux/GBU+LUX+Home#GBULUXHome-ReleaseInformation) page.

For the grid to function it also expects several paths to be defined or referenced via a map through require.config. The example below shows a complete config.js with the relevant grid mappings:

<pre>
requirejs.config({
    baseUrl: 'js',
    // Path mappings for the logical module names
    paths:   {
        'knockout':             'libs/knockout/knockout-3.4.2.debug',
        'knockout-amd-helpers': 'libs/knockout/knockout-amd-helpers',
        'knockout-mapping':     'libs/knockout/knockout.mapping-latest.debug',
        'jquery':               'libs/jquery/jquery-3.3.1',
        'jqueryui-amd':         'libs/jquery/jqueryui-amd-1.12.1',
        'dform':                'libs/jquery/dform/dform',
        'dform.core':           'libs/jquery/dform/dform.core',
        'dform.converters':     'libs/jquery/dform/dform.converters',
        'dform.extensions':     'libs/jquery/dform/dform.extensions',
        'promise':              'libs/es6-promise/es6-promise.min',
        'hammerjs':             'libs/hammer/hammer-2.0.8.min',
        'ojdnd':                'libs/dnd-polyfill/dnd-polyfill-1.0.0.min',
        'ojs':                  'libs/oj/v5.1.0/min',
        'ojL10n':               'libs/oj/v5.1.0/ojL10n',
        'ojtranslations':       'libs/oj/v5.1.0/resources',
        'signals':              'libs/js-signals/signals.min',
        'text':                 'libs/require/text',
        'css':                  'libs/require-css/css',
        'customElements':       'libs/webcomponents/custom-elements.min',
        <strong style="color: #a11;">
        'underscore': 'libs/underscore/underscore-1.8.3-min',</strong>
        'lodash':     'libs/lodash/lodash',
        'moment':     'libs/moment/moment',
        <strong style="color: #a11;">
        // LUX CCA
        'oj-lux':     'libs/lux/v5.1.0/debug/oj-lux/',</strong>
        <strong style="color: #a11;">
        //LUX Utils
        'luxBindingBridge':            'libs/lux/v5.1.0/debug/luxutils/luxBindingBridge',</strong>
        'AbstractCCAWidget':           'libs/lux/v5.1.0/debug/luxutils/AbstractCCAWidget',
        'koWidget':                    'libs/lux/v5.1.0/debug/luxutils/koWidget',
        'luxtree':                     'libs/lux/v5.1.0/debug/luxutils/luxtree',
        'luxhierarchy':                'libs/lux/v5.1.0/debug/luxutils/luxhierarchy',
        'luxlogininfo':                'libs/lux/v5.1.0/debug/luxutils/luxlogininfo',
        'luxSubscriptionStoreFactory': 'libs/lux/v5.1.0/debug/luxutils/luxSubscriptionStoreFactory',
        'luxCustomBindings':           'libs/lux/v5.1.0/debug/luxutils/luxCustomBindings',
        'keyCode':                     'libs/lux/v5.1.0/debug/luxutils/keyCode',

        //LUX Patterns
        'luxbreadcrumbs':          'libs/lux/v5.1.0/debug/components/breadcrumbs/luxbreadcrumbs',
        'luxbreadcrumbs_template': 'libs/lux/v5.1.0/debug/components/breadcrumbs/luxbreadcrumbs_template.html',
        'luxsearchmodel':          'libs/lux/v5.1.0/debug/components/search/luxsearchmodel',
        'luxsearch':               'libs/lux/v5.1.0/debug/components/search/luxsearch',
        'luxsearchresults':        'libs/lux/v5.1.0/debug/components/search/luxsearchresults',
        'luxnavtree':              'libs/lux/v5.1.0/debug/components/navigation/luxnavtree',
        'luxshuttle':              'libs/lux/v5.1.0/debug/components/shuttle/luxshuttle',
        'luxsignin':               'libs/lux/v5.1.0/debug/components/signin/luxsignin',
        'luxpin':                  'libs/lux/v5.1.0/debug/components/pin/luxpin',
        'luxscrollposition':       'libs/lux/v5.1.0/debug/components/scrollposition/luxscrollposition',
        'luxautocomplete':         'libs/lux/v5.1.0/debug/components/autocomplete/luxautocomplete',
        'luxautocompleteinput':    'libs/lux/v5.1.0/debug/components/autocomplete/luxautocompleteinput',
        'luxdynamicform':          'libs/lux/v5.1.0/debug/components/dynamicforms/luxdynamicform',
        'luxdynamicformfields':    'libs/lux/v5.1.0/debug/components/dynamicforms/luxdynamicformfields',
        'luxdynamicformview':      'libs/lux/v5.1.0/debug/components/dynamicforms/luxdynamicformview',
        'luxformfieldref':         'libs/lux/v5.1.0/debug/components/dynamicforms/luxformfieldref',
        'luxDashboard':            'libs/lux/v5.1.0/debug/components/dashboard',
        'luxDashboardAdmin':       'libs/lux/v5.1.0/debug/components/dashboardAdmin',
        <strong style="color: #a11;">
        /* grid related */
        'gridapi.grid':            'libs/lux/v5.1.0/debug/components/grid/assets/grid',
        'gridapi.namespace':       'libs/lux/v5.1.0/debug/components/grid/assets/coreDefinitions/lux/widgetNamespace',
        'gridapi.textInput':       'libs/lux/v5.1.0/debug/components/grid/assets/coreDefinitions/lux/textInput',
        'gridapi.columnGroupMenu': 'libs/lux/v5.1.0/debug/components/grid/columnGroupMenu/lux',
        'ganttapi.gantt':          'libs/lux/v5.1.0/debug/components/grid/assets/gantt',

        'gridapi.columns':         'libs/lux/v5.1.0/debug/components/grid/columns/luxColumns',
        'gridapi.columnMenu':      'libs/lux/v5.1.0/debug/components/grid/columnMenu/lux',
        'gridapi.i18n':            'libs/lux/v5.1.0/debug/components/grid/i18n/lux',

        'gridapi.migrate':         'libs/jquery/migrate',
        'gridapi.mousewheel':      'libs/jquery/jquery.mousewheel',
        'gridapi.module':          'libs/lux/v5.1.0/debug/luxutils/module',
        'markup':                  'libs/lux/v5.1.0/debug/luxutils/markup',
        'gridapi.consoleLogger':   'libs/lux/v5.1.0/debug/luxutils/consoleLogger',
        'gridapi.jszip':           'libs/jszip/jszip',
        'timsort':                 'libs/timsort/timsort'</strong>
    },
    packages: [
        {
            name:     'luxDashboard',
            location: 'libs/lux/v5.1.0/debug/components/dashboard'
        },
        {
            name:     'luxDashboardAdmin',
            location: 'libs/lux/v5.1.0/debug/components/dashboardAdmin'
        },
        {
            name:     'luxToolbarGrid',
            location: 'libs/lux/v5.1.0/debug/components/toolbarGrid'
        }
    ],
    map: {
        '*': {<strong style="color: #a11;">
            'gridapi.jquery':                        'jquery',
            'gridapi.jqueryui-scroll-parent':        'jqueryui-amd/scroll-parent',
            'gridapi.jqueryui':                      'jqueryui-amd/widget',
            'gridapi.markup':                        'markup',
            'gridapi.logger':                        'gridapi.consoleLogger',
            'gridapi.text':                          'text',
            'gridapi.grid/keyCode':                  'keyCode',
            'gridapi.lodash':                        'lodash',
            'gridapi.grid/config/themeConfigurator': 'libs/require/empty',
            'gridapi.underscore':                    'underscore',
            'gridapi.moment':                        'moment',
            'ganttapi.moment':                       'moment',
            'gridapi.timsort':                       'timsort'</strong>
        }
    },
    // Shim configurations for modules that do not expose AMD
    shim: {
        'jquery': {
            exports: ['jquery', '$']
        },
        <strong style="color: #a11;">
        // lux related
        underscore: {
            exports: '_',
            init:    function() {
                'use strict';
                return this._.noConflict();
            }
        },
        </strong>
        // dynamic forms engine
        'dform': {
            deps: ['jquery']
        },
        'dform.core': {
            deps: ['jquery', 'dform']
        },
        'dform.converters': {
            deps: ['jquery', 'dform', 'dform.core']
        },
        'dform.extensions': {
            deps: ['jquery', 'dform', 'dform.core']
        },
        'knockout-mapping': {
            deps:    ['knockout'],
            exports: 'knockout-mapping'
        },
        <strong style="color: #a11;">
        // grid related
        'gridapi.mousewheel': {
            deps:    ['jquery'],
            exports: 'jQuery.fn.mousewheel'
        },
        'gridapi.migrate': {
            deps: ['jquery']
        }</strong>
    },
    // This section configures the i18n plugin. It is merging the Oracle JET built-in translation
    // resources with a custom translation file.
    // Any resource file added, must be placed under a directory named "nls". You can use a path mapping or you can define
    // a path that is relative to the location of this main.js file.
    config: {
        ojL10n: {
            merge: {
                'ojtranslations/nls/ojtranslations': 'resources/nls/menu'
            }
        }
    }
});
</pre>


## Example

Here is an example on how to configure a LUX grid basic component in your page:

```html
    <oj-lux-grid-basic
        width="100%"
        height="197px"
        selection-mode="check"
        editable=true
        columns="[[grid.columnDefinitions]]"
        get-column-icon-markup-callback="{{grid.getIconMarkup}}"
        data="[[dataSource]]"

        pagination.page-size="[[pageSize]]"

        toolbar.spec="[[grid.toolbarSpec]]" 
        toolbar.is-element-disabled-state-callback="{{grid.toolbarElementDisabledState}}"
        toolbar.should-include-element-callback="{{grid.toolbarShouldIncludeElement}}"

        on-lux-grid-toolbar-action-event={{grid.toolbarActionHandler}}
        on-lux-grid-cell-updated={{grid.gridCellUpdatedHandler}}
        on-lux-grid-link-clicked={{grid.gridLinkClickedHandler}}>
    </oj-lux-grid-basic>
```
For a complete example, please see [LUX Data Grid - Basic](http://lux.us.oracle.com:8080/lux-features/?root=home&home=patterns-gridbasic-basicccademo) demo.

## Notes
As LUX basic grid provides just a data grid with most common grid features that you can just configure and use, if you are looking to make use of other advanced features from our LUX general data grid, you should look into using the **`<lux-grid>`** component instead. However, with **`<lux-grid>`** component, you would need to know the LUX general data grid APIs in details, before you can configure a reasonable customized data grid for your own use.
