
/**
 * @module oj-lux-toolbar
 * @requires ojcore
 * @requires knockout
 */

/**
 * Copyright (c) 2016 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * lux-toolbar is a composite component. This component basically extends the JET ojToolbar and provides
 * a way to create the toolbar items declaratively.
 */

/* jshint ignore:start */
var obj = /** @lends module:oj-lux-toolbar */{
/**
         * Options
         * <dl>

<dt>id</dt>
<dd>
Specify the HTML element id for the toolbar component.
</dd>
<dt>top-border</dt>
<dd>
Optional boolean property to display the toolbar top border. Default is false, not to display the top border.
</dd>
<dt>bottom-border</dt>
<dd>
Optional boolean property to display the toolbar bottom border. Default is false, not to display the bottom border.
</dd>
<dt>items</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: Array of the toolbar items. Toolbar item can be one of the followings:
                    <ul>
                        <li><b>Separator</b>: object to specify a toolbar separator item
                            <ul>
                                <li><b>type</b>: 'separator'</li>
                            </ul>
                        </li>
                        <li><b>Push Button</b>: object to specify a toolbar push button item
                            <ul>
                                <li><b>component</b>: 'button'</li>
                                <li><b>type</b>: 'push'</li>
                                <li><b>id</b>: HTML element id for the button</li>
                                <li><b>event</b>: Button event handler</li>
                                <li><b>Support the following options from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojButton.html#members-section">ojButton</a>
                                </b>: chroming, disabled, display, icons, label, rootAttributes</li>
                            </ul>
                        </li>
                        <li><b>Toggle Button</b>: object to specify a toolbar toggle button item
                            <ul>
                                <li><b>component</b>: 'buttonset'</li>
                                <li><b>type</b>: 'checkbox'</li>
                                <li><b>id</b>: HTML element id for the button</li>
                                <li><b>checkbox</b>: Checkbox button object</li>
                                <li><b>Support the following options and event from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojButtonset.html#members-section">ojButtonset</a>
                                </b>: checked, chroming, rootAttributes, optionChange event</li>
                                <li><b>For checkbox button, support the following options from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojButton.html#members-section">ojButton</a>
                                </b>: disabled, display, icons, label</li>
                            </ul>
                        </li>
                        <li><b>Button Set - Single Option Selection</b>: object to specify a toolbar button set item that supports single option selection
                            <ul>
                                <li><b>component</b>: 'buttonset'</li>
                                <li><b>type</b>: 'radiogroup'</li>
                                <li><b>radios</b>: List of radio button objects</li>
                                <li><b>staticLabel</b>: Label to display before the button group</li>
                                <li><b>Support the following options and event from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojButtonset.html#members-section">ojButtonset</a>
                                </b>: checked, chroming, rootAttributes, optionChange event</li>
                                <li><b>For radio buttons, support the following options from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojButton.html#members-section">ojButton</a>
                                </b>: disabled, display, icons, label</li>
                            </ul>
                        </li>
                        <li><b>Button Set - Multiple Option Selections</b>: object to specify a toolbar button set item that supports multiple option selections
                            <ul>
                                <li><b>component</b>: 'buttonset'</li>
                                <li><b>type</b>: 'checkboxset'</li>
                                <li><b>checkboxes</b>: List of checkbox button objects</li>
                                <li><b>staticLabel</b>: Label to display before the button group</li>
                                <li><b>Support the following options and event from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojButtonset.html#members-section">ojButtonset</a>
                                </b>: checked, chroming, rootAttributes, optionChange event</li>
                                <li><b>For checkbox buttons, support the following options from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojButton.html#members-section">ojButton</a>
                                </b>: disabled, display, icons, label</li>
                            </ul>
                        </li>
                        <li><b>Input Text</b>: object to specify a toolbar input text field item
                            <ul>
                                <li><b>component</b>: 'input'</li>
                                <li><b>type</b>: 'text'</li>
                                <li><b>staticLabel</b>: String to display before the input text field</li>
                                <li><b>For input text, support the following attributes</b>: id, placeholder, visible</li>
                            </ul>
                        </li>
                        <li><b>Menu Button</b>: object to specify a toolbar menu button item
                            <ul>
                                <li><b>component</b>: 'button'</li>
                                <li><b>type</b>: 'menu'</li>
                                <li><b>id</b>: HTML element id for the button</li>
                                <li><b>Support the following options from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojButton.html#members-section">ojButton</a>
                                </b>: chroming, disabled, display, icons, label, menu, rootAttributes</li>
                            </ul>
                        </li>
                        <li><b>Menu</b>: object to specify a menu associated with a toolbar menu button
                            <ul>
                                <li><b>component</b>: 'menu'</li>
                                <li><b>id</b>: HTML element id for the menu</li>
                                <li><b>label</b>: Label for the menu</li>
                                <li><b>Support the following options and events from
                                <a href="http://jet.us.oracle.com/oj-components/jsdocs/oj.ojMenu.html#members-section">ojMenu</a>
                                </b>: disabled, select event</li>
                                <li><b>menuItems</b>: list of menu item objects. Menu item can be one of the followings:
                                    <ul>
                                        <li><b>type</b>: 'separator'</li>
                                        <li><b>type</b>: 'subMenu', 'item'
                                            <ul>
                                                <li><b>id</b>: HTML element id for the element</li>
                                                <li><b>label</b>: Label for the element</li>
                                                <li><b>disabled</b>: disabled flag</li>
                                                <li><b>icon</b>: icon to display before the label</li>
                                                <li><b>href</b>: href link</li>
                                            </ul>
                                        </li>
                                    </ul>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
                <li><i>Example</i> : <br/><br/>In the following example, toolbarItems specifies a list of toolbar items that would be rendered in a toolbar component, similar to the
                <a href="http://jet.us.oracle.com/5.1.0/jetCookbook.html?component=toolbars&demo=toolbar">JET Single Toolbar</a>.

<pre class="prettyprint" >
define([
     'ojs/ojcore',
     'knockout',
     'jquery',
     'ojs/ojknockout',
     'ojs/ojcomposite',
     'lux-composites/toolbar/loader',
     'ojs/ojswitch'
 ], function(oj, ko, $) {
     'use strict';

    // view model for toolbar demo
    var ViewModel = function() {

        var self = this;

        self.elementId = 'my-toolbar';
        self.showTopBorder = true;
        self.showBottomBorder = true;

        self.buttonClick = function(data, event) {
            console.log("push button: "+event.currentTarget.id);
            return (true);
        };

        // observable array bound to the button set
        self.isButtonSet = ko.observableArray([]);
        // 2-way binding
        self.userText = ko.computed(function() {
            return ("button set count: " + self.isButtonSet().length);
        }, self);
        self.buttonSetOptionChange = function(event, ui) {
            if (ui.option === "checked") {
                console.log("button set: " + ui.value);
            }
        };

        // observable array bound to the toggle button
        self.toggle = ko.observableArray([]);
        self.toggleOptionChange = function(event, ui) {
            if (ui.option === "checked") {
                console.log("toggle button: " + ui.value);
            }
        };

        // observable bound to the radio Buttonset
        self.drink = ko.observable("tea");
        self.radioSetOptionChange = function(event, ui) {
            if (ui.option === "checked") {
                console.log("radio button: " + ui.value);
            }
        };

        self.selectedMenuItem = ko.observable("None");
        self.menuItemSelect = function(event, ui) {
            self.selectedMenuItem(ui.item.children("a").text());
        };

        this.disableBookmarkId = 'bookmark';
        self.disableBookmark = ko.observable(true);

        self.toolbarItems = [
            {
                component: 'button', type: 'push', id: 'chat', label: 'Chat',
                event: {
                    click: self.buttonClick
                },
                icons: {
                    start: 'demo-icon-font demo-chat-icon-24'
                }
            },
            {
                component: 'button', type: 'push', id: 'paint', label: 'Paint',
                event: {
                    click: self.buttonClick
                },
                icons: {
                    start: 'demo-icon-font demo-palette-icon-24'
                }
            },
            {
                component: 'button', type: 'push', id: 'bookmark', label: 'Bookmark',
                event: {
                    click: self.buttonClick
                },
                disabled: self.disableBookmark,
                icons: {
                    start: 'demo-icon-font demo-bookmark-icon-16'
                }
            },
            { type: 'separator' },
            {
                component: 'buttonset', type: 'checkbox', id: 'toggleWrapper',
                checked: self.toggle,
                optionChange: self.toggleOptionChange,
                checkbox: {
                    id: 'check', value: 'check', label: 'Toggle'
                }
            },
            {
                component: 'button', type: 'menu', id: 'buttonMenu', label: 'Menu Button', menu: "#toolbarMenu"
            },
            {
                component: 'menu', id: 'toolbarMenu', label: 'Toolbar Menu',
                select: self.menuItemSelect,
                menuItems: [
                    {
                        type: 'item', id: 'Save', label: 'Save',
                        href: '#',
                        icon: 'oj-fwk-icon oj-fwk-icon-arrow-n'
                    },
                    {
                        type: 'subMenu', id: 'Zoom', label: 'Zoom',
                        href: '#',
                        menuItems: [
                            {
                                type: 'item', id: 'zoomIn', label: 'Zoom In',
                                href: '#',
                                icon: 'oj-fwk-icon oj-fwk-icon-arrow-n'
                            },
                            {
                                type: 'item', id: 'zoomOut', label: 'Zoom Out',
                                href: '#',
                                icon: 'demo-icon-font demo-bookmark-icon-16'
                            }
                        ]
                    },
                    { type: 'separator'},
                    {
                        type: 'item', id: 'Print', label: 'Print',
                        href: '#',
                        icon: 'demo-icon-font demo-chat-icon-24'
                    }
                ]
            },
            { type: 'separator' },
            {
                component: 'buttonset', type: 'radiogroup', id: 'radioWrapper',
                checked: self.drink,
                optionChange: self.radioSetOptionChange,
                radios: [
                    {id: 'coffee', name: 'beverage', label: 'Coffee'},
                    {id: 'tea', name: 'beverage', label: 'Tea'},
                    {id: 'milk', name: 'beverage', label: 'Milk'}
                ],
            },
            { type: 'separator' },
            {
                component: 'buttonset', type: 'checkboxset', id: 'checkboxSet',
                checked: self.isButtonSet,
                optionChange: self.buttonSetOptionChange,
                checkboxes: [
                    {id: 'library', label: 'Library', icons: { start: 'demo-icon-font demo-library-icon-24' }},
                    {id: 'home', label: 'Home', icons: { start: 'demo-icon-font demo-home-icon-24' }},
                    {id: 'grid', label: 'Grid', icons: { start: 'demo-icon-font demo-grid-icon-16' }}
                ],
            }
        ];

    };
    return ViewModel;
});
</pre>

            </ul>
        </div>
    </div>
</dd>

* </dl>
*/
    options: {
    }
};
/* jshint ignore:end */
