/**
 * Copyright (c) 2018 Oracle and/or its affiliates. All rights reserved.
 * @author Daniel Hoang
 */
define([
    'ojs/ojcore',
    'knockout',
    'jquery',
    'lodash'
],
function(oj, ko, $, _) {
    'use strict';

    var ViewModel = function(context) {
        var _busyContext, _setupComplete, _uniqueNumber, _createUniqueId;

        var self = this;

        self.toolbarClassNames = ko.observableArray([]);

        self.toolbarClasses = ko.computed(function() {
            return this.toolbarClassNames().join(' ');
        }, this);

        // Create a debounce function to call on ojToolbar refresh().
        // Per ojToolbar, a refresh() is required in the following circumstances:
        // - After buttons or buttonsets are added to, removed from, or reordered within the toolbar.
        // * After a change to the disabled status of any of the buttons in the toolbar.
        // * After a programmatic change to the checked status of a radio button in the toolbar (which should be done
        //   via Buttonset's checked option). This applies only to radios, not to checkboxes or push buttons.
        // - After the reading direction (LTR vs. RTL) changes.
        self._refreshToolbar = _.debounce(function() {
            // Lazy loading toolbar element
            if (!self.toolbarElement) {
                var toolbarElement = self._getToolbarElement();
                if (toolbarElement) {
                    self.toolbarElement = $(toolbarElement);
                }
            }
            if (self.toolbarElement) {
                // TODO: once we move from ojToolbar to oj-toolbar, we can access toolbar element directly (and no jquery)
                self.toolbarElement.ojToolbar('refresh');
            }
        }, 500);

        // Set up busy context
        _busyContext = oj.Context.getContext(context.element).getBusyContext();
        _setupComplete = _busyContext.addBusyState({'descriptions': 'oj-lux-toolbar startup processing'});

        // Extract unique number from the unique string. This is used to generate component's element IDs.
        _uniqueNumber = parseInt(context.unique.replace(/[^0-9\.]/g, ''), 10);

        // internal method to generate element ID from a given prefix string
        _createUniqueId = function(idPrefix) {
            return (idPrefix + _uniqueNumber);
        };

        // Save this composite component's element.
        self.composite = context.element;

        // Create unique element IDs
        self.elementId = _createUniqueId('lux-toolbar-id-');

        if (context.properties.topBorder) {
            self.toolbarClassNames.push('oj-toolbar-top-border');
        }
        if (context.properties.bottomBorder) {
            self.toolbarClassNames.push('oj-toolbar-bottom-border');
        }

        // signal that all the setup work is done
        _setupComplete();
    };

    // expose toolbar element for host element to use (e.g. toolbarGrid widget)
    ViewModel.prototype._getToolbarElement = function() {
        return (this.composite ? this.composite.querySelector('#' + this.elementId) : undefined);
    };

    ViewModel.prototype._refreshToolbarElement = function() {
        this._refreshToolbar();
    };

    return ViewModel;
});

